package cn.fw.morax.common.utils;

import cn.fw.morax.common.annotation.ExcelField;
import cn.fw.morax.common.pojo.ExcelFieldData;
import cn.hutool.poi.excel.ExcelWriter;
import org.apache.poi.ss.usermodel.*;

import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author kurisu
 * @description excel
 * @date 2019/5/30.
 */
public class ExcelDataUtil {


    /**
     * 获取中文名与属性名
     *
     * @param source
     * @return
     */
    public static Map<String, String> getTitleMappingField(Class<?> source) {
        List<ExcelFieldData> headerFields = getHeaderFields(source);
        return headerFields.stream().collect(Collectors.toMap(ExcelFieldData::getTitle, ExcelFieldData::getName, (v1, v2) -> v2, LinkedHashMap::new));
    }

    /**
     * 获取排序后的标题
     *
     * @param source
     * @return
     */
    public static List<ExcelFieldData> getHeaderFields(Class<?> source) {
        //获取字段对象
        Field[] fields = source.getDeclaredFields();
        if (fields.length == 0) {
            return new ArrayList<>();
        }
        List<ExcelFieldData> excelFieldDatas = new ArrayList<>();
        Arrays.stream(fields).forEach(field -> {
            field.setAccessible(true);
            //获取字段是否有导出标识注解
            ExcelField excelField = field.getAnnotation(ExcelField.class);
            if (PublicUtil.isNotEmpty(excelField)) {
                excelFieldDatas.add(ExcelFieldData.builder()
                        .name(field.getName())
                        .title(excelField.title())
                        .charSize(excelField.charSize())
                        .sort(excelField.sort())
                        .build());
            }
        });
        excelFieldDatas.sort(Comparator.comparing(ExcelFieldData::getSort));
        return excelFieldDatas;
    }


    /**
     * 获取表头信息
     *
     * @param writer
     * @param source
     * @return 列数量
     */
    public static Map<String, String> setExcelHeadAndWidth(ExcelWriter writer, Class<?> source, Boolean autoWidth) {
        if (PublicUtil.isEmpty(source)) {
            return new HashMap<>();
        }
        List<ExcelFieldData> headerFields = ExcelDataUtil.getHeaderFields(source);
        for (ExcelFieldData excelField : headerFields) {
            writer.addHeaderAlias(excelField.getTitle(), excelField.getName());

            CellStyle cellStyle = writer.createCellStyle(excelField.getSort(), 0);
            // 内容水平居中
            cellStyle.setAlignment(HorizontalAlignment.CENTER);
            // 内容垂直居中
            cellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
            Font font = writer.createFont();
            font.setFontHeight((short) 20);
            cellStyle.setFont(font);

            if (! autoWidth) {
                if (PublicUtil.isEmpty(excelField.getCharSize()) || excelField.getCharSize().equals(0)) {
                    continue;
                }
                if (excelField.getCharSize().compareTo(255) > 0) {
                    writer.setColumnWidth(excelField.getSort(), 255);
                } else {
                    writer.setColumnWidth(excelField.getSort(), excelField.getCharSize());
                }
            }

        }

        return headerFields.stream().collect(Collectors.toMap(ExcelFieldData::getName, ExcelFieldData::getTitle, (v1, v2) -> v2, LinkedHashMap::new));
    }

    /**
     * 自适应宽度(中文支持)
     * @param sheet
     */
    public static void setAutoSizeColumn(Sheet sheet) {
        Integer size = sheet.getLastRowNum();
        for (int columnNum = 0; columnNum <= size; columnNum++) {
            int columnWidth = sheet.getColumnWidth(columnNum) / 256;
            for (int rowNum = 0; rowNum <= sheet.getLastRowNum(); rowNum++) {
                Row currentRow;
                //当前行未被使用过
                if (sheet.getRow(rowNum) == null) {
                    currentRow = sheet.createRow(rowNum);
                } else {
                    currentRow = sheet.getRow(rowNum);
                }

                if (currentRow.getCell(columnNum) != null) {
                    Cell currentCell = currentRow.getCell(columnNum);
                    if (currentCell.getCellType() == CellType.STRING) {
                        int length = currentCell.getStringCellValue().getBytes().length;
                        if (columnWidth < length) {
                            columnWidth = length;
                        }
                    }
                }
            }
            sheet.setColumnWidth(columnNum, columnWidth * 256);
        }
    }

}
