package cn.fw.morax.rpc.erp;

import cn.fw.data.base.domain.common.Message;
import cn.fw.ehr.sdk.api.ManagerApi;
import cn.fw.ehr.sdk.api.PostApi;
import cn.fw.ehr.sdk.api.StaffApi;
import cn.fw.ehr.sdk.api.result.*;
import cn.fw.erp.sdk.api.RoleApi;
import cn.fw.erp.sdk.api.UserRoleApi;
import cn.fw.erp.sdk.api.enums.AuthStatusEnum;
import cn.fw.erp.sdk.api.enums.RoleTypeEnum;
import cn.fw.erp.sdk.api.enums.RoleUseTypeEnum;
import cn.fw.erp.sdk.api.result.RoleInfo;
import cn.fw.erp.sdk.api.result.UserRoleDataRange;
import cn.fw.erp.sdk.api.result.UserRoleInfo;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.StringUtils;
import cn.fw.morax.rpc.ehr.dto.PerformanceStaffDTO;
import cn.fw.morax.rpc.ehr.dto.PostInfoDTO;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.morax.rpc.erp.dto.RoleInfoDTO;
import cn.fw.morax.rpc.erp.dto.RpcUserRoleInfoDTO;
import cn.fw.morax.rpc.erp.dto.RpcUserRoleShopDTO;
import com.alibaba.fastjson.JSON;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.cache.CacheKey;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @className : EhrRpcService
 * @description : 人事
 * @date: 2021-01-19 17:34
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class ErpRpcService {
    private final UserRoleApi userRoleApi;
    private final RoleApi roleApi;

    /**
     * 获取角色对应员工
     *
     * @param roleCode
     * @param shopId
     * @return
     */
    @Cacheable(cacheNames = "rpc_erp", key = "':rpc:erp:shop:roleCode:'+ #roleCode", unless = "#result == null")
    @Nullable
    public List<RpcUserRoleInfoDTO> getUserRoleInfo(String roleCode, Long shopId) {
        if (PublicUtil.isEmpty(roleCode)) {
            return new ArrayList<>();
        }
        try {
            Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleInfos(roleCode, shopId);
            log.info("获取角色编码和门店/商家对应员工信息 userRoleApi.queryUserRoleInfos 传入参数：roleCode:{}, value:{} 返回结果：{}", roleCode, shopId, JSON.toJSON(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                return PublicUtil.copyList(msg.getData(), RpcUserRoleInfoDTO.class);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    /**
     * 获取角色对应员工
     *
     * @param roleCode
     * @param shopIds
     * @return
     */
    public List<RpcUserRoleInfoDTO> getUserEnableRoleInfos(String roleCode, List<Long> shopIds) {
        if (PublicUtil.isEmpty(roleCode)) {
            return new ArrayList<>();
        }
        try {
            Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleInfoList(roleCode, shopIds);
            log.info("获取角色编码和门店/商家对应员工信息 userRoleApi.queryUserRoleInfoList 传入参数：roleCode:{}, value:{} 返回结果：{}", roleCode, JSON.toJSON(shopIds), JSON.toJSON(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                List<RpcUserRoleInfoDTO> result = Lists.newArrayListWithCapacity(msg.getData().size());
                for (UserRoleInfo userRole : msg.getData()) {
                    RpcUserRoleInfoDTO rpcUserRoleInfoDTO = PublicUtil.copy(userRole, RpcUserRoleInfoDTO.class);
                    if (PublicUtil.isEmpty(userRole.getDataRangeList())) {
                        continue;
                    }
                    List<RpcUserRoleShopDTO> ranges = Lists.newArrayListWithCapacity(userRole.getDataRangeList().size());
                    for (UserRoleDataRange range : userRole.getDataRangeList()) {
                        if (AuthStatusEnum.DISABLED.getValue().equals(range.getStatus())) {
                            continue;
                        }
                        RpcUserRoleShopDTO shopDTO = new RpcUserRoleShopDTO(range.getRangeValue(), range.getRangeName());
                        ranges.add(shopDTO);
                    }
                    rpcUserRoleInfoDTO.setRangeList(ranges);
                    result.add(rpcUserRoleInfoDTO);
                }
                return result;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    /**
     * 获取角色对应员工
     *
     * @param userId
     * @param roleCode
     * @return
     */
    @Cacheable(cacheNames = "rpc_erp", key = "':rpc:erp:user:manager:code:'+ #roleCode", unless = "#result == null")
    @Nullable
    public List<Long> queryUserRoleShopIds(Long userId, String roleCode) {
        if (PublicUtil.isEmpty(roleCode)) {
            return new ArrayList<>();
        }
        try {
            Message<List<UserRoleDataRange>> msg = userRoleApi.queryUserRoleAllRange(userId, roleCode);
            log.info("查询数据范围内的角色用户信息 userRoleApi.queryUserRoleAllRange 传入参数：userId:{}, roleCode:{} 返回结果：{}", userId, roleCode, JSON.toJSON(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                return msg.getData().stream()
                        .map(UserRoleDataRange::getRangeValue)
                        .collect(Collectors.toList());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    /**
     * 查询门店下有角色的人员
     *
     * @param roleCodes
     * @param shopId
     * @return
     */
    @Nullable
    public List<RpcUserRoleInfoDTO> queryUserRoleShopIds(List<String> roleCodes, Long shopId) {
        if (PublicUtil.isEmpty(roleCodes)) {
            return new ArrayList<>();
        }
        try {
            Message<List<UserRoleInfo>> msg = userRoleApi.queryRoleUsers(roleCodes, shopId);
            log.info("查询门店下有角色的人员 userRoleApi.queryRoleUsers 传入参数：roleCodes:{}, shopId:{} 返回结果：{}", JSON.toJSON(roleCodes), shopId, JSON.toJSON(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                    return PublicUtil.copyList(msg.getData(), RpcUserRoleInfoDTO.class);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    /**
     * 查询人员是否拥有某个角色
     *
     * @param userId
     * @param roleCode
     * @return
     */
    public boolean hasRole(Long userId, String roleCode) {
        try {
            Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleList(userId);
            log.info("获取人员角色信息 userRoleApi.queryUserRoleList 传入参数：userId:{} 返回结果：{}", userId, JSON.toJSONString(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                return Optional.ofNullable(msg.getData())
                        .orElse(Lists.newArrayList())
                        .stream()
                        .anyMatch(userRole -> userRole.getRoleCode().equals(roleCode));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return Boolean.FALSE;
    }

    /**
     * 查询人员是否拥有某个角色
     *
     * @param userId
     * @return
     */
    public List<Long> manageShopIds(Long userId) {
        try {
            Message<List<UserRoleInfo>> msg = userRoleApi.queryUserRoleDetailList(userId);
            log.info("获取人员所有角色信息 userRoleApi.queryUserRoleDetailList 传入参数：userId:{} 返回结果：{}", userId, JSON.toJSONString(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                Integer manageRoleType = RoleUseTypeEnum.MANAGE.getValue();
                Integer enableStatus = AuthStatusEnum.ENABLED.getValue();

                List<Long> manageShopIds = msg.getData().stream()
                            .filter(userRole -> manageRoleType.equals(userRole.getUseType()))
                            .flatMap(userRole -> userRole.getDataRangeList().stream())
                            .filter(range -> enableStatus.equals(range.getStatus()))
                            .map(UserRoleDataRange::getRangeValue).distinct()
                            .collect(Collectors.toList());
                return manageShopIds;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    /**
     * 查询所有管理角色
     *
     * @param
     * @return
     */
    @Cacheable(cacheNames = "rpc_erp", key = "':rpc:erp:manager:code'", unless = "#result == null")
    @Nullable
    public List<RoleInfoDTO> manageRoles() {
        try {
            Message<List<RoleInfo>> msg = roleApi.queryRoleInfoList(null, RoleUseTypeEnum.MANAGE.getValue());
            log.info("获取管理角色信息 roleApi.queryRoleInfoList 传入参数：useType:{} 返回结果：{}", RoleUseTypeEnum.MANAGE.getValue(), JSON.toJSONString(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                return PublicUtil.copyList(msg.getData(), RoleInfoDTO.class);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new ArrayList<>();
    }

    /**
     * 查询所有管理角色
     *
     * @param
     * @return
     */
    @Cacheable(cacheNames = "rpc_erp", key = "':rpc:erp:role:code:'+ #roleCode", unless = "#result == null")
    @Nullable
    public RoleInfoDTO queryRole(String roleCode) {
        try {
            Message<RoleInfo> msg = roleApi.queryRoleInfo(roleCode);
            log.info("获取管理角色信息 roleApi.queryRoleInfo 传入参数：roleCode:{} 返回结果：{}", roleCode, JSON.toJSONString(msg));
            if (msg.isSuccess() && !Objects.isNull(msg.getData())) {
                return PublicUtil.copy(msg.getData(), RoleInfoDTO.class);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

}
