package cn.fw.morax.server.consumer;


import cn.fw.ehr.sdk.api.mq.StaffPostChangeEvent;
import cn.fw.erp.sdk.api.enums.RoleUseTypeEnum;
import cn.fw.morax.common.utils.MessageFormatUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PostInfoDTO;
import cn.fw.morax.rpc.erp.ErpRpcService;
import cn.fw.morax.rpc.erp.dto.RoleInfoDTO;
import cn.fw.morax.service.biz.kpi.KpiGroupUserBizService;
import cn.fw.morax.service.biz.kpi.KpiPoolBizService;
import cn.fw.morax.service.data.kpi.KpiGroupService;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.spring.annotation.RocketMQMessageListener;
import org.apache.rocketmq.spring.core.RocketMQListener;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author : jiangchao
 * Date: 2022/05/07
 * Time: 17:34
 * Description: 人事调岗
 */
@Slf4j
@Service
@RocketMQMessageListener(topic = StaffPostChangeEvent.TOPIC, consumerGroup = "${spring.application.name}_" + StaffPostChangeEvent.TOPIC)
@RequiredArgsConstructor
public class EhrStaffPostConsumer implements RocketMQListener<StaffPostChangeEvent> {

    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final KpiPoolBizService kpiPoolBizService;
    private final KpiGroupService kpiGroupService;
    private final EhrRpcService ehrRpcService;
    private final ErpRpcService erpRpcService;

    /***
     * 人员调岗完成事件
     * 原岗位基础角色 普通角色、新岗位基础角色 管理角色 ——> 触发保护期
     * @param result
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void onMessage(StaffPostChangeEvent result) {
        log.info("人事系统人员调岗MQ消息：{}", JSONObject.toJSONString(result));
        //同一个门店 不触发保护期
        if (result.getPostId().equals(result.getNewPostId())) {
            return;
        }
        Long oldPostId = result.getPostId();
        Long newPostId = result.getNewPostId();
        List<Long> postIds = new ArrayList<Long>(){{add(oldPostId);add(newPostId);}};
        List<PostInfoDTO> postInfos = ehrRpcService.getPostInfoList(postIds);
        Map<Long, String> postRoleCodeMap = postInfos.stream().collect(Collectors.toMap(PostInfoDTO::getId, PostInfoDTO::getRoleCode, (v1, v2) -> v1));
        String oldPostRoleCode = postRoleCodeMap.get(oldPostId);
        String newPostRoleCode = postRoleCodeMap.get(newPostId);

        RoleInfoDTO oldRoleInfo = erpRpcService.queryRole(oldPostRoleCode);
        //管理角色没有保护期
        if (PublicUtil.isEmpty(oldRoleInfo) || RoleUseTypeEnum.MANAGE.getValue().equals(oldRoleInfo.getUseType())) {
            log.info("原岗位是管理角色没有保护期，{}，人员调岗：{}", JSON.toJSONString(oldRoleInfo), JSON.toJSONString(result));
            return;
        }

        RoleInfoDTO newRoleInfo = erpRpcService.queryRole(newPostRoleCode);
        if (PublicUtil.isEmpty(newRoleInfo) || (! RoleUseTypeEnum.MANAGE.getValue().equals(newRoleInfo.getUseType()))) {
            log.info("新岗位不是管理角色没有保护期，{}，人员调岗：{},", JSON.toJSONString(oldRoleInfo), JSON.toJSONString(result));
            return;
        }
        log.info("人员调岗有保护期：{}, 原岗位:{}，新岗位：{}", JSON.toJSONString(result), JSON.toJSONString(oldRoleInfo), JSON.toJSONString(newRoleInfo));
        //旧岗位基础角色是  普通角色， 新岗位基础角色是  管理角色    触发保护期
        KpiGroup originKpiGroup = kpiGroupService.getOne(Wrappers.<KpiGroup>lambdaQuery()
                .in(KpiGroup::getPostId, result.getPostId())
                .in(KpiGroup::getStatus, SettingStatusEnum.EFFECTIVE)
                .apply(MessageFormatUtil.transferWithQuotationMarks("FIND_IN_SET({0}, shop_ids)", result.getShopId()))
        );
        Long userId = result.getStaffId();
        //开始计算绩效时间
        LocalDate startKpiDate = kpiGroupUserBizService.calcStartKpiDate(userId);
        //失效当前所有绩效池数据
        kpiPoolBizService.invalidKpiPool(originKpiGroup, userId, startKpiDate);
    }
}


