package cn.fw.morax.server.controller.app;


import cn.fw.common.exception.BusinessException;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.eval.EvalGroupRank;
import cn.fw.morax.domain.dto.eval.EvalDTO;
import cn.fw.morax.domain.dto.eval.EvalGroupRankDTO;
import cn.fw.morax.domain.dto.eval.EvalGroupRankQueryDto;
import cn.fw.morax.domain.dto.eval.EvalRewardDistDTO;
import cn.fw.morax.domain.vo.eval.*;
import cn.fw.morax.service.biz.SettingDraftBizService;
import cn.fw.morax.service.biz.eval.*;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 考评
 *
 * @author jiangchao
 * @since 2022-11-09
 */
@Slf4j
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@Validated
@IgnoreAuth
@RestController
@RequestMapping("/app/eval")
public class EvalController {

    private final EvalRewardService evalRewardService;
    private final EvalGroupBizService evalGroupBizService;
    private final SettingDraftBizService settingDraftBizService;
    private final EvalGroupRankBizService evalGroupRankBizService;
    private final EvalIndicatorValueBizService evalIndicatorValueBizService;

    /**
     * 考评草稿详情
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/eval-detail")
    @ControllerMethod("考评草稿详情")
    public Message<EvalDTO> draftEvalDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftEvalDetail(id));
    }

    /**
     * 考评数据录入详情
     *
     * @param id
     * @return
     */
    @GetMapping("/report-detail")
    @ControllerMethod("考评数据录入详情")
    public Message<EvalReportVO> reportDetail(@NotNull(message = "录入详情id不能为空") Long id) {

        return success(evalIndicatorValueBizService.reportDetail(id));
    }

    /**
     * 奖惩分配详情
     *
     */
    @GetMapping("/reward-dist")
    @ControllerMethod("奖惩分配详情")
    public Message<EvalRewardDistVO> rewardDistDetail(@NotNull(message = "奖惩分配id不能为空") Long id) {
        return success(evalRewardService.rewardDistDetail(id));
    }

    /**
     * 奖惩分配
     *
     */
    @PostMapping("/reward-dist")
    @ControllerMethod("奖惩分配")
    public Message<Void> rewardDist(@Validated @RequestBody EvalRewardDistDTO rewardDist,
                                    @CurrentUser LoginAuthBean currentUser) {
        evalRewardService.rewardDist(rewardDist, currentUser);
        return success();
    }

    /**
     * 考评排名组详情
     *
     * @return
     */
    @GetMapping("/rank-group")
    @ControllerMethod("考评排名组详情")
    public Message<EvalGroupRankVO> evalGroupRankDetail(@NotNull(message = "考评排名组id不能为空") Long id) {
        return success(evalGroupBizService.evalGroupRankDetail(id));
    }

    /**
     * 考评排名组草稿详情（审批）
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/eval-rank-detail")
    @ControllerMethod("考评排名组草稿详情（审批）")
    public Message<EvalGroupRankDTO> draftEvalGroupRankDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftEvalGroupRankDetail(id));
    }

    /**
     * 考评排名组详情
     *
     * @return
     */
    @GetMapping("/rank-groups")
    @ControllerMethod("考评排名组详情")
    public Message<List<EvalGroupVO>> rankEvalGroupDetail(@Validated EvalGroupRankQueryDto dto) {
        return success(evalGroupBizService.rankEvalGroupDetail(dto));
    }

    /**
     * 考评人员排名
     *
     * @param dto
     * @return
     */
    @GetMapping("/staff-rank")
    @ControllerMethod("考评人员排名")
    public Message<List<EvalUserPoolVO>> staffRank(@Validated EvalGroupRankQueryDto dto) {
        if (PublicUtil.isEmpty(dto.getMonthly())) {
            throw new BusinessException("参数错误");
        }
        return success(evalGroupRankBizService.staffRank(dto));
    }

    /**
     * 考评门店排名
     *
     * @param dto
     * @return
     */
    @GetMapping("/shop-rank")
    @ControllerMethod("考评门店排名")
    public Message<List<EvalShopPoolVO>> shopRank(@Validated EvalGroupRankQueryDto dto) {
        if (PublicUtil.isEmpty(dto.getMonthly())) {
            throw new BusinessException("参数错误");
        }
        return success(evalGroupRankBizService.shopRank(dto));
    }

}
