package cn.fw.morax.server.controller.app;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.kpi.KpiGroupDTO;
import cn.fw.morax.domain.dto.kpi.KpiGroupIndicatorRankDTO;
import cn.fw.morax.domain.dto.kpi.KpiGroupRankConfigQueryDTO;
import cn.fw.morax.domain.dto.kpi.KpiGroupRankDTO;
import cn.fw.morax.domain.dto.query.*;
import cn.fw.morax.domain.enums.KpiAssessTypeEnum;
import cn.fw.morax.domain.vo.SelectorVO;
import cn.fw.morax.domain.vo.kpi.*;
import cn.fw.morax.service.biz.SettingDraftBizService;
import cn.fw.morax.service.biz.kpi.*;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.annotation.IgnoreUserToken;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.jni.Local;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 绩效池查询
 *
 * @author jiangchao
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/kpi")
@IgnoreAuth
public class KpiController {

    private final KpiStarSpecialRuleBizService kpiStarSpecialRuleBizService;
    private final KpiGroupIndicatorBizService kpiGroupIndicatorBizService;
    private final KpiGroupUserBizService kpiGroupUserBizService;
    private final KpiPoolBizService kpiPoolBizService;
    private final KpiGroupBizService kpiGroupBizService;
    private final SettingDraftBizService settingDraftBizService;
    private final KpiAssessBizService kpiAssessBizService;

    /**
     * 绩效池列表
     *
     * @param dto
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/staffs/pool")
    @ControllerMethod("绩效池列表")
    public Message<AppPage<KpiPoolVO>> queryKpiPool(@Validated KpiPoolQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getMonthly())) {
            dto.setMonthly(YearMonth.now());
        }
        return success(kpiGroupUserBizService.queryKpiPool(dto));
    }

    /**
     * 绩效池详情（查看他人绩效使用）
     *
     * @param kpiPoolId 绩效池id
     * @param yearMonth 月份
     * @return
     */
    @GetMapping("/pool/detail")
    @ControllerMethod("绩效池详情")
    public Message<KpiPoolDetailVO> queryKpiPoolDetail(@NotNull(message = "绩效池id不能为空") Long kpiPoolId,
                                                       @RequestParam(required = false) YearMonth yearMonth) {
        return success(kpiPoolBizService.queryPoolDetail(kpiPoolId, yearMonth));
    }

    /**
     * 绩效星级特殊规则详情
     *
     * @param KpiStarSpecialRuleId  绩效星级特殊规则id
     * @return
     */
    @GetMapping("/star-adjustment/detail")
    @ControllerMethod("绩效星级特殊规则详情")
    public Message<KpiStarSpecialRuleVO> getKpiStarSpecialRuleDetail(@NotNull(message = "绩效星级特殊规则id不能为空") Long KpiStarSpecialRuleId) {
        return success(kpiStarSpecialRuleBizService.getKpiStarSpecialRuleDetail(KpiStarSpecialRuleId));
    }

    /**
     * 绩效详情查询 (查看自己绩效详情)
     *
     * @param userId    用户id
     * @param yearMonth 月度
     * @return 绩效池详情
     * @ignoreParams userId
     */
    @GetMapping("/pool/own-detail")
    @ControllerMethod("绩效详情查询")
    public Message<KpiPoolDetailVO> kpiPoolOwnDetail(@CurrentUser Long userId,
                                                     @RequestParam(required = false) YearMonth yearMonth) {
        if (Objects.isNull(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        return success(kpiPoolBizService.queryUserPoolDetail(userId, yearMonth));
    }

    /**
     * 本月可查看的其他绩效池选项
     *
     * @param user      用户id
     * @param userId    用户id [管理层查看人员薪酬详情时需传值]
     * @param yearMonth 月度 [员工人查看自己的酬详情时需传值]
     * @return 薪酬池选项
     * @ignoreParams user
     */
    @GetMapping("/pool/selector")
    @IgnoreUserToken
    @ControllerMethod("本月可查看的其他绩效池选项")
    public Message<Set<SelectorVO>> salaryPoolSelector(@CurrentUser LoginAuthBean user, Long userId, @RequestParam(required = false) YearMonth yearMonth) {
        if (Objects.isNull(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        if (Objects.isNull(userId)) {
            userId = user.getUserId();
        }
        return success(kpiGroupUserBizService.queryUserKpiPool(userId, yearMonth));
    }

    /**
     * 员工绩效得分详情
     *
     * @param kpiPoolId 绩效池id
     * @param kpiGroupIndicatorId   绩效组指标id
     * @param yearMonth     月份
     * @return
     */
    @GetMapping("/kpi-indicator/score-detail")
    @IgnoreAuth
    @ControllerMethod("员工绩效得分详情")
    public Message<KpiGroupIndicatorDetailVO> getHitIndicatorLadder(Long kpiPoolId,
                                                                           Long kpiGroupIndicatorId,
                                                                           @RequestParam(required = false) YearMonth yearMonth) {
        if (PublicUtil.isEmpty(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        return success(kpiGroupIndicatorBizService.getHitIndicatorLadder(kpiPoolId, kpiGroupIndicatorId, yearMonth));
    }


    /**
     * 查询绩效组不考核人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/kpi-ignored/staffs")
    @IgnoreAuth
    @ControllerMethod("查询绩效组不考核人员")
    public Message<AppPage<KpiGroupUserVO>> queryKpiIgnoredStaffs(@Validated KpiGroupUserPageDTO dto) {
        return success(kpiGroupUserBizService.queryKpiIgnoredStaffs(dto));
    }

    /**
     * 绩效池排名列表
     *
     * @param dto
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/staffs/pool-rank")
    @ControllerMethod("绩效池排名列表")
    public Message<AppPage<KpiPoolVO>> queryKpiPoolRank(@Validated KpiPoolRankQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getYearMonth())) {
            dto.setYearMonth(YearMonth.now());
        }
        if (PublicUtil.isEmpty(dto.getKpiGroupId()) && PublicUtil.isEmpty(dto.getKpiGroupRankId())) {
            throw new BusinessException("参数错误");
        }
        return success(kpiGroupUserBizService.queryKpiPoolRank(dto));
    }

    /**
     * 查询绩效星级配置
     *
     * @param kpiGroupId
     * @return
     * @ignoreParams currentUser
     */
    @GetMapping("/group/star-config")
    @ControllerMethod("查询绩效星级配置")
    public Message<List<KpiStarLaddersVO>> queryKpiGroup(Long kpiGroupId) {
        return success(kpiGroupUserBizService.queryKpiGroup(kpiGroupId));
    }

    /**
     * 绩效组详情
     *
     * @param id 绩效组id
     * @return
     */
    @GetMapping("/group-config")
    @ControllerMethod("绩效组详情")
    public Message<KpiGroupVO> kpiGroupDetail(@NotNull(message = "绩效组id不能为空") Long id) {
        return success(kpiGroupBizService.kpiGroupDetail(id));
    }

    /**
     * 实时查询绩效组人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/group-config/real-time/staffs")
    @ControllerMethod("实时查询绩效组人员")
    public Message<List<KpiGroupUserVO>> queryShopPostRealTimeStaffs(@Validated KpiGroupStaffQueryDTO dto) {
        return success(kpiGroupUserBizService.queryShopPostRealTimeStaffs(dto));
    }

    /**
     * 绩效组指标排名
     *
     * @param dto
     * @return
     */
    @GetMapping("/kpi-indicator/rank")
    @ControllerMethod("绩效组指标排名")
    public Message<KpiIndicatorRankVO> queryKpiIndicatorRank(KpiGroupIndicatorRankDTO dto) {
        if (PublicUtil.isEmpty(dto.getKpiGroupIndicatorParamId()) && PublicUtil.isEmpty(dto.getKpiGroupIndicatorPreconditionId())) {
            throw new BusinessException("参数错误");
        }
        return success(kpiGroupIndicatorBizService.queryKpiIndicatorRank(dto));
    }

    /**
     * 绩效组草稿详情
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/kpi-detail")
    @ControllerMethod("绩效组草稿详情")
    public Message<KpiGroupDTO> draftKpiGroupDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftKpiGroupDetail(id));
    }

    /**
     * 人员质量考评预期
     *
     * @param userId
     * @return
     */
    @GetMapping("/assess-expected")
    @ControllerMethod("人员质量考评预期")
    public Message<KpiAssessResultVO> assessExpected(@NotNull(message = "用户id不能为空") Long userId) {
//                                                     @NotNull(message = "人员质量考核类型不能为空") Integer type) {
//        KpiAssessTypeEnum kpiAssessType = KpiAssessTypeEnum.ofValue(type);
//        BV.notNull(kpiAssessType, "参数错误");
        return success(kpiAssessBizService.assessExpected(userId, LocalDate.now().minusDays(1)));
    }

    /**
     * 人员质量考评处罚奖励详情
     *
     * @param businessToken
     * @return
     */
    @GetMapping("/kpi-assess")
    @ControllerMethod("人员质量考评处罚奖励详情")
    public Message<KpiAssessResultVO> staffKpiAssessDetail(@NotNull(message = "token不能为空") String businessToken) {
        return success(kpiAssessBizService.staffKpiAssessDetail(businessToken));
    }

    /**
     * 绩效报表-绩效得分率排名详情
     *
     * @param dto
     * @return
     */
    @GetMapping("/report/rank-pool")
    @ControllerMethod("绩效报表-绩效得分率排名详情")
    public Message<List<KpiPoolVO>> queryKpiRankPool(@Validated KpiGroupRankConfigQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getYearMonth())) {
            dto.setYearMonth(YearMonth.from(LocalDate.now().minusDays(1)));
        }
        return success(kpiPoolBizService.queryKpiRankPool(dto));
    }

    /**
     * 绩效报表-配置详情
     *
     * @param dto 绩效组配置详情入参
     * @return
     */
    @GetMapping("/report-group-config")
    @ControllerMethod("绩效报表-配置详情")
    public Message<List<KpiGroupVO>> reportGroupConfig(@Validated KpiGroupRankConfigQueryDTO dto) {
        return success(kpiGroupBizService.reportGroupConfig(dto));
    }

    /**
     * 绩效排名组草稿详情（审批）
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/kpi-rank-detail")
    @ControllerMethod("绩效排名组草稿详情（审批）")
    public Message<KpiGroupRankDTO> draftKpiGroupRankDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftKpiGroupRankDetail(id));
    }


}
