package cn.fw.morax.server.controller.app;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.dto.query.SalaryGroupStaffQueryDTO;
import cn.fw.morax.domain.dto.query.SalaryPoolQueryDTO;
import cn.fw.morax.domain.dto.query.StaffSalaryQueryDTO;
import cn.fw.morax.domain.dto.salary.SalaryGroupDTO;
import cn.fw.morax.domain.dto.salary.SalaryGroupRankConfigQueryDTO;
import cn.fw.morax.domain.vo.SelectorVO;
import cn.fw.morax.domain.vo.salary.*;
import cn.fw.morax.service.biz.SettingDraftBizService;
import cn.fw.morax.service.biz.salary.SalaryConfirmBizService;
import cn.fw.morax.service.biz.salary.SalaryGroupBizService;
import cn.fw.morax.service.biz.salary.SalaryGroupUserBizService;
import cn.fw.morax.service.biz.salary.SalaryPoolBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 薪酬池查询
 *
 * @author jiangchao
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/salary")
@IgnoreAuth
public class SalaryController {

    private final SalaryGroupBizService salaryGroupBizService;
    private final SalaryPoolBizService salaryPoolBizService;
    private final SalaryConfirmBizService salaryConfirmBizService;
    private final SalaryGroupUserBizService salaryGroupUserBizService;
    private final SettingDraftBizService settingDraftBizService;

    /**
     * 薪酬池查询
     *
     * @param queryDTO 筛选条件
     * @return 薪酬池列表
     */
    @GetMapping("/pool/list")
    @IgnoreAuth
    @ControllerMethod("薪酬池查询")
    public Message<AppPage<SalaryPoolVO>> salaryPool(SalaryPoolQueryDTO queryDTO) {
        if (PublicUtil.isNotEmpty(queryDTO.getYearMonth())) {
            queryDTO.setMonthly(queryDTO.getYearMonth());
        }
        if (PublicUtil.isEmpty(queryDTO.getMonthly())) {
            queryDTO.setMonthly(YearMonth.now());
        }
        return success(salaryPoolBizService.queryPool(queryDTO));
    }


    /**
     * 通过薪酬池id查询详情 [common]
     *
     * @param poolId 薪酬池id
     * @return 薪酬池详情
     */
    @GetMapping("/pool/detail")
    @IgnoreAuth
    @ControllerMethod("薪酬池查询")
    public Message<StaffSalaryVO> salaryPoolDetail(@NotNull(message = "薪酬池id不能为空") Long poolId,
                                                   @RequestParam(required = false) YearMonth yearMonth) {
        return success(salaryPoolBizService.queryPoolDetail(poolId, yearMonth));
    }

    /**
     * 薪酬池详情查询 [owner]
     *
     * @param userId    用户id+
     * @param yearMonth 月度
     * @return 薪酬池详情
     * @ignoreParams userId
     */
    @GetMapping("/pool/own-detail")
    @ControllerMethod("薪酬池查询")
    public Message<StaffSalaryVO> salaryPoolOwnDetail(@CurrentUser Long userId, @RequestParam(required = false) YearMonth yearMonth) {
        if (Objects.isNull(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        return success(salaryPoolBizService.queryUserPoolDetail(userId, yearMonth));
    }

    /**
     * 本月可查看的其他薪酬池选项
     *
     * @param user      用户id
     * @param userId    用户id [管理层查看人员薪酬详情时需传值]
     * @param yearMonth 月度 [员工人查看自己的酬详情时需传值]
     * @return 薪酬池选项
     * @ignoreParams user
     */
    @GetMapping("/pool/selector")
    @IgnoreAuth
    @ControllerMethod("薪酬池查询")
    public Message<Set<SelectorVO>> salaryPoolSelector(@CurrentUser LoginAuthBean user, Long userId,
                                                       @RequestParam(required = false) YearMonth yearMonth) {
        if (Objects.isNull(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        if (Objects.isNull(userId)) {
            userId = user.getUserId();
        }
        return success(salaryPoolBizService.salaryPoolSelector(userId, yearMonth));
    }

    /**
     * 薪酬确认
     *
     * @param fid 签名附件id
     * @param id  确认id
     * @return
     */
    @PostMapping("/person/confirm")
    @IgnoreAuth
    @ControllerMethod("薪酬确认")
    public Message<Void> confirmSalary(@NotBlank(message = "签名不能为空") String fid, @NotNull(message = "确认id不能为空") Long id) {
        salaryConfirmBizService.confirmSalary(id, fid);
        return success();
    }

    /**
     * 薪酬项目阶梯达成详情
     *
     * @param salaryPoolId  薪酬池id
     * @param salaryGroupProjectId  薪酬项目组id
     * @param yearMonth 月份
     * @return
     */
    @GetMapping("/project/detail")
    @IgnoreAuth
    @ControllerMethod("薪酬项目阶梯达成详情")
    public Message<SalaryGroupIndicatorDetailVO> getHitProjectSetting(Long salaryPoolId,
                                                                      Long salaryGroupProjectId,
                                                                      @RequestParam(required = false) YearMonth yearMonth) {
        if (PublicUtil.isEmpty(yearMonth)) {
            yearMonth = YearMonth.now();
        }
        return success(salaryPoolBizService.getHitProjectSetting(salaryPoolId, salaryGroupProjectId, yearMonth));
    }

    /**
     * 实时查询薪酬组人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/group-config/real-time/staffs")
    @ControllerMethod("实时查询薪酬组人员")
    public Message<List<SalaryGroupUserVO>> queryShopPostRealTimeStaffs(@Validated SalaryGroupStaffQueryDTO dto) {
        return success(salaryGroupUserBizService.queryShopPostRealTimeStaffs(dto));
    }

    /**
     * 薪酬组设置详情
     *
     * @param id 薪酬组id
     * @return
     */
    @GetMapping("/group-config")
    @ControllerMethod("薪酬组设置详情")
    public Message<SalaryGroupVO> getSalaryGroupDetail(@NotNull(message = "薪酬组id不能为空") Long id) {
        return success(salaryGroupBizService.getSalaryGroupDetail(id));
    }

    /**
     * 薪酬组草稿详情
     *
     * @param id 草稿id
     * @return
     */
    @GetMapping("/salary-detail")
    @ControllerMethod("绩效组草稿详情")
    public Message<SalaryGroupDTO> draftSalaryGroupDetail(@NotNull(message = "草稿id不能为空") Long id) {
        return success(settingDraftBizService.draftSalaryGroupDetail(id));
    }

    /**
     * 薪酬排名查询
     *
     * @return 薪酬池列表
     */
    @GetMapping("/pool/rank")
    @IgnoreAuth
    @ControllerMethod("薪酬排名查询")
    public Message<List<SalaryPoolVO>> salaryRankPool(@Validated SalaryGroupRankConfigQueryDTO dto) {
        if (PublicUtil.isEmpty(dto.getYearMonth())) {
            dto.setYearMonth(YearMonth.from(LocalDate.now().minusDays(1)));
        }
        return success(salaryPoolBizService.salaryRankPool(dto));
    }

}
