package cn.fw.morax.server.controller.erp;

import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.morax.domain.dto.kpi.KpiGroupDTO;
import cn.fw.morax.domain.dto.salary.SalaryGroupDTO;
import cn.fw.morax.domain.dto.query.SalaryGroupQueryDTO;
import cn.fw.morax.domain.dto.query.SalaryGroupStaffQueryDTO;
import cn.fw.morax.domain.dto.query.SalaryGroupUserPageDTO;
import cn.fw.morax.domain.vo.salary.SalaryGroupProjectVO;
import cn.fw.morax.domain.vo.salary.SalaryGroupUserVO;
import cn.fw.morax.domain.vo.salary.SalaryGroupVO;
import cn.fw.morax.domain.vo.salary.SalaryProjectSelectorVO;
import cn.fw.morax.service.biz.salary.SalaryGroupBizService;
import cn.fw.morax.service.biz.salary.SalaryGroupUserBizService;
import cn.fw.morax.service.biz.salary.SalaryProjectBizService;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 薪酬组配置
 *
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryGroupController
 * @description : 薪酬组配置
 * @date : 2022-05-12 09:27
 */
@Slf4j
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.ERP)
@IgnoreAuth
@Validated
@RequestMapping("/erp/salary/group")
public class SalaryGroupController {

    private final SalaryGroupUserBizService salaryGroupUserBizService;
    private final SalaryGroupBizService salaryGroupBizService;
    private final SalaryProjectBizService salaryProjectBizService;

    /**
     * 薪酬项配置列表
     *
     * @return
     */
    @GetMapping("/page")
    @ControllerMethod("薪酬项配置列表")
    public Message<AppPage<SalaryGroupVO>> salaryGroupConfigPage(@Validated SalaryGroupQueryDTO dto) {
        return success(salaryGroupBizService.salaryGroupConfigPage(dto));
    }

    /**
     * 薪酬组设置详情
     *
     * @param id 薪酬组id
     * @return
     */
    @GetMapping("/detail")
    @ControllerMethod("薪酬组设置详情")
    public Message<SalaryGroupVO> getSalaryGroupDetail(@NotNull(message = "薪酬组id不能为空") Long id) {
        return success(salaryGroupBizService.getSalaryGroupDetail(id));
    }

    /**
     * 薪酬组保存，1：提交审批 2：保存草稿
     *
     * @param dto 薪酬组数据
     * @param currentUser
     * @return
     * @ignoreParams currentUser
     */
    @PostMapping("/save/{submit}")
    @ControllerMethod("薪酬组保存")
    public Message<Void> saveSalaryGroup(@Validated @RequestBody SalaryGroupDTO dto,
                                         @CurrentUser LoginAuthBean currentUser,
                                         @PathVariable("submit") Integer submit) {
        Boolean isSubmit = (submit == 1);
        salaryGroupBizService.saveSalaryGroup(dto, currentUser, isSubmit);
        return success();
    }

    /**
     * 薪酬组配置指标选择器
     *
     * @param postId  岗位id
     * @param shopIds 适用门店ids
     * @return
     */
    @GetMapping("/indicator/selector")
    @ControllerMethod("薪酬组配置指标选择器")
    public Message<List<SalaryProjectSelectorVO>> indicatorSelector(@NotNull(message = "岗位不能为空") Long postId,
                                                                    @NotBlank(message = "适用门店不能为空") String shopIds) {
        return success(salaryProjectBizService.indicatorSelector(postId, shopIds));
    }

    /**
     * 薪酬组配置所有指标
     *
     * @return
     */
    @GetMapping("/all-indicator")
    @ControllerMethod("薪酬组配置指标选择器")
    public Message<List<SalaryProjectSelectorVO>> allIndicators() {
        return success(salaryProjectBizService.allIndicators());
    }

    /**
     * 薪酬组项目设置详情
     * * fixme 暂时屏蔽 应该是不需要的
     *
     * @param id 薪酬组项目id
     * @return
     * @ignore
     */
    @GetMapping("/project/detail")
    @ControllerMethod("薪酬组项目设置详情")
    public Message<SalaryGroupProjectVO> getSalaryGroupProjectDetail(@NotNull(message = "薪酬组项目id不能为空") Long id) {
        return success(salaryGroupBizService.getSalaryGroupProjectDetail(id));
    }

    /**
     * 实时查询薪酬组人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/real-time/staffs")
    @ControllerMethod("实时查询薪酬组人员")
    public Message<List<SalaryGroupUserVO>> queryShopPostRealTimeStaffs(@Validated SalaryGroupStaffQueryDTO dto) {
        return success(salaryGroupUserBizService.queryShopPostRealTimeStaffs(dto));
    }

    /**
     * 查询薪酬组人员
     *
     * @param dto
     * @return
     */
    @GetMapping("/staffs")
    @ControllerMethod("查询薪酬组人员")
    public Message<AppPage<SalaryGroupUserVO>> querySalaryGroupUsers(@Validated SalaryGroupUserPageDTO dto) {
        return success(salaryGroupUserBizService.querySalaryGroupUsers(dto));
    }

    /**
     * 禁用薪酬组
     *
     * @param id 薪酬组id
     * @return
     */
    @GetMapping("/disable")
    @ControllerMethod("禁用薪酬组")
    public Message<Void> disableGroup(@NotNull(message = "薪酬组id不能为空") Long id,
                                      @CurrentUser LoginAuthBean currentUser) {
        salaryGroupBizService.disableGroup(id, currentUser);
        return success();
    }

}
