package cn.fw.morax.server.task;

import cn.fw.morax.common.constant.TimeTaskConstant;
import cn.fw.morax.service.biz.eval.EvalCalculateService;
import cn.fw.morax.service.biz.eval.EvalIndicatorValueBizService;
import cn.fw.morax.service.biz.eval.EvalRewardService;
import cn.fw.morax.service.biz.kpi.IndicatorBizService;
import cn.fw.morax.service.biz.kpi.KpiCalculateService;
import cn.fw.morax.service.biz.kpi.KpiPoolBizService;
import lombok.AllArgsConstructor;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.YearMonth;

/**
 * @author : jiangchao
 * @className : EvalCalcTask
 * @description : 考评定时器
 * @date : 2023-01-12 15:29
 */
@Component
@Slf4j
@RequiredArgsConstructor
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class EvalCalcTask {

    private final EvalRewardService evalRewardService;
    private final EvalCalculateService evalCalculateService;
    private final EvalIndicatorValueBizService evalIndicatorValueBizService;

    /**
     * 需要计算考评得分考评组写入缓存 每天4点
     */
    @Scheduled(cron = TimeTaskConstant.CACHE_CALCULATE_EVAL_GROUP)
    public void cacheCalculableEvalGroup() {
        evalCalculateService.cacheCalculableEvalGroupIds(LocalDate.now().minusDays(1L));
    }

    /**
     * 计算考评 每10分钟执行一次
     */
    @Scheduled(initialDelay = 1000L * 15, fixedRate = 1000L * 600)
    public void calculateCacheEvalGroup() {
        evalCalculateService.calculateCacheEvalGroup();
    }

    /**
     * 缓存需要上报的考评指标
     */
    @Scheduled(cron = "0 0 9 30 * ?")
    public void cacheReportEvalIndicator() {
        evalIndicatorValueBizService.cacheReportEvalIndicator(LocalDate.now());
    }

    /**
     * 持久化需要上报的考评指标
     */
    @Scheduled(initialDelay = 1000L * 35, fixedRate = 1000L * 50)
    @Async
    public void persistenceReportTodo() {
        evalIndicatorValueBizService.persistenceReportTodo();
    }

    /**
     * 每5分钟检查待办推送
     */
    @Scheduled(cron = "0 0/30 * * * ?")
    public void pushReportTodo() {
        evalIndicatorValueBizService.pushReportTodo();
    }

    /**
     * 缓存分配奖惩的考评池
     */
    @Scheduled(cron = "0 0 10 * * ?")
    public void cacheDistEvalShopPoolIds() {
        evalRewardService.cacheDistEvalShopPoolIds(LocalDate.now().minusDays(1));
    }

    /**
     * 分配奖惩
     */
    @Scheduled(initialDelay = 1000L * 35, fixedRate = 1000L * 50)
    @Async
    public void distributionEvalReward() {
        evalRewardService.distributionEvalReward();
    }

    /**
     * 缓存考评排名组 每天6点
     */
    @Scheduled(cron = TimeTaskConstant.CACHE_CALCULATE_EVAL_GROUP_RANK)
    public void cacheCalculableEvalGroupRank() {
        evalCalculateService.cacheCalculableEvalGroupRank(LocalDate.now().minusDays(1L));
    }

    /**
     * 计算考评排名组 每600秒执行一次
     */
    @Scheduled(initialDelay = 1000L * 35, fixedRate = 1000L * 600)
    @Async
    public void calcEvalGroupRank() {
        evalCalculateService.calcEvalGroupRank();
    }

}
