package cn.fw.morax.server.task;

import cn.fw.morax.common.constant.TimeTaskConstant;
import cn.fw.morax.domain.db.kpi.KpiAssess;
import cn.fw.morax.service.biz.kpi.IndicatorBizService;
import cn.fw.morax.service.biz.kpi.KpiCalculateAssessService;
import cn.fw.morax.service.biz.kpi.KpiCalculateService;
import cn.fw.morax.service.biz.kpi.KpiPoolBizService;
import cn.fw.morax.service.data.kpi.KpiAssessService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.YearMonth;

/**
 * @author : kurisu
 * @className : KpiCalcTask
 * @description : 绩效处理任务
 * @date : 2022-04-07 15:29
 */
@Component
@Slf4j
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class KpiCalcTask {
    private final KpiCalculateService kpiCalculateService;
    private final KpiPoolBizService kpiPoolBizService;
    private final KpiCalculateAssessService kpiCalculateAssessService;
    private final IndicatorBizService indicatorBizService;

    @Autowired
    public KpiCalcTask(final KpiCalculateService kpiCalculateService,
                       final KpiPoolBizService kpiPoolBizService,
                       final KpiCalculateAssessService kpiCalculateAssessService,
                       final IndicatorBizService indicatorBizService) {
        this.kpiCalculateAssessService = kpiCalculateAssessService;
        this.kpiCalculateService = kpiCalculateService;
        this.kpiPoolBizService = kpiPoolBizService;
        this.indicatorBizService = indicatorBizService;
    }


    /**
     * 持久化业务系统上报的数据
     */
    @Scheduled(initialDelay = 1000 * 10, fixedRate = 1000 * 15)
    public void persistence() {
        indicatorBizService.persistenceUserData();
        indicatorBizService.persistenceShopData();
        indicatorBizService.persistenceUserStageData();
        indicatorBizService.persistenceShopStageData();
    }

    /**
     * 需要计算绩效得分的绩效组写入缓存 每天4点
     */
    @Scheduled(cron = TimeTaskConstant.CACHE_CALCULATE_KPI_GROUP)
    public void cacheCalculableKpiGroup() {
        kpiCalculateService.cacheCalculableKpiGroupIds(LocalDate.now().minusDays(1L));
    }

    /**
     * 计算绩效得分 每60秒执行一次
     */
    @Scheduled(initialDelay = 1000L * 15, fixedRate = 1000L * 60)
    public void calculateKpi() {
        kpiCalculateService.calculateKpi();
    }

    /**
     * 计算绩效得分率 每40秒执行一次
     */
    @Scheduled(initialDelay = 1000L * 20, fixedRate = 1000L * 40)
    @Async
    public void calcuScoringRate() {
        kpiPoolBizService.calcuScoringRate();
    }

    /**
     * 需要计算绩效排名的绩效组写入缓存 每天5点
     */
    @Scheduled(cron = TimeTaskConstant.CACHE_CALCULATE_KPI_GROUP_RANK)
    public void cacheCalculableKpiGroupRank() {
        kpiPoolBizService.cacheCalculableKpiGroupRank(LocalDate.now().minusDays(1L));
    }

    /**
     * 计算用户星级 每50秒执行一次
     */
    @Scheduled(initialDelay = 1000L * 35, fixedRate = 1000L * 50)
    @Async
    public void calcuStarLevel() {
        kpiPoolBizService.calcuStarLevel();
    }

    /**
     * 缓存人员质量评测
     */
    @Scheduled(cron = "0 50 5 * * ?")
    public void cacheKpiAssess() {
        kpiCalculateAssessService.cacheCalcKpiAssessIds(LocalDate.now().minusDays(1L), Boolean.FALSE);
    }

}
