package cn.fw.morax.server.task;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.morax.common.constant.TimeTaskConstant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.KpiGroup;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicator;
import cn.fw.morax.domain.db.kpi.KpiGroupIndicatorParam;
import cn.fw.morax.domain.dto.query.KpiGroupRepeatQueryDTO;
import cn.fw.morax.domain.enums.IndicatorCodeTypeEnum;
import cn.fw.morax.domain.enums.SettingStatusEnum;
import cn.fw.morax.service.biz.kpi.KpiGroupBizService;
import cn.fw.morax.service.biz.kpi.KpiGroupDataService;
import cn.fw.morax.service.data.kpi.KpiGroupIndicatorParamService;
import cn.fw.morax.service.data.kpi.KpiGroupIndicatorService;
import cn.fw.morax.service.data.kpi.KpiGroupService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.*;
import java.util.concurrent.locks.Lock;
import java.util.stream.Collectors;

/**
 * @author : jiangchao
 * @className : KpiGroupStatusTask
 * @description : 绩效组配置状态定时器
 * @date : 2022-04-07 15:29
 */
@Component
@Slf4j
@RequiredArgsConstructor
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class KpiGroupTask {

    private final KpiGroupDataService kpiGroupDataService;
    private final KpiGroupBizService kpiGroupBizService;
    private final KpiGroupService kpiGroupService;
    private final KpiGroupIndicatorService kpiGroupIndicatorService;
    private final KpiGroupIndicatorParamService kpiGroupIndicatorParamService;
    private final DistributedLocker distributedLocker;

    private final static String LOCK_KEY = "kpi:group";

    /**
     * 每个月1号凌晨3点执行
     * 1. 将待生效数据改为生效中
     * 2. 将之前的配置设置为失效
     * 3. 处理重复数据，合并、拆分门店的绩效组配置需要把其他的重复配置失效
     */
    @Scheduled(cron = TimeTaskConstant.KPI_GROUP_STATUS)
    @Transactional(rollbackFor = Exception.class)
    public void processCurMonthEffectKpis() {
        Lock lock = distributedLocker.lock(LOCK_KEY);
        if (((RLock) lock).isLocked()) {
            try {
                log.info("定时任务【每月绩效组配置状态改变】开始执行");
                LocalDate currentTime = LocalDate.now();
                List<KpiGroup> beEffectiveKpiGroups = kpiGroupService.list(Wrappers.<KpiGroup>lambdaQuery()
                        .eq(KpiGroup::getStatus, SettingStatusEnum.BE_EFFECTIVE)
                        .eq(KpiGroup::getBeginTime, currentTime)
                        .eq(KpiGroup::getId, Boolean.TRUE)
                );
                if (PublicUtil.isEmpty(beEffectiveKpiGroups)) {
                    log.info("绩效组定时器：暂无即将生效的绩效组配置");
                    return;
                }
                //之前配置都设置为无效（之前配置一定被使用过）
                List<Long> preIds =  beEffectiveKpiGroups.stream()
                        .filter(kpiGroup -> PublicUtil.isNotEmpty(kpiGroup.getPreId())).map(KpiGroup::getPreId).collect(Collectors.toList());
                kpiGroupDataService.modifyStatusByIds(preIds, SettingStatusEnum.INEFFECTIVE);

                //以岗位、门店的维度，出现重复的配置，将创建时间最早的设置为失效
                this.processCurMonthRepeatKpis(beEffectiveKpiGroups);

                //生效的配置
                kpiGroupDataService.modifyStatusByKpis(beEffectiveKpiGroups, SettingStatusEnum.EFFECTIVE);
            } catch (Exception e){
                log.error(e.getMessage(), e);
            } finally {
                lock.unlock();
            }
        }
    }

    /**
     * 处理和当月生效kpi 重复的kpi数据
     * @param beEffectiveKpiGroups
     */
    @Transactional(rollbackFor = Exception.class)
    public void processCurMonthRepeatKpis(List<KpiGroup> beEffectiveKpiGroups) {
        Map<Long, List<KpiGroup>> groupKpiMap = beEffectiveKpiGroups.stream().collect(Collectors.groupingBy(KpiGroup::getGroupId));
        Long groupId = null;
        Long postId = null;
        List<KpiGroup> kpis = null;
        Set<Long> shopIds = null;
        List<KpiGroup> beEffectiveKpis = null;
        Map<Long, List<KpiGroup>> postKpiMap = null;
        for (Map.Entry<Long, List<KpiGroup>> entry : groupKpiMap.entrySet()) {
            groupId = entry.getKey();
            beEffectiveKpis = entry.getValue();
            postKpiMap = beEffectiveKpis.stream().collect(Collectors.groupingBy(KpiGroup::getPostId));
            for (Map.Entry<Long, List<KpiGroup>> postKpi : postKpiMap.entrySet()) {
                postId = postKpi.getKey();
                kpis = postKpi.getValue();
                shopIds = new HashSet<>();
                for (KpiGroup kpiGroup : kpis) {
                    shopIds.addAll(kpiGroup.getShopIds());
                }
                KpiGroupRepeatQueryDTO repeatQueryDTO = KpiGroupRepeatQueryDTO.builder()
                        .shopIds(new ArrayList<>(shopIds))
                        .status(SettingStatusEnum.EFFECTIVE)
                        .groupId(groupId)
                        .postId(postId)
                        .build();
                //处理当月重复数据
                List<KpiGroup> effectKpiGroups = kpiGroupService.queryRepeatKpiGroup(repeatQueryDTO);
                kpiGroupDataService.modifyStatusByKpis(effectKpiGroups, SettingStatusEnum.INEFFECTIVE);
            }
        }
    }

    /**
     * 处理绩效旧组合指标
     */
    @Transactional(rollbackFor = Exception.class)
    public void processCombine() {
        LocalDate date = LocalDate.now().minusDays(1);
        List<KpiGroup> allKpiGroups = kpiGroupService.queryKgiGroupsByDay(date);
        List<Long> kpiGroupIds = allKpiGroups.stream().map(KpiGroup::getId).collect(Collectors.toList());
        if (PublicUtil.isEmpty(kpiGroupIds)) {
            return;
        }

        List<KpiGroupIndicator> kpiGroupIndicators = kpiGroupIndicatorService.list(Wrappers.<KpiGroupIndicator>lambdaQuery()
                .in(KpiGroupIndicator::getKpiGroupId, kpiGroupIds)
                .eq(KpiGroupIndicator::getYn, Boolean.TRUE)
        );
        List<Long> kpiGroupIndicatorIds = kpiGroupIndicators.stream().map(KpiGroupIndicator::getId).collect(Collectors.toList());
        List<KpiGroupIndicatorParam> kpiGroupIndicatorParams = kpiGroupIndicatorParamService.list(Wrappers.<KpiGroupIndicatorParam>lambdaQuery()
                .in(KpiGroupIndicatorParam::getKpiGroupIndicatorId, kpiGroupIndicatorIds)
                .eq(KpiGroupIndicatorParam::getYn, Boolean.TRUE)
        );
        Map<Long, List<KpiGroupIndicatorParam>> indicatorParamMap = kpiGroupIndicatorParams.stream()
                .collect(Collectors.groupingBy(KpiGroupIndicatorParam::getKpiGroupIndicatorId));
        List<KpiGroupIndicator> updateKpiGroupIndicators = new ArrayList<>();
        for (KpiGroupIndicator kpiGroupIndicator : kpiGroupIndicators) {
            List<KpiGroupIndicatorParam> params = indicatorParamMap.get(kpiGroupIndicator.getId());
            if (params.size() > 1) {
                kpiGroupIndicatorService.removeById(kpiGroupIndicator.getId());
                kpiGroupIndicatorParamService.removeByIds(params.stream().map(KpiGroupIndicatorParam::getId).collect(Collectors.toList()));
                continue;
            }
            KpiGroupIndicatorParam param = params.stream().findFirst().get();
            kpiGroupIndicator.setCode(param.getIndicatorCode());
            kpiGroupIndicator.setCodeType(IndicatorCodeTypeEnum.INDICATOR);
            updateKpiGroupIndicators.add(kpiGroupIndicator);
        }
        if (PublicUtil.isNotEmpty(updateKpiGroupIndicators)) {
            kpiGroupIndicatorService.updateBatchById(updateKpiGroupIndicators);
        }
    }

}
