package cn.fw.morax.server.task;

import cn.fw.common.cache.locker.DistributedLocker;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.constant.TimeTaskConstant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.dto.kpi.KpiGroupInfoDTO;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.domain.vo.kpi.KpiGroupIndicatorParamVO;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.ManagerDTO;
import cn.fw.morax.rpc.ehr.dto.ManagerStaffRoleDTO;
import cn.fw.morax.rpc.ehr.dto.ManagerStaffShopDTO;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.service.biz.CommonService;
import cn.fw.morax.service.biz.kpi.KpiGroupRankBizService;
import cn.fw.morax.service.data.kpi.*;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Maps;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.map.MultiKeyMap;
import org.redisson.api.RLock;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.locks.Lock;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : jiangchao
 * @className : KpiReportRatioTask
 * @description : 绩效报表定时器
 * @date : 2022-04-07 15:29
 */
@Component
@Slf4j
@RequiredArgsConstructor
@ConditionalOnProperty(prefix = "task", name = "switch", havingValue = "on")
public class KpiReportTask {

    private final KpiGroupIndicatorTargetHitLogService kpiGroupIndicatorTargetHitLogService;
    private final KpiGroupIndicatorHitLogService kpiGroupIndicatorHitLogService;
    private final KpiGroupIndicatorParamService kpiGroupIndicatorParamService;
    private final ReportKpiDimValueExtraService reportKpiDimValueExtraService;
    private final PlatformTransactionManager platformTransactionManager;
    private final IndicatorUserValueService indicatorUserValueService;
    private final ReportKpiDimValueService reportKpiDimValueService;
    private final KpiGroupIndicatorService kpiGroupIndicatorService;
    private final ReportKpiRankCodeService reportKpiRankCodeService;
    private final KpiGroupRankBizService kpiGroupRankBizService;
    private final ReportKpiValueService reportKpiValueService;
    private final TransactionDefinition transactionDefinition;
    private final ReportKpiRankService reportKpiRankService;
    private final KpiGroupRankService kpiGroupRankService;
    private final DistributedLocker distributedLocker;
    private final KpiGroupService kpiGroupService;
    private final KpiPoolService kpiPoolService;
    private final CommonService commonService;
    private final EhrRpcService ehrRpcService;


    @Value("${spring.cache.custom.global-prefix}:kpi:group:report")
    @Getter
    private String kpiGroupReportDistKey;

    /**
     * 绩效报表定时任务
     *
     */
    @Scheduled(cron = TimeTaskConstant.KPI_REPORT)
    @Transactional(rollbackFor = Exception.class)
    public void kpiReportTask() {
        this.kpiReport(LocalDate.now().minusDays(1));
    }

    /**
     * 绩效报表定时任务（
     * 人员质量评测每月2号计算上月、计算完成后会操作上月绩效分，所以再次抽取上月报表）
     */
    @Scheduled(cron = TimeTaskConstant.KPI_MONTH_FINAL_REPORT)
    @Transactional(rollbackFor = Exception.class)
    public void kpiLastMonthReportTask() {
        this.kpiReport(YearMonth.now().minusMonths(1L).atEndOfMonth());
    }

    /**
     * 绩效报表定时任务
     *
     * @param date
     */
    public void kpiReport(LocalDate date) {
        Lock lock = distributedLocker.lock(getKpiGroupReportDistKey());
        if (! ((RLock) lock).isLocked()) {
            return;
        }
        TransactionStatus transactionStatus = platformTransactionManager.getTransaction(transactionDefinition);
        try {
            log.info("定时任务【绩效报表数据抽取-新】开始执行");
            //查询所有需要处理的排名组
            List<KpiGroupRank> kpiGroupRanks = queryNeedRankGroup(date);
            if (CollectionUtils.isEmpty(kpiGroupRanks)){
                log.warn("没有需要处理的绩效排名组");
                return;
            }
            //清理数据,每天留一条当月最新数据,每个月留一条月最后一天的数据
            cleanData(date);
            for (KpiGroupRank kpiGroupRank : kpiGroupRanks) {
                //抽取排名组数据
                createRankReport(kpiGroupRank, kpiGroupRank.getGroupId(), date);
            }
            platformTransactionManager.commit(transactionStatus);
        } catch (Exception e){
            platformTransactionManager.rollback(transactionStatus);
            log.error(e.getMessage(), e);
        } finally {
            lock.unlock();
        }
    }

    /**
     * 绩效排名组报表数据抽取
     * @param kpiGroupRank
     * @param groupId
     * @param date
     */
    private void createRankReport(KpiGroupRank kpiGroupRank, Long groupId, LocalDate date) {
        //获取绩效排名组下的所有绩效组信息
        List<KpiGroup> kpiGroups = this.getKpiGroups(kpiGroupRank.getKpiGroupIds(), date);
        if (CollectionUtils.isEmpty(kpiGroups)){
            log.error("绩效排名组id [{}] 下不存在有效的绩效组信息",kpiGroupRank.getId());
            return;
        }
        List<Long> kpiGroupIds = kpiGroups.stream().map(KpiGroup::getId).collect(Collectors.toList());
        List<KpiPool> pools = kpiPoolService.list(Wrappers.<KpiPool>lambdaQuery()
                .in(KpiPool::getKpiGroupId, kpiGroupIds)
                .eq(KpiPool::getMonthly, YearMonth.from(date))
                .eq(KpiPool::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isEmpty(pools)) {
            return;
        }
        //保存 报表绩效排名组 信息
        ReportKpiRank reportKpiRank = saveReportKpiRank(kpiGroupRank, groupId, date, kpiGroups, kpiGroupIds);

        //绩效排名组下归属的所有绩效组的对应指标集合
        List<KpiGroupIndicator> kpiGroupIndicators = kpiGroupIndicatorService.list(Wrappers.<KpiGroupIndicator>lambdaQuery()
                .in(KpiGroupIndicator::getKpiGroupId, kpiGroupIds)
                .eq(KpiGroupIndicator::getYn, Boolean.TRUE)
                .orderByAsc(KpiGroupIndicator::getId)
        );

        //一个绩效排名组对应多个绩效组,也对应绩效组下的所有指标集合
        saveReportKpiRankCodes(date, reportKpiRank, kpiGroupIndicators);
        List<ReportKpiDimValue> userDims = new ArrayList();
        //人员维度
        List<ReportKpiValue> userKpiValues = calcUser(kpiGroupIndicators, userDims, date, pools, reportKpiRank);
        //门店维度
        calcShop(kpiGroupIndicators, pools, reportKpiRank, userKpiValues, userDims, date);
        //管理者维度
        calcManager(kpiGroupIndicators, pools, reportKpiRank, userKpiValues, userDims, date);
    }

    private List<ReportKpiValue> calcUser(List<KpiGroupIndicator> kpiGroupIndicators,
                                          List<ReportKpiDimValue> userDims,
                                          LocalDate date,
                                          List<KpiPool> pools,
                                          ReportKpiRank reportKpiRank) {
        MultiKeyMap<Long, BigDecimal> userIndicatorHitMap = this.getUserIndicatorHitMap(kpiGroupIndicators, date,pools);
        List<ReportKpiValue> reportKpiValues = new ArrayList<>();
        //以人的维度为基础,计算各个指标信息对应的数据
        //绩效排名组下的所有用户集合
        for (KpiPool pool : pools) {
            ReportKpiDimValue reportKpiDimValue = getDim(pool, date, reportKpiRank.getId());
            userDims.add(reportKpiDimValue);
            reportKpiDimValueService.save(reportKpiDimValue);
            final Long userId = pool.getUserId();
            final Long reportKpiDimId = reportKpiDimValue.getId();
            //绩效排名组下的所有指标集合
            for (KpiGroupIndicator kpiGroupIndicator : kpiGroupIndicators) {
                if (! pool.getKpiGroupId().equals(kpiGroupIndicator.getKpiGroupId())) {
                    continue;
                }
                final Long kpiGroupIndicatorId = kpiGroupIndicator.getId();
                //kpi_group_indicator_hit_log 表 UNIQUE KEY `salary_project_hit_log_index` (`kpi_group_indicator_id`,`data_date`,`user_id`) USING BTREE
                BigDecimal kpiValue = this.convertHitValue(kpiGroupIndicator, userIndicatorHitMap.get(userId, kpiGroupIndicatorId));
                ReportKpiValue reportKpiValue = this.convertDB(kpiGroupIndicator, reportKpiDimId, date, kpiValue) ;
                reportKpiValues.add(reportKpiValue);
                //是组合指标,还要向下处理他的子指标
                if (IndicatorCodeTypeEnum.COMBINE_INDICATOR.equals(kpiGroupIndicator.getCodeType())) {
                    List<KpiGroupIndicatorParamVO> kpiGroupIndicatorParamVos = kpiGroupIndicatorParamService.getKpiGroupIndicatorParamVos(kpiGroupIndicator.getId());
                    for (KpiGroupIndicatorParamVO kpiGroupIndicatorParam : kpiGroupIndicatorParamVos) {
                        if (TargetTypeEnum.NO.equals(kpiGroupIndicatorParam.getTargetType())) {
                            //无目标
                            Map<Long, BigDecimal> userValueMap = queryUserOriginValue(kpiGroupIndicatorParam.getIndicatorCode(), userId, date);
                            BigDecimal value = userValueMap.getOrDefault(userId, BigDecimal.ZERO);
                            ReportKpiValue reportKpiValue1 = this.convertNoTargetDB(kpiGroupIndicator, kpiGroupIndicatorParam, date, reportKpiDimId, value);
                            reportKpiValues.add(reportKpiValue1);
                        } else {
                            //有目标
                            Map<Long, KpiGroupIndicatorTargetHitLog> targetHitLogMap = queryUserTargetValue(kpiGroupIndicatorParam.getId(), userId, date);
                            KpiGroupIndicatorTargetHitLog hitLog = targetHitLogMap.get(userId);
                            BigDecimal value = Optional.ofNullable(hitLog).map(KpiGroupIndicatorTargetHitLog::getReachValue).orElse(BigDecimal.ZERO);
                            ReportKpiValue reportKpiValue1 = this.convertTargetDB(kpiGroupIndicator, kpiGroupIndicatorParam, date, reportKpiDimId, value);
                            reportKpiValues.add(reportKpiValue1);
                        }
                    }
                }
            }
        }
        if (!CollectionUtils.isEmpty(reportKpiValues)){
            reportKpiValueService.saveBatch(reportKpiValues);
        }
        return reportKpiValues;
    }

    public void calcShop(List<KpiGroupIndicator> kpiGroupIndicators,
                          List<KpiPool> pools,
                          ReportKpiRank reportKpiRank,
                          List<ReportKpiValue> userKpiValues,
                          List<ReportKpiDimValue> userDims,
                          LocalDate date) {
        Map<Long, String> shopMap = pools.stream().collect(Collectors.toMap(KpiPool::getShopId, KpiPool::getShopName, (v1, v2) -> v1));
        for (Long shopId : shopMap.keySet()) {
            Set<Long> shopDimReportIds = userDims.stream()
                    .filter(user -> shopId.equals(user.getShopId()) && Boolean.TRUE.equals(user.getInclusion()))
                    .map(ReportKpiDimValue::getId).collect(Collectors.toSet());
            BigDecimal kpiScoreRatio = calcAverageKpiScoreRatio(userDims, shopDimReportIds);
            final Long shopReportKpiDimId = this.saveShopDimDB(shopId, shopMap, date, reportKpiRank.getId(), kpiScoreRatio);

            List<ReportKpiValue> shopReportKpiValues = new ArrayList<>();
            for (KpiGroupIndicator kpiGroupIndicator : kpiGroupIndicators) {
                BigDecimal value = calcAverageValue(userKpiValues, shopDimReportIds, kpiGroupIndicator.getCode());
                shopReportKpiValues.add(this.convertShopValueDB(kpiGroupIndicator, value, shopReportKpiDimId, date));
                if (IndicatorCodeTypeEnum.COMBINE_INDICATOR.equals(kpiGroupIndicator.getCodeType())) {
                    List<KpiGroupIndicatorParam> kpiGroupIndicatorParams = kpiGroupIndicatorParamService.list(Wrappers.<KpiGroupIndicatorParam>lambdaQuery()
                            .eq(KpiGroupIndicatorParam::getKpiGroupIndicatorId, kpiGroupIndicator.getId())
                            .eq(KpiGroupIndicatorParam::getYn, Boolean.TRUE)
                    );
                    for (KpiGroupIndicatorParam kpiGroupIndicatorParam : kpiGroupIndicatorParams) {
                        BigDecimal paramValue = calcAverageValue(userKpiValues, shopDimReportIds, kpiGroupIndicatorParam.getIndicatorCode());
                        shopReportKpiValues.add(this.convertShopValueDBPcode(kpiGroupIndicatorParam, paramValue, shopReportKpiDimId, date,kpiGroupIndicator));
                    }
                }
            }
            if (!CollectionUtils.isEmpty(shopReportKpiValues)){
                reportKpiValueService.saveBatch(shopReportKpiValues);
            }
        }
    }

    public void calcManager(List<KpiGroupIndicator> kpiGroupIndicators,
                            List<KpiPool> pools,
                            ReportKpiRank reportKpiRank,
                            List<ReportKpiValue> userKpiValues,
                            List<ReportKpiDimValue> userDims,
                            LocalDate date) {
        List<Long> shopIds = pools.stream().map(KpiPool::getShopId).collect(Collectors.toList());
        List<ManagerDTO> managerDTOS = ehrRpcService.getRealTimeShopManager(new ArrayList<>(shopIds));
        log.info("查询门店实时管理者：{}，{}", JSON.toJSONString(shopIds), JSON.toJSONString(managerDTOS));
        if (PublicUtil.isEmpty(managerDTOS)) {
            return;
        }
        List<ReportKpiValue> reportKpiValues = new ArrayList<>();
        Map<Long, StaffBaseInfoDTO> managerMap = staffInfoMap(managerDTOS);
        final Long reportKpiRankId = reportKpiRank.getId();
        for (ManagerDTO manager : managerDTOS) {
            if (PublicUtil.isEmpty(manager.getManageStaffList())) {
                continue;
            }
            List<Long> manageStaffIds = manager.getManageStaffList().stream().map(StaffBaseInfoDTO::getId).distinct().collect(Collectors.toList());
            Set<Long> managerStaffDimReportIds = userDims.stream()
                    .filter(user -> manageStaffIds.contains(user.getUserId()) && Boolean.TRUE.equals(user.getInclusion()))
                    .map(ReportKpiDimValue::getId).collect(Collectors.toSet());
            if (PublicUtil.isEmpty(managerStaffDimReportIds)) {
                continue;
            }
            BigDecimal kpiScoreRatio = calcAverageKpiScoreRatio(userDims, managerStaffDimReportIds);
            final Long reportSalaryDimId = this.saveManagerDimDB(manager, managerMap, date, reportKpiRankId, manageStaffIds, kpiScoreRatio);

            for (KpiGroupIndicator kpiGroupIndicator : kpiGroupIndicators) {
                BigDecimal value = calcAverageValue(userKpiValues, managerStaffDimReportIds, kpiGroupIndicator.getCode());
                reportKpiValues.add(this.convertShopValueDB(kpiGroupIndicator, value, reportSalaryDimId, date));
                if (IndicatorCodeTypeEnum.COMBINE_INDICATOR.equals(kpiGroupIndicator.getCodeType())) {
                    List<KpiGroupIndicatorParam> kpiGroupIndicatorParams = kpiGroupIndicatorParamService.list(Wrappers.<KpiGroupIndicatorParam>lambdaQuery()
                            .eq(KpiGroupIndicatorParam::getKpiGroupIndicatorId, kpiGroupIndicator.getId())
                            .eq(KpiGroupIndicatorParam::getYn, Boolean.TRUE)
                    );
                    for (KpiGroupIndicatorParam kpiGroupIndicatorParam : kpiGroupIndicatorParams) {
                        BigDecimal paramValue = calcAverageValue(userKpiValues, managerStaffDimReportIds, kpiGroupIndicatorParam.getIndicatorCode());
                        reportKpiValues.add(this.convertShopValueDBPcode(kpiGroupIndicatorParam, paramValue, reportSalaryDimId, date,kpiGroupIndicator));
                    }
                }
            }
        }
        if (PublicUtil.isNotEmpty(reportKpiValues)) {
            reportKpiValueService.saveBatch(reportKpiValues);
        }
    }

    public BigDecimal convertHitValue(KpiGroupIndicator kpiGroupIndicator, BigDecimal value) {
        if (PublicUtil.isEmpty(value)) {
            return BigDecimal.ZERO;
        }
        if (IndicatorCodeTypeEnum.COMBINE_INDICATOR.equals(kpiGroupIndicator.getCodeType())) {
            return value.multiply(Constant.ONE_HUNDRED);
        }
        List<KpiGroupIndicatorParamVO> kpiGroupIndicatorParamVos = kpiGroupIndicatorParamService.getKpiGroupIndicatorParamVos(kpiGroupIndicator.getId());
        KpiGroupIndicatorParamVO paramVO = kpiGroupIndicatorParamVos.stream().findFirst().get();
        //有目标 直接乘100
        if (! TargetTypeEnum.NO.equals(paramVO.getTargetType())) {
            return value.multiply(Constant.ONE_HUNDRED);
        }
        if (DataTypeEnum.RATIO.equals(paramVO.getDataType())) {
            return value.multiply(Constant.ONE_HUNDRED);
        }
        return value;
    }

    public BigDecimal calcAverageValue(List<ReportKpiValue> userKpiValues, Set<Long> shopDimReportIds, String code) {
        Double averageRatioDouble = userKpiValues.stream()
                .filter(user -> shopDimReportIds.contains(user.getReportKpiDimId()) && code.equals(user.getCode()))
                .mapToDouble(userValue -> Optional.ofNullable(userValue.getValue()).orElse(BigDecimal.ZERO).doubleValue())
                .average()
                .orElse(0);
        return new BigDecimal(averageRatioDouble.toString());
    }
    public BigDecimal calcAverageKpiScoreRatio(List<ReportKpiDimValue> userKpiValues, Set<Long> shopDimReportIds) {
        Double averageRatioDouble = userKpiValues.stream()
                .filter(user -> shopDimReportIds.contains(user.getId()))
                .mapToDouble(userValue -> Optional.ofNullable(userValue.getKpiScoreRatio()).orElse(BigDecimal.ZERO).doubleValue())
                .average()
                .orElse(0);
        return new BigDecimal(averageRatioDouble.toString());
    }

    public Long saveShopDimDB(Long shopId, Map<Long, String> shopMap,
                                           LocalDate date, Long reportKpiRankId, BigDecimal kpiScoreRatio) {
        ReportKpiDimValue reportKpiDimValue = new ReportKpiDimValue();
        reportKpiDimValue.setDimension(ReportDimensionEnum.SHOP);
        reportKpiDimValue.setReferId(shopId);
        reportKpiDimValue.setShopId(shopId);
        reportKpiDimValue.setShopName(shopMap.get(shopId));
        reportKpiDimValue.setDataDate(date);
        reportKpiDimValue.setReportKpiRankId(reportKpiRankId);
        reportKpiDimValue.setKpiScoreRatio(kpiScoreRatio);
        reportKpiDimValueService.save(reportKpiDimValue);
        return reportKpiDimValue.getId();
    }

    public Long saveManagerDimDB(ManagerDTO manager, Map<Long, StaffBaseInfoDTO> managerMap,
                                              LocalDate date, Long reportKpiRankId, List<Long> manageStaffIds, BigDecimal kpiScoreRatio) {
        ReportKpiDimValue reportKpiDimValue = new ReportKpiDimValue();
        reportKpiDimValue.setDimension(ReportDimensionEnum.MANAGER);
        reportKpiDimValue.setDataDate(date);
        reportKpiDimValue.setReportKpiRankId(reportKpiRankId);
        reportKpiDimValue.setUserId(manager.getStaffId());
        reportKpiDimValue.setUserName(manager.getStaffName());
        reportKpiDimValue.setKpiScoreRatio(kpiScoreRatio);
        if (managerMap.containsKey(manager.getStaffId())) {
            StaffBaseInfoDTO staffInfo = managerMap.get(manager.getStaffId());
            reportKpiDimValue.setShopId(staffInfo.getShopId());
            reportKpiDimValue.setShopName(staffInfo.getShopName());
            reportKpiDimValue.setPostId(staffInfo.getPostId());
            reportKpiDimValue.setPostName(staffInfo.getPostName());
        }
        reportKpiDimValueService.save(reportKpiDimValue);

        List<String> roleCodes = manager.getScopeList().stream()
                .flatMap(managerStaffVo -> managerStaffVo.getRoleList().stream())
                .map(ManagerStaffRoleDTO::getRoleCode)
                .distinct()
                .collect(Collectors.toList());
        List<String> roleNames = manager.getScopeList().stream()
                .flatMap(managerStaffVo -> managerStaffVo.getRoleList().stream())
                .map(ManagerStaffRoleDTO::getRoleName)
                .distinct()
                .collect(Collectors.toList());
        List<Long> shopIds = manager.getScopeList().stream()
                .flatMap(managerStaffVo -> managerStaffVo.getShopList().stream())
                .map(ManagerStaffShopDTO::getShopId)
                .distinct()
                .collect(Collectors.toList());

        ReportKpiDimValueExtra dimExtra = ReportKpiDimValueExtra.builder()
                .reportKpiDimValueId(reportKpiDimValue.getId())
                .roleCodes(roleCodes)
                .roleNames(roleNames)
                .shopIds(shopIds)
                .staffIds(manageStaffIds)
                .build();
        reportKpiDimValueExtraService.save(dimExtra);

        return reportKpiDimValue.getId();
    }

    public ReportKpiValue convertShopValueDB(KpiGroupIndicator kpiGroupIndicator, BigDecimal value, Long shopReportKpiDimId, LocalDate date) {
        ReportKpiValue reportKpiValue = new ReportKpiValue();
        reportKpiValue.setCode(kpiGroupIndicator.getCode());
        reportKpiValue.setCodeType(kpiGroupIndicator.getCodeType());
        reportKpiValue.setValue(value);
        reportKpiValue.setDataDate(date);
        reportKpiValue.setReportKpiDimId(shopReportKpiDimId);
        reportKpiValue.setKpiGroupIndicatorId(kpiGroupIndicator.getId());
        return reportKpiValue;
    }

    public ReportKpiValue convertShopValueDBPcode(KpiGroupIndicatorParam kpiGroupIndicatorParam, BigDecimal value, Long shopReportKpiDimId, LocalDate date,KpiGroupIndicator indicator) {
        ReportKpiValue reportKpiValue = new ReportKpiValue();
        reportKpiValue.setCode(kpiGroupIndicatorParam.getIndicatorCode());
        reportKpiValue.setCodeType(IndicatorCodeTypeEnum.INDICATOR);
        reportKpiValue.setValue(value);
        reportKpiValue.setDataDate(date);
        reportKpiValue.setReportKpiDimId(shopReportKpiDimId);
        reportKpiValue.setKpiGroupIndicatorId(kpiGroupIndicatorParam.getKpiGroupIndicatorId());
        reportKpiValue.setParentCode(indicator.getCode());
        return reportKpiValue;
    }


    public ReportKpiValue convertDB(KpiGroupIndicator kpiGroupIndicator, Long reportKpiDimId, LocalDate date, BigDecimal kpiValue) {
        ReportKpiValue reportKpiValue = new ReportKpiValue();
        //kpi_group_indicator_hit_log 表 UNIQUE KEY `salary_project_hit_log_index` (`kpi_group_indicator_id`,`data_date`,`user_id`) USING BTREE
        reportKpiValue.setCode(kpiGroupIndicator.getCode());
        reportKpiValue.setCodeType(kpiGroupIndicator.getCodeType());
        reportKpiValue.setValue(kpiValue);
//        if (IndicatorCodeTypeEnum.COMBINE_INDICATOR.equals(kpiGroupIndicator.getCodeType())){
//            //组合指标暂时处理成百分比
//            reportKpiValue.setValue(kpiValue.multiply(Constant.ONE_HUNDRED));
//        }else {
//            List<KpiGroupIndicatorParamVO> kpiGroupIndicatorParamVos = kpiGroupIndicatorParamService.getKpiGroupIndicatorParamVos(kpiGroupIndicator.getId());
//            Optional<KpiGroupIndicatorParamVO> kpiGroupIndicatorParamVO = kpiGroupIndicatorParamVos.stream().filter(e-> kpiGroupIndicator.getCode().equals(e.getIndicatorCode())).findFirst();
//            if (kpiGroupIndicatorParamVO.isPresent()){
//                DataTypeEnum dataType = kpiGroupIndicatorParamVO.get().getDataType();
//                if (DataTypeEnum.RATIO.equals(dataType)){
//                    reportKpiValue.setValue(kpiValue.multiply(Constant.ONE_HUNDRED));
//                }
//            }
//        }
        reportKpiValue.setDataDate(date);
        reportKpiValue.setReportKpiDimId(reportKpiDimId);
        reportKpiValue.setKpiGroupIndicatorId(kpiGroupIndicator.getId());
        return reportKpiValue;
    }

    public ReportKpiValue convertNoTargetDB(KpiGroupIndicator kpiGroupIndicator,
                                            KpiGroupIndicatorParamVO kpiGroupIndicatorParam,
                                            LocalDate date,
                                            Long reportKpiDimId,
                                            BigDecimal value) {
        ReportKpiValue reportKpiValue = new ReportKpiValue();
        reportKpiValue.setCode(kpiGroupIndicatorParam.getIndicatorCode());
        reportKpiValue.setCodeType(IndicatorCodeTypeEnum.INDICATOR);
        reportKpiValue.setParentCode(kpiGroupIndicator.getCode());
        reportKpiValue.setValue(value);
        //百分数处理
        if (DataTypeEnum.RATIO.equals(kpiGroupIndicatorParam.getDataType())){
            reportKpiValue.setValue(value.multiply(Constant.ONE_HUNDRED));
        }
        reportKpiValue.setDataDate(date);
        reportKpiValue.setReportKpiDimId(reportKpiDimId);
        reportKpiValue.setKpiGroupIndicatorId(kpiGroupIndicator.getId());
        return reportKpiValue;
    }
    public ReportKpiValue convertTargetDB(KpiGroupIndicator kpiGroupIndicator,
                                            KpiGroupIndicatorParamVO kpiGroupIndicatorParam,
                                            LocalDate date,
                                            Long reportKpiDimId,
                                            BigDecimal value) {
        ReportKpiValue reportKpiValue = new ReportKpiValue();
        reportKpiValue.setCode(kpiGroupIndicatorParam.getIndicatorCode());
        reportKpiValue.setCodeType(IndicatorCodeTypeEnum.INDICATOR);
        reportKpiValue.setParentCode(kpiGroupIndicator.getCode());
        //有目标 乘以 100
        reportKpiValue.setValue(value.multiply(Constant.ONE_HUNDRED));
        reportKpiValue.setDataDate(date);
        reportKpiValue.setReportKpiDimId(reportKpiDimId);
        reportKpiValue.setKpiGroupIndicatorId(kpiGroupIndicator.getId());
        return reportKpiValue;
    }

    public MultiKeyMap<Long, BigDecimal> getUserIndicatorHitMap(List<KpiGroupIndicator> kpiGroupIndicators, LocalDate date, List<KpiPool> pools) {
        List<KpiGroupIndicatorHitLog> indicatorHitLogs = kpiGroupIndicatorHitLogService.list(Wrappers.<KpiGroupIndicatorHitLog>lambdaQuery()
                .in(KpiGroupIndicatorHitLog::getKpiGroupIndicatorId, kpiGroupIndicators.stream().map(KpiGroupIndicator::getId).collect(Collectors.toList()))
                .in(KpiGroupIndicatorHitLog::getUserId, pools.stream().map(KpiPool::getUserId).collect(Collectors.toList()))
                .eq(KpiGroupIndicatorHitLog::getDataDate, date)
                .eq(KpiGroupIndicatorHitLog::getYn, Boolean.TRUE)
        );
        MultiKeyMap<Long, BigDecimal> userGroupIndicatorHitMap = new MultiKeyMap<>();
        for (KpiGroupIndicatorHitLog hitLog : indicatorHitLogs) {
            userGroupIndicatorHitMap.put(hitLog.getUserId(), hitLog.getKpiGroupIndicatorId(), hitLog.getKpiValue());
        }
        return userGroupIndicatorHitMap;
    }



    public Map<Long, KpiGroupIndicatorTargetHitLog> queryUserTargetValue(Long referId, Long userId, LocalDate dataDate) {
        List<KpiGroupIndicatorTargetHitLog> targetHitLogs = kpiGroupIndicatorTargetHitLogService.list(Wrappers.<KpiGroupIndicatorTargetHitLog>lambdaQuery()
                .eq(KpiGroupIndicatorTargetHitLog::getReferId, referId)
                .eq(KpiGroupIndicatorTargetHitLog::getTargetType, IndicatorTypeEnum.EXAMINE)
                .eq(KpiGroupIndicatorTargetHitLog::getDataDate, dataDate)
                .eq(KpiGroupIndicatorTargetHitLog::getUserId, userId)
                .eq(KpiGroupIndicatorTargetHitLog::getYn, Boolean.TRUE)
        );
        return targetHitLogs.stream()
                .collect(Collectors.toMap(KpiGroupIndicatorTargetHitLog::getUserId, Function.identity(), (v1, v2) -> v1));
    }



    public Map<Long, BigDecimal> queryUserOriginValue(String code, Long userId, LocalDate dataDate) {
        Map<Long, BigDecimal> referValueMap = Maps.newHashMapWithExpectedSize(1);
        //无目标
        LambdaQueryWrapper<IndicatorUserValue> queryWrapper = Wrappers.<IndicatorUserValue>lambdaQuery();
        queryWrapper.eq(IndicatorUserValue::getUserId, userId);
        List<IndicatorUserValue> indicatorUserValues = indicatorUserValueService.list(queryWrapper
                .eq(IndicatorUserValue::getIndicatorCode, code)
                .eq(IndicatorUserValue::getDataDate, dataDate)
                .eq(IndicatorUserValue::getDimensionType, DimensionTypeEnum.STAFF)
                .eq(IndicatorUserValue::getYn, Boolean.TRUE)
        );
        for (IndicatorUserValue userValue : indicatorUserValues) {
            BigDecimal indicatorValue = commonService.queryIndicatorValue(code, userValue);
            referValueMap.put(userValue.getUserId(), indicatorValue);
            //如果原始值是百分比,乘以100,由于业务系统这个可能不准还是配置指标里面的data_type
            //if (IndicatorValueTypeEnum.RATIO.equals(userValue.getValueType())){
            //    referValueMap.put(userValue.getUserId(), indicatorValue.multiply(Constant.ONE_HUNDRED));
            //}
        }
        return referValueMap;
    }

    private List<ReportKpiRankCode> saveReportKpiRankCodes(LocalDate date, ReportKpiRank reportKpiRank, List<KpiGroupIndicator> kpiGroupIndicators) {
        List<ReportKpiRankCode> saveReportKpiRankCode = new ArrayList<>(kpiGroupIndicators.size());
        AtomicInteger order = new AtomicInteger();
        Set<String> codes = new HashSet<>();
        for (int i = 0; i < kpiGroupIndicators.size(); i++) {
            KpiGroupIndicator kpiGroupIndicator = kpiGroupIndicators.get(i);
            //一个编码只能展示一次
            if (! codes.add(kpiGroupIndicator.getCode())) {
                continue;
            }
            ReportKpiRankCode reportKpiRankCode = new ReportKpiRankCode();
            reportKpiRankCode.setReportKpiRankId(reportKpiRank.getId());
            reportKpiRankCode.setKpiGroupIndicatorId(kpiGroupIndicator.getId());
            reportKpiRankCode.setCode(kpiGroupIndicator.getCode());
            reportKpiRankCode.setCodeType(kpiGroupIndicator.getCodeType());
            reportKpiRankCode.setIndicatorName(kpiGroupIndicator.getName());
            reportKpiRankCode.setOrderNum(order.incrementAndGet());
            reportKpiRankCode.setDataDate(date);
            saveReportKpiRankCode.add(reportKpiRankCode);
            if (IndicatorCodeTypeEnum.COMBINE_INDICATOR.equals(kpiGroupIndicator.getCodeType())) {
                List<KpiGroupIndicatorParam> kpiGroupIndicatorParams = kpiGroupIndicatorParamService.list(Wrappers.<KpiGroupIndicatorParam>lambdaQuery()
                        .eq(KpiGroupIndicatorParam::getKpiGroupIndicatorId, kpiGroupIndicator.getId())
                        .eq(KpiGroupIndicatorParam::getYn, Boolean.TRUE)
                );
                for (KpiGroupIndicatorParam kpiGroupIndicatorParam : kpiGroupIndicatorParams) {
                    ReportKpiRankCode reportKpiRankCode1 = new ReportKpiRankCode();
                    reportKpiRankCode1.setReportKpiRankId(reportKpiRank.getId());
                    reportKpiRankCode1.setKpiGroupIndicatorId(kpiGroupIndicator.getId());
                    reportKpiRankCode1.setCode(kpiGroupIndicatorParam.getIndicatorCode());
                    reportKpiRankCode1.setCodeType(IndicatorCodeTypeEnum.INDICATOR);
                    reportKpiRankCode1.setIndicatorName(kpiGroupIndicatorParam.getIndicatorName());
                    reportKpiRankCode1.setOrderNum(order.incrementAndGet());
                    reportKpiRankCode1.setDataDate(date);
                    reportKpiRankCode1.setParentCode(kpiGroupIndicator.getCode());
                    saveReportKpiRankCode.add(reportKpiRankCode1);
                }
            }
        }
        reportKpiRankCodeService.saveBatch(saveReportKpiRankCode);
        return saveReportKpiRankCode;
    }

    private ReportKpiRank saveReportKpiRank(KpiGroupRank kpiGroupRank, Long groupId, LocalDate date, List<KpiGroup> kpiGroups, List<Long> kpiGroupIds) {

        ReportKpiRank reportKpiRank = new ReportKpiRank();
        reportKpiRank.setRankId(kpiGroupRank.getId());
        reportKpiRank.setRankName(kpiGroupRank.getName());
        reportKpiRank.setKpiGroupIds(kpiGroupIds);
        //门店id
        reportKpiRank.setShopIds(kpiGroups.stream().map(KpiGroup::getShopIds).distinct().collect(ArrayList::new, List::addAll, List::addAll));
        reportKpiRank.setMonthly(YearMonth.from(date));
        reportKpiRank.setDataDate(date);
        reportKpiRank.setGroupId(groupId);
        //KpiGroupInfoDTO
        List<KpiGroupInfoDTO> kpiGroupInfoDTOS = kpiGroups.stream().map(k -> {
            KpiGroupInfoDTO dto = new KpiGroupInfoDTO();
            dto.setId(k.getId());
            dto.setKgc(k.getKgc());
            dto.setKgcName(k.getName());
            return dto;
        }).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(kpiGroupInfoDTOS)){
            kpiGroupInfoDTOS = Collections.emptyList();
        }
        reportKpiRank.setKpiGroupInfo(JSON.toJSONString(kpiGroupInfoDTOS));
        reportKpiRankService.save(reportKpiRank);
        return reportKpiRank;
    }


    private List<KpiGroupRank> queryNeedRankGroup(LocalDate date) {
        List<KpiGroupRank> kpiGroupRanks = kpiGroupRankService.list(Wrappers.<KpiGroupRank>lambdaQuery()
                .eq(KpiGroupRank::getYn, Boolean.TRUE)
        );
        return  kpiGroupRanks;
    }


    private void cleanData(LocalDate endDate) {
        List<ReportKpiRank> reportKpiRanks = reportKpiRankService.list(Wrappers.<ReportKpiRank>lambdaQuery()
//                .eq(ReportKpiRank::getRankId, kpiGroupRank.getId())
                //.ge(ReportKpiRank::getDataDate, startDate.with(TemporalAdjusters.firstDayOfMonth()))
                //取当前日期对应月的第一天
                .ge(ReportKpiRank::getDataDate, endDate.with(TemporalAdjusters.firstDayOfMonth()))
                .le(ReportKpiRank::getDataDate, endDate)
        );
        if (CollectionUtils.isEmpty(reportKpiRanks)){
            return;
        }
        kpiGroupRankBizService.cleanRankData(reportKpiRanks);
    }



    public List<KpiGroup> getKpiGroups(List<Long> kpiGroupIds, LocalDate date) {
        if (PublicUtil.isEmpty(kpiGroupIds)) {
            return new ArrayList<>();
        }
        List<Integer> effectStatus = new ArrayList<Integer>(){{
            add(SettingStatusEnum.EFFECTIVE.getValue());
            add(SettingStatusEnum.INEFFECTIVE.getValue());
        }};
        List<KpiGroup> kpiGroups = kpiGroupService.list(Wrappers.<KpiGroup>lambdaQuery()
                .in(KpiGroup::getId, kpiGroupIds)
                .eq(KpiGroup::getYn, Boolean.TRUE)
                .in(KpiGroup::getStatus, effectStatus)
                .le(KpiGroup::getBeginTime, date)
                .and(wrapper -> wrapper.ge(KpiGroup::getOverTime, date).or().isNull(KpiGroup::getOverTime))
        );
        return kpiGroups;
    }

    public Map<Long, StaffBaseInfoDTO> staffInfoMap(List<ManagerDTO> managerDTOS) {
        List<Long> staffIds = managerDTOS.stream().map(ManagerDTO::getStaffId).collect(Collectors.toList());
        List<StaffBaseInfoDTO> staffInfos = ehrRpcService.queryStaffBaseInfo(staffIds);
        return staffInfos.stream().collect(Collectors.toMap(StaffBaseInfoDTO::getId, Function.identity(), (v1, v2) -> v1));
    }

    public ReportKpiDimValue getDim(KpiPool pool, LocalDate date, Long reportKpiRankId){
        ReportKpiDimValue reportKpiDimValue = new ReportKpiDimValue();
        reportKpiDimValue.setDimension(ReportDimensionEnum.STAFF);
        reportKpiDimValue.setReferId(pool.getUserId());
        reportKpiDimValue.setKpiPoolId(pool.getId());
        reportKpiDimValue.setUserId(pool.getUserId());
        reportKpiDimValue.setUserName(pool.getUserName());
        reportKpiDimValue.setPostId(pool.getPostId());
        reportKpiDimValue.setPostName(pool.getPostName());
        reportKpiDimValue.setShopId(pool.getShopId());
        reportKpiDimValue.setShopName(pool.getShopName());
        reportKpiDimValue.setDataDate(date);
        reportKpiDimValue.setReportKpiRankId(reportKpiRankId);
        reportKpiDimValue.setInclusion(pool.getInclusion());
        BigDecimal kpiScoreRatio = Optional.ofNullable(pool.getKpiScoreRatio()).orElse(BigDecimal.ZERO);
        reportKpiDimValue.setKpiScoreRatio(kpiScoreRatio.multiply(Constant.ONE_HUNDRED));
        return reportKpiDimValue;
    }

}
