package cn.fw.morax.service.biz.calculator.salary;

import cn.fw.morax.common.config.SpecialIndicatorCodeVal;
import cn.fw.morax.common.utils.StringUtils;
import cn.fw.morax.domain.db.kpi.IndicatorUserValue;
import cn.fw.morax.domain.db.kpi.KpiPool;
import cn.fw.morax.domain.db.salary.*;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.service.biz.calculator.Calculator;
import cn.fw.morax.service.biz.salary.SalaryPoolCommonService;
import cn.fw.morax.service.data.kpi.IndicatorUserValueService;
import cn.fw.morax.service.data.kpi.KpiPoolService;
import cn.fw.morax.service.data.salary.*;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.lang.NonNull;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Consumer;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryBaseCalculator
 * @description : 基础薪酬计算器
 * @date : 2022-04-18 11:29
 */
@Slf4j
public abstract class SalaryBaseCalculator implements Calculator<SalaryGroupProject, SalaryGroupUser, BigDecimal> {
    @Autowired
    protected SalaryGroupProjectSettinService salaryGroupProjectSettinService;
    @Autowired
    protected SalaryGroupUserService salaryGroupUserService;
    @Autowired
    protected SalaryGroupProjectHitLogService salaryGroupProjectHitLogService;
    @Autowired
    protected SalaryGroupProjectTargetHitLogService salaryGroupProjectTargetHitLogService;
    @Autowired
    protected SalaryGroupProjectPreconditionLogService salaryGroupProjectPreconditionLogService;
    @Autowired
    protected SalaryGroupProjectParamService salaryGroupProjectParamService;
    @Autowired
    protected SalaryGroupProjectPreconditionService salaryGroupProjectPreconditionService;
    @Autowired
    protected SalaryPoolCommonService salaryPoolCommonService;
    @Autowired
    protected IndicatorUserValueService indicatorUserValueService;
    @Autowired
    protected SpecialIndicatorCodeVal specialIndicatorCodeVal;
    @Autowired
    protected KpiPoolService kpiPoolService;

    @Value("${special-indicator-code.work-age}")
    @Getter
    private String workAgeCode;
    @Value("${special-indicator-code.work-day-proportion}")
    @Getter
    private String workDayProportion;

    /**
     * 获取计算类型
     *
     * @return 计算类型
     */
    public abstract SalaryCalMethodEnum getCalMethod();

    /**
     * 计算动态类型的金额
     *
     * @param pool                薪酬池
     * @param date                日期
     * @param extraSalaryTypeEnum 动态薪酬类型
     * @return 薪酬池详情
     */
    public SalaryPoolDetail calcDynamicMoney(@NonNull SalaryPool pool, @NonNull LocalDate date, @NonNull ExtraSalaryTypeEnum extraSalaryTypeEnum) {
        return null;
    }

    /**
     * 记录命中配置
     *
     * @param salaryGroupProjectId 指标配置id
     * @param user 人员信息
     * @param consumer
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveProjectHitLog(Long salaryGroupProjectId, SalaryGroupUser user, Consumer<SalaryGroupProjectHitLog> consumer) {
        final Long userId = user.getUserId();
        final LocalDate dataDate = user.getDataDate();
        final Long groupId = user.getGroupId();
        SalaryGroupProjectHitLog hitLog = SalaryGroupProjectHitLog.builder()
                .salaryGroupProjectId(salaryGroupProjectId)
                .userId(userId)
                .salaryGroupProjectSettinId(-1L)
                .salaryValue(BigDecimal.ZERO)
                .hitCommissionValue(BigDecimal.ZERO)
                .dataDate(dataDate)
                .groupId(groupId)
                .build();
        if (consumer != null) {
            consumer.accept(hitLog);
        }
        salaryGroupProjectHitLogService.save(hitLog);
    }

    /**
     * 清除project记录命中配置
     *
     * @param salaryGroupProjectId 指标配置id
     * @param user 人员信息
     */
    @Transactional(rollbackFor = Exception.class)
    public void clearProjectHitLog(Long salaryGroupProjectId, SalaryGroupUser user) {
        final Long userId = user.getUserId();
        final LocalDate dataDate = user.getDataDate();
        final Long groupId = user.getGroupId();
        salaryGroupProjectHitLogService.remove(Wrappers.<SalaryGroupProjectHitLog>lambdaQuery()
                .eq(SalaryGroupProjectHitLog::getUserId, userId)
                .eq(SalaryGroupProjectHitLog::getSalaryGroupProjectId, salaryGroupProjectId)
                .eq(SalaryGroupProjectHitLog::getDataDate, dataDate)
                .eq(SalaryGroupProjectHitLog::getGroupId, groupId)
        );
    }

    /**
     * 记录目标命中
     *
     * @param referId       id
     * @param indicatorType 指标类型
     * @param value         原始值
     * @param reachValue    达成目标后的值
     * @param user          人员信息
     */
    @Transactional(rollbackFor = Exception.class)
    public void saveTargetHitLog(Long referId, IndicatorTypeEnum indicatorType, BigDecimal value, BigDecimal reachValue, SalaryGroupUser user) {
        final Long userId = user.getUserId();
        final LocalDate dataDate = user.getDataDate();
        final Long groupId = user.getGroupId();

        salaryGroupProjectTargetHitLogService.remove(Wrappers.<SalaryGroupProjectTargetHitLog>lambdaQuery()
                .eq(SalaryGroupProjectTargetHitLog::getUserId, userId)
                .eq(SalaryGroupProjectTargetHitLog::getReferId, referId)
                .eq(SalaryGroupProjectTargetHitLog::getTargetType, indicatorType)
                .eq(SalaryGroupProjectTargetHitLog::getDataDate, dataDate)
                .eq(SalaryGroupProjectTargetHitLog::getGroupId, groupId)
        );

        SalaryGroupProjectTargetHitLog log = new SalaryGroupProjectTargetHitLog();
        log.setReferId(referId);
        log.setTargetType(indicatorType);
        log.setUserId(userId);
        log.setValue(value);
        log.setReachValue(reachValue);
        log.setDataDate(dataDate);
        log.setGroupId(groupId);
        log.setYn(Boolean.TRUE);
        salaryGroupProjectTargetHitLogService.save(log);
    }


    /**
     * 查询薪酬指标详细配置
     *
     * @param salaryGroupProjectId 薪资组指标配置id
     * @return
     */
    protected List<SalaryGroupProjectSettin> queryProjectSettins(final Long salaryGroupProjectId) {
        List<SalaryGroupProjectSettin> list = salaryGroupProjectSettinService.list(Wrappers.<SalaryGroupProjectSettin>lambdaQuery()
                .eq(SalaryGroupProjectSettin::getSalaryGroupProjectId, salaryGroupProjectId)
                .eq(SalaryGroupProjectSettin::getYn, Boolean.TRUE));
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list;
    }

    /**
     * 查询薪酬指标项前置条件
     *
     * @param salaryGroupProjectId 薪资组指标配置id
     * @return
     */
    protected List<SalaryGroupProjectPrecondition> queryPrecondition(final Long salaryGroupProjectId) {
        List<SalaryGroupProjectPrecondition> list = salaryGroupProjectPreconditionService.list(Wrappers.<SalaryGroupProjectPrecondition>lambdaQuery()
                .eq(SalaryGroupProjectPrecondition::getSalaryGroupProjectId, salaryGroupProjectId)
                .eq(SalaryGroupProjectPrecondition::getYn, Boolean.TRUE)
                .orderByAsc(SalaryGroupProjectPrecondition::getSort)
        );
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list;
    }

    /**
     * 校验前置条件
     *
     * @param list
     * @return
     */
    protected boolean examinePrecondition(List<SalaryGroupProjectPrecondition> list, SalaryGroupUser user) {
        if (CollectionUtils.isEmpty(list)) {
            return Boolean.TRUE;
        }
        for (SalaryGroupProjectPrecondition precondition : list) {
            String indicatorCode = precondition.getIndicatorCode();
            JSONObject jsonObject = queryUserIndicatorValue(user, user.getDataDate(), indicatorCode);
            final BigDecimal indicatorValue = Optional.ofNullable(jsonObject.getBigDecimal(indicatorCode)).orElse(BigDecimal.ZERO);
            BigDecimal userValue = indicatorValue;
            TargetTypeEnum targetType = precondition.getTargetType();
            final BigDecimal condValue = precondition.getCondValue();
            if (!TargetTypeEnum.NO.equals(targetType)) {
                TargetCalcTypeEnum targetCalcType = precondition.getTargetCalcType();
                BigDecimal targetValue = precondition.getTargetValue();
                userValue = calculateTargetValue(targetCalcType, targetValue, userValue);
                saveTargetHitLog(precondition.getId(), IndicatorTypeEnum.PRE, indicatorValue, userValue, user);
            }
            if (userValue.compareTo(condValue) < 0) {
                return Boolean.FALSE;
            }
        }
        return Boolean.TRUE;
    }

    /**
     * 经过目标计算后的用户指标值
     *
     * @param targetCalcType
     * @param targetValue
     * @param userValue
     * @return
     */
    protected BigDecimal calculateTargetValue(TargetCalcTypeEnum targetCalcType, BigDecimal targetValue, BigDecimal userValue) {
        if (Objects.isNull(targetValue) || targetValue.compareTo(BigDecimal.ZERO) <= 0) {
            return userValue;
        }
        if (TargetCalcTypeEnum.TARGET_VALUE.equals(targetCalcType)) {
            return userValue.divide(targetValue, 4, RoundingMode.HALF_UP);
        }
        BigDecimal diffValue = BigDecimal.ONE.subtract(targetValue);
        BigDecimal userDiffValue = userValue.subtract(targetValue);
        if (userDiffValue.compareTo(BigDecimal.ZERO) <= 0) {
            return BigDecimal.ZERO;
        }
        if (diffValue.compareTo(BigDecimal.ZERO) <= 0) {
            return userDiffValue;
        }
        return userDiffValue.divide(diffValue, 4, RoundingMode.HALF_UP);
    }

    /**
     * 查询用户指标值
     *
     * @param user
     * @param date
     * @param indicatorCode
     * @return
     */
    protected JSONObject queryUserIndicatorValue(SalaryGroupUser user, LocalDate date, String indicatorCode) {
        JSONObject jsonObject = new JSONObject();
        Long userId = user.getUserId();
        if (specialIndicatorCodeVal.getKpiRatio().equals(indicatorCode)) {
            KpiPool kpiPool = kpiPoolService.getOne(Wrappers.<KpiPool>lambdaQuery()
                            .eq(KpiPool::getUserId, userId)
                            .eq(KpiPool::getPostId, user.getPostId())
                            .eq(KpiPool::getShopId, user.getShopId())
                            .eq(KpiPool::getMonthly, YearMonth.from(date))
                            .eq(KpiPool::getYn, Boolean.TRUE)
                    , Boolean.FALSE);
            BigDecimal averageKpiScoreRatio = Optional.ofNullable(kpiPool).map(KpiPool::getAverageKpiScoreRatio).orElse(BigDecimal.ZERO);
            jsonObject.put(specialIndicatorCodeVal.getKpiRatio(), averageKpiScoreRatio);
            return jsonObject;
        }

        IndicatorUserValue userValue = indicatorUserValueService.getOne(Wrappers.<IndicatorUserValue>lambdaQuery()
                        .eq(IndicatorUserValue::getUserId, userId)
                        .eq(IndicatorUserValue::getIndicatorCode, indicatorCode)
                        .eq(IndicatorUserValue::getDataDate, date)
                        .eq(IndicatorUserValue::getYn, Boolean.TRUE),
                Boolean.FALSE
        );
        String value = Optional.ofNullable(userValue).map(IndicatorUserValue::getIndicatorValue).orElse(null);
        if (StringUtils.isEmpty(value)) {
            log.error("[{}]指标值为空", indicatorCode);
            return jsonObject;
        }
        try {
            jsonObject = JSONObject.parseObject(value);
        } catch (Exception e) {
            log.error("[{}]指标值转化失败", indicatorCode, e);
        }
        return jsonObject;
    }

    /**
     * 查询提成指标项
     *
     * @param salaryGroupProjectId
     * @param paramType
     * @return
     */
    protected List<SalaryGroupProjectParam> queryProjectParam(final Long salaryGroupProjectId, final ParamTypeEnum paramType) {
        List<SalaryGroupProjectParam> list = salaryGroupProjectParamService.list(Wrappers.<SalaryGroupProjectParam>lambdaQuery()
                .eq(SalaryGroupProjectParam::getSalaryGroupProjectId, salaryGroupProjectId)
                .eq(SalaryGroupProjectParam::getParamType, paramType)
                .eq(SalaryGroupProjectParam::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(list)) {
            return Collections.emptyList();
        }
        return list;
    }

    /**
     * 计算用户最终值
     *
     * @param projectParamList
     * @param user
     * @return
     */
    protected BigDecimal calculateParamValue(List<SalaryGroupProjectParam> projectParamList, SalaryGroupUser user) {
        BigDecimal commissionValue = BigDecimal.ZERO;
        for (SalaryGroupProjectParam projectParam : projectParamList) {
            String indicatorCode = projectParam.getIndicatorCode();
            boolean isCap = Boolean.TRUE.equals(projectParam.getCap());
            BigDecimal proportion = projectParam.getProportion();
            TargetTypeEnum targetType = projectParam.getTargetType();
            ParamTypeEnum paramType = projectParam.getParamType();
            JSONObject jsonObject = queryUserIndicatorValue(user, user.getDataDate(), indicatorCode);
            final BigDecimal userOriginValue = Optional.ofNullable(jsonObject.getBigDecimal(indicatorCode)).orElse(BigDecimal.ZERO);
            BigDecimal _calcValue = userOriginValue;
            if (ParamTypeEnum.COMMISSION.equals(paramType) && indicatorCode.equals(getWorkAgeCode())) {
                _calcValue = _calcValue.multiply(Optional.ofNullable(jsonObject.getBigDecimal(getWorkDayProportion())).orElse(BigDecimal.ZERO));
            }
            if (!TargetTypeEnum.NO.equals(targetType)) {
                _calcValue = calculateTargetValue(projectParam.getTargetCalcType(), projectParam.getTargetValue(), _calcValue);
                saveTargetHitLog(projectParam.getId(), IndicatorTypeEnum.EXAMINE, userOriginValue, _calcValue, user);
            }
            if (isCap) {
                _calcValue = _calcValue.compareTo(BigDecimal.ONE) > 0 ? BigDecimal.ONE : _calcValue;
            }
            commissionValue = commissionValue.add(proportion.multiply(_calcValue));
        }
        return commissionValue;
    }

}
