package cn.fw.morax.service.biz.eval;

import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.StringUtils;
import cn.fw.morax.common.utils.ThreadPoolUtil;
import cn.fw.morax.domain.bo.eval.EvalGroupCalculableBO;
import cn.fw.morax.domain.bo.eval.EvalGroupUserShop;
import cn.fw.morax.domain.bo.kpi.EvalGroupCalculableRankBO;
import cn.fw.morax.domain.db.eval.*;
import cn.fw.morax.domain.enums.*;
import cn.fw.morax.service.biz.calculator.eval.kpi.EvalKpiBaseCalculator;
import cn.fw.morax.service.biz.calculator.eval.reward.EvalRewardBaseCalculator;
import cn.fw.morax.service.data.eval.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.*;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
 * @author jiangchao
 * @des: 考评计算
 * @date 2023/1/12 17:39
 */
@Slf4j
@Service
public class EvalCalculateService {

    private final EvalPoolRewardDetailService evalPoolRewardDetailService;
    private final EvalPoolCommonService evalPoolCommonService;
    private final EvalUserPoolService evalUserPoolService;
    private final EvalShopPoolService evalShopPoolService;
    private final EvalGroupService evalGroupService;
    private final EvalGroupRankStageService evalGroupRankStageService;
    private final EvalGroupIndicatorService evalGroupIndicatorService;
    private final EvalPoolIndicatorDetailService evalPoolIndicatorDetailService;
    private final EvalGroupRewardService evalGroupRewardService;
    private final EvalGroupRewardDimService evalGroupRewardDimService;
    private final EvalGroupUserService evalGroupUserService;
    private final StringRedisTemplate stringRedisTemplate;
    private final EvalGroupRankService evalGroupRankService;
    private final PlatformTransactionManager platformTransactionManager;
    private final TransactionDefinition transactionDefinition;
    private final Map<EvalRewardCalMethodEnum, EvalRewardBaseCalculator> calculatorRewardMap;
    private final Map<ScoreWayEnum, EvalKpiBaseCalculator> calculatorEvalMap;

    public EvalCalculateService(final EvalPoolRewardDetailService evalPoolRewardDetailService,
                                final EvalPoolCommonService evalPoolCommonService,
                                final EvalUserPoolService evalUserPoolService,
                                final EvalShopPoolService evalShopPoolService,
                                final EvalGroupService evalGroupService,
                                final EvalGroupIndicatorService evalGroupIndicatorService,
                                final EvalPoolIndicatorDetailService evalPoolIndicatorDetailService,
                                final EvalGroupRewardService evalGroupRewardService,
                                final EvalGroupUserService evalGroupUserService,
                                final StringRedisTemplate stringRedisTemplate,
                                final EvalGroupRankService evalGroupRankService,
                                final PlatformTransactionManager platformTransactionManager,
                                final TransactionDefinition transactionDefinition,
                                final EvalGroupRankStageService evalGroupRankStageService,
                                final EvalGroupRewardDimService evalGroupRewardDimService,
                                final List<EvalRewardBaseCalculator> rewardBaseCalculators,
                                final List<EvalKpiBaseCalculator> kpiBaseCalculators) {
        this.evalPoolRewardDetailService = evalPoolRewardDetailService;
        this.evalPoolCommonService = evalPoolCommonService;
        this.evalUserPoolService = evalUserPoolService;
        this.evalShopPoolService = evalShopPoolService;
        this.evalGroupService = evalGroupService;
        this.evalGroupIndicatorService = evalGroupIndicatorService;
        this.evalPoolIndicatorDetailService = evalPoolIndicatorDetailService;
        this.evalGroupRewardService = evalGroupRewardService;
        this.evalGroupUserService = evalGroupUserService;
        this.stringRedisTemplate = stringRedisTemplate;
        this.evalGroupRankService = evalGroupRankService;
        this.platformTransactionManager = platformTransactionManager;
        this.transactionDefinition = transactionDefinition;
        this.evalGroupRankStageService = evalGroupRankStageService;
        this.evalGroupRewardDimService = evalGroupRewardDimService;
        this.calculatorRewardMap = rewardBaseCalculators.stream().collect(Collectors.toMap(EvalRewardBaseCalculator::getCalMethod, v -> v));
        this.calculatorEvalMap = kpiBaseCalculators.stream().collect(Collectors.toMap(EvalKpiBaseCalculator::getCalMethod, v -> v));
    }

    @Value("${spring.cache.custom.global-prefix}:calculable:eval-group")
    @Getter
    private String calcEvalGroup;

    @Value("${spring.cache.custom.global-prefix}:eval:calculate-rank")
    @Getter
    private String calcEvalRankKey;

    /**
     * 缓存需要计算绩效的绩效组信息
     *
     * @param localDate 日期
     */
    public void cacheCalculableEvalGroupIds(final LocalDate localDate) {
        log.info("定时任务【缓存需要计算的考评组】开始执行");
        List<EvalGroup> evalGroups = evalGroupService.getAllEffectGroups(localDate);
        Set<Long> evalGroupIds = evalGroups.stream().map(EvalGroup::getId).collect(Collectors.toSet());
        if (CollectionUtils.isEmpty(evalGroupIds)) {
            return;
        }
        log.info("缓存需要计算的考评组信息，evalGroupIds：{}，日期：{}", JSON.toJSONString(evalGroupIds), localDate);
        cacheGroupIds(evalGroupIds, localDate);
    }


    private void cacheGroupIds(final Set<Long> evalGroupIds, final LocalDate localDate) {
        String[] array = evalGroupIds.stream()
                .map(evalGroupId -> new EvalGroupCalculableBO(evalGroupId, localDate))
                .map(JSONObject::toJSONString)
                .toArray(String[]::new);
        stringRedisTemplate.opsForSet().add(getCalcEvalGroup(), array);
    }


    public void calculateCacheEvalGroup() {
        BoundSetOperations<String, String> setOps = stringRedisTemplate.boundSetOps(getCalcEvalGroup());
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        List<String> overflowsList = new ArrayList<>();
        String str;
        while ((str = setOps.pop()) != null) {
            final EvalGroupCalculableBO bo = JSONObject.parseObject(str, EvalGroupCalculableBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                String finalStr = str;
                threadPool.execute(() -> {
                    TransactionStatus transactionStatus = platformTransactionManager.getTransaction(transactionDefinition);
                    try {
                        calculateEvalGroup(bo, EvalScopeEnum.STAFF);
                        calculateEvalGroup(bo, EvalScopeEnum.SHOP);
                        platformTransactionManager.commit(transactionStatus);
                    } catch (Exception ex) {
                        platformTransactionManager.rollback(transactionStatus);
                        log.error("计算考评数据失败：{}", bo, ex);
                        setOps.add(finalStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsList.add(str);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsList)) {
            for (String s : overflowsList) {
                setOps.add(s);
            }
        }
    }

    public void calculateEvalGroup(EvalGroupCalculableBO bo, EvalScopeEnum scopeType) {
        log.info("考评组计算：{}", JSON.toJSONString(bo));
        final Long evalGroupId = bo.getEvalGroupId();
        final LocalDate localDate = bo.getLocalDate();

        EvalGroup evalGroup = evalGroupService.getById(evalGroupId);
        if (Objects.isNull(evalGroup)) {
            log.error("考评组[{}]不存在，终止计算！", evalGroupId);
            return;
        }
        EvalGroupRankStage evalGroupRankStage = evalGroupRankStageService.getById(evalGroup.getEvalGroupRankStageId());
        if (Objects.isNull(evalGroupRankStage)) {
            log.error("考评排名组阶段[{}]不存在，终止计算！", JSON.toJSONString(evalGroup));
            return;
        }
        List<EvalGroupUserShop> userShops = this.buildEvalGroupObj(evalGroup, evalGroupRankStage, localDate, scopeType);
        if (CollectionUtils.isEmpty(userShops)) {
            log.error("考评组[{}]人员为空，终止计算！", evalGroupId);
            return;
        }

        List<EvalGroupRewardDim> rewardDims = evalGroupRewardDimService.list(Wrappers.<EvalGroupRewardDim>lambdaQuery()
                .eq(EvalGroupRewardDim::getEvalGroupId, evalGroupId)
                .eq(EvalGroupRewardDim::getType, scopeType)
                .eq(EvalGroupRewardDim::getYn, Boolean.TRUE)
        );
        List<EvalGroupReward> rewards = evalGroupRewardService.list(Wrappers.<EvalGroupReward>lambdaQuery()
                .eq(EvalGroupReward::getEvalGroupId, evalGroupId)
                .eq(EvalGroupReward::getYn, Boolean.TRUE)
        );
        Map<Long, List<EvalGroupReward>> dimRewardMap = rewards.stream().collect(Collectors.groupingBy(EvalGroupReward::getEvalGroupDimId));
        List<EvalGroupIndicator> indicators = evalGroupIndicatorService.list(Wrappers.<EvalGroupIndicator>lambdaQuery()
                .eq(EvalGroupIndicator::getEvalGroupId, evalGroupId)
                .eq(EvalGroupIndicator::getYn, Boolean.TRUE)
        );

        List<EvalPoolRewardDetail> rewardDetails = new ArrayList<>();
        BigDecimal totalScore = indicators.stream()
                .map(indicator -> BigDecimal.valueOf(indicator.getBaseScore()))
                .reduce(BigDecimal.ZERO, BigDecimal::add);

        //计算
        calcRewardRnkCond(userShops, rewardDims, dimRewardMap, scopeType);
        calcTotalPushMoney(userShops, rewardDims, dimRewardMap, scopeType, rewardDetails);
        for (EvalGroupUserShop userShop : userShops) {
            userShop.setIndicatorTotalScore(totalScore);
            calcRewardMoney(rewards, userShop, rewardDetails);
            calcEvalKpi(userShop, indicators);
        }
        updatePool(evalGroup, localDate, scopeType, rewardDetails, userShops);
    }

    public void updatePool(EvalGroup evalGroup, LocalDate localDate, EvalScopeEnum scopeType,
                           List<EvalPoolRewardDetail> rewardDetails, List<EvalGroupUserShop> userShops) {
        final Long evalGroupId = evalGroup.getId();
        List<Long> poolIds = userShops.stream().map(EvalGroupUserShop::getPoolId).collect(Collectors.toList());
        if (PublicUtil.isNotEmpty(rewardDetails)) {
            evalPoolRewardDetailService.remove(Wrappers.<EvalPoolRewardDetail>lambdaQuery()
                    .in(EvalPoolRewardDetail::getPoolId, poolIds)
                    .eq(EvalPoolRewardDetail::getEvalGroupId, evalGroupId)
                    .eq(EvalPoolRewardDetail::getDataDate, localDate)
            );
            evalPoolRewardDetailService.saveBatch(rewardDetails);
        }
        if (EvalScopeEnum.STAFF.equals(scopeType)) {
            for (EvalGroupUserShop userShop : userShops) {
                BigDecimal scoreRatio = BigDecimal.ZERO;
                if (userShop.getIndicatorTotalScore().compareTo(BigDecimal.ZERO) > 0) {
                    scoreRatio = userShop.getScore().divide(userShop.getIndicatorTotalScore(), 2, RoundingMode.DOWN);
                }
                BigDecimal reward = (Boolean.TRUE.equals(userShop.getIgnored())) ? BigDecimal.ZERO : userShop.getReward();
                evalUserPoolService.update(Wrappers.<EvalUserPool>lambdaUpdate()
                        .eq(EvalUserPool::getId, userShop.getPoolId())
                        .eq(EvalUserPool::getYn, Boolean.TRUE)
                        .set(EvalUserPool::getReward, reward)
                        .set(EvalUserPool::getScore, userShop.getScore())
                        .set(EvalUserPool::getScoreRatio, scoreRatio)
                        .set(EvalUserPool::getMonthly, YearMonth.from(localDate))
                        .set(EvalUserPool::getDataDate, localDate)
                );
            }
            return;
        }
        for (EvalGroupUserShop userShop : userShops) {
            BigDecimal scoreRatio = BigDecimal.ZERO;
            if (userShop.getIndicatorTotalScore().compareTo(BigDecimal.ZERO) > 0) {
                scoreRatio = userShop.getScore().divide(userShop.getIndicatorTotalScore(), 2, RoundingMode.DOWN);
            }
            evalShopPoolService.update(Wrappers.<EvalShopPool>lambdaUpdate()
                    .eq(EvalShopPool::getId, userShop.getPoolId())
                    .eq(EvalShopPool::getYn, Boolean.TRUE)
                    .set(EvalShopPool::getReward, userShop.getReward())
                    .set(EvalShopPool::getScore, userShop.getScore())
                    .set(EvalShopPool::getScoreRatio, scoreRatio)
                    .set(EvalShopPool::getMonthly, YearMonth.from(localDate))
                    .set(EvalShopPool::getDataDate, localDate)
            );
        }
    }


    public List<EvalGroupUserShop> buildEvalGroupObj(EvalGroup evalGroup, EvalGroupRankStage evalGroupRankStage, LocalDate localDate, EvalScopeEnum scopeType) {
        LocalDate beginDate = evalGroupRankStage.getBeginTime();
        List<EvalGroupUserShop> userShops = new ArrayList<>();
        if (EvalScopeEnum.STAFF.equals(scopeType)) {
            List<EvalGroupUser> users = evalGroupUserService.list(Wrappers.<EvalGroupUser>lambdaQuery()
                    .eq(EvalGroupUser::getEvalGroupRankStageId, evalGroupRankStage.getId())
                    .eq(EvalGroupUser::getDataDate, localDate)
                    .eq(EvalGroupUser::getYn, Boolean.TRUE)
            );

            Map<Long, EvalUserPool> userPoolMap = evalPoolCommonService.inspectionUserPoolMap(evalGroup, evalGroupRankStage, users, localDate);
            for (EvalGroupUser user : users) {
                EvalUserPool userPool = userPoolMap.get(user.getUserId());
                userShops.add(convertBO(user, userPool));
            }
            return userShops;
        }
        List<Long> distShopIds = this.getEvalDistShopIds(evalGroup);
        Map<Long, EvalShopPool> shopPoolBOMap = evalPoolCommonService.inspectionShopPools(evalGroup, localDate, distShopIds);
        for (Long shopId : evalGroup.getShopIds()) {
            EvalShopPool shopPool = shopPoolBOMap.get(shopId);
            Boolean ignored = ! distShopIds.contains(shopId);
            userShops.add(convertBO(shopPool, evalGroup, beginDate, localDate, ignored));
        }
        return userShops;
    }


    public List<Long> getEvalDistShopIds(EvalGroup evalGroup) {
        List<EvalGroupRewardDim> rewardDims = evalGroupRewardDimService.list(Wrappers.<EvalGroupRewardDim>lambdaQuery()
                .eq(EvalGroupRewardDim::getEvalGroupId, evalGroup.getId())
                .eq(EvalGroupRewardDim::getType, EvalScopeEnum.SHOP)
                .eq(EvalGroupRewardDim::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isEmpty(rewardDims)) {
            return new ArrayList<>();
        }
        return rewardDims.stream().flatMap(dim -> dim.getShopIds().stream()).distinct().collect(Collectors.toList());
    }



    /**
     * 计算绩效组人员的绩效数据
     * 计算单个人员绩效数据
     *
     */
    private void calcEvalKpi(EvalGroupUserShop userShop, List<EvalGroupIndicator> indicators) {
//        KpiPool kpiPool = kpiPoolCommonService.inspectionPool(user);
//        if (Boolean.TRUE.equals(user.getIgnored())) {
//            return;
//        }
        final Long poolId = userShop.getPoolId();
        final Long evalGroupId = userShop.getEvalGroupId();
        List<EvalPoolIndicatorDetail> indicatorDetails = new ArrayList<>();
        for (EvalGroupIndicator indicator : indicators) {
            EvalKpiBaseCalculator calculator = calculatorEvalMap.get(indicator.getScoreWay());
            if (calculator == null) {
                continue;
            }
            BigDecimal score = calculator.calculate(userShop, indicator);
            EvalPoolIndicatorDetail indicatorDetail = createPoolIndicatorValue(score, indicator, userShop.getGroupId());
            indicatorDetail.setDataDate(userShop.getDataDate());
            indicatorDetail.setPoolId(poolId);
            indicatorDetail.setScopeType(userShop.getScopeType());
            indicatorDetails.add(indicatorDetail);
        }

        if (CollectionUtils.isNotEmpty(indicatorDetails)) {
            evalPoolIndicatorDetailService.remove(Wrappers.<EvalPoolIndicatorDetail>lambdaQuery()
                    .eq(EvalPoolIndicatorDetail::getPoolId, poolId)
                    .eq(EvalPoolIndicatorDetail::getEvalGroupId, evalGroupId)
                    .eq(EvalPoolIndicatorDetail::getScopeType, userShop.getScopeType())
                    .eq(EvalPoolIndicatorDetail::getDataDate, userShop.getDataDate())
                    .eq(EvalPoolIndicatorDetail::getYn, Boolean.TRUE));

            evalPoolIndicatorDetailService.saveBatch(indicatorDetails);
        }
        BigDecimal totalScore = BigDecimal.ZERO;
        for (EvalPoolIndicatorDetail indicatorValue : indicatorDetails) {
            totalScore = totalScore.add(indicatorValue.getScore());
        }
        userShop.setScore(totalScore);
    }


    /**
     * 计算考评排名条件
     *
     * @param
     */
    private void calcRewardRnkCond(List<EvalGroupUserShop> userShops, List<EvalGroupRewardDim> rewardDims,
                                   Map<Long, List<EvalGroupReward>> dimRewardMap, EvalScopeEnum scopeType) {
        for (EvalGroupRewardDim dim : rewardDims) {
            List<EvalGroupUserShop> filterUserShops = filterUserShops(dim, scopeType, userShops);
            if (PublicUtil.isEmpty(filterUserShops)) {
                continue;
            }
            for (EvalGroupReward evalGroupReward : dimRewardMap.getOrDefault(dim.getId(), new ArrayList<>())) {
                final String rewardProjectName = evalGroupReward.getName();
                EvalRewardBaseCalculator calculator = calculatorRewardMap.get(evalGroupReward.getCalMethod());
                if (Objects.isNull(calculator)) {
                    log.error("[{}_{}]计算器不存在", evalGroupReward.getId(), rewardProjectName);
                    return;
                }

                calculator.calculateRankPrecondition(filterUserShops, evalGroupReward);
            }
        }
    }

    public List<EvalGroupUserShop> filterUserShops(EvalGroupRewardDim dim, EvalScopeEnum scopeType, List<EvalGroupUserShop> userShops) {
        List<EvalGroupUserShop> filterUserShops = new ArrayList<>();
        if (EvalScopeEnum.STAFF.equals(scopeType)) {
            filterUserShops = userShops.stream().filter(userShop -> dim.getPostIds().contains(userShop.getPostId())).collect(Collectors.toList());
        } else {
            filterUserShops = userShops.stream().filter(userShop -> dim.getShopIds().contains(userShop.getShopId())).collect(Collectors.toList());
        }
        return filterUserShops;
    }

    /**
     * 计算绩效金额
     *
     * @param rewardDetails
     */
    private void calcTotalPushMoney(List<EvalGroupUserShop> userShops, List<EvalGroupRewardDim> rewardDims,
                                    Map<Long, List<EvalGroupReward>> dimRewardMap, EvalScopeEnum scopeType, List<EvalPoolRewardDetail> rewardDetails) {

        for (EvalGroupRewardDim dim : rewardDims) {
            List<EvalGroupReward> totalCalcRewards = dimRewardMap.getOrDefault(dim.getId(), new ArrayList<>()).stream()
                    .filter(reward -> EvalRewardCalMethodEnum.RANK.equals(reward.getCalMethod()) ||
                            EvalRewardCalMethodEnum.TOTAL_PROPORTION.equals(reward.getCalMethod()))
                    .collect(Collectors.toList());

            for (EvalGroupReward evalGroupReward : totalCalcRewards) {
                List<EvalGroupUserShop> filterUserShops = filterUserShops(dim, scopeType, userShops);
                if (PublicUtil.isEmpty(filterUserShops)) {
                    continue;
                }
                final String rewardProjectName = evalGroupReward.getName();
                EvalRewardBaseCalculator calculator = calculatorRewardMap.get(evalGroupReward.getCalMethod());
                if (Objects.isNull(calculator)) {
                    log.error("[{}_{}]计算器不存在", evalGroupReward.getId(), rewardProjectName);
                    return;
                }
                calculator.calculateGroup(userShops, evalGroupReward);
                for (EvalGroupUserShop userShop : filterUserShops) {
                    EvalPoolRewardDetail detail = createDetail(userShop);
                    detail.setScopeType(userShop.getScopeType());
                    detail.setEvalGroupRewardId(evalGroupReward.getId());
                    detail.setAmount(Optional.ofNullable(userShop.getEvalGroupRewardAmount()).orElse(BigDecimal.ZERO));
                    rewardDetails.add(detail);
                }
            }
        }
    }
    /**
     * 计算绩效金额
     *
     * @param rewardDetails
     */
    private void calcRewardMoney(List<EvalGroupReward> rewards, EvalGroupUserShop userShop, List<EvalPoolRewardDetail> rewardDetails) {

        List<EvalGroupReward> calcRewards = rewards.stream()
                .filter(reward ->
                        EvalRewardCalMethodEnum.FIXATION.equals(reward.getCalMethod()) ||
                        EvalRewardCalMethodEnum.LADDER.equals(reward.getCalMethod()) ||
                        EvalRewardCalMethodEnum.LADDER_DOUBLE.equals(reward.getCalMethod()))
                .collect(Collectors.toList());
        for (EvalGroupReward evalGroupReward : calcRewards) {
            final String rewardProjectName = evalGroupReward.getName();
            EvalRewardBaseCalculator calculator = calculatorRewardMap.get(evalGroupReward.getCalMethod());
            if (Objects.isNull(calculator)) {
                log.error("[{}_{}]计算器不存在", evalGroupReward.getId(), rewardProjectName);
                return;
            }
            final BigDecimal salaryMoney = calculator.calculate(userShop, evalGroupReward);
            if (Objects.nonNull(salaryMoney)) {
                EvalPoolRewardDetail detail = createDetail(userShop);
                detail.setScopeType(userShop.getScopeType());
                detail.setEvalGroupRewardId(evalGroupReward.getId());
                detail.setAmount(salaryMoney);
                rewardDetails.add(detail);

                userShop.setReward(userShop.getReward().add(salaryMoney));
            }
        }
    }

    /**
     * 缓存需要计算考评排名
     *
     * @param localDate
     */
    public void cacheCalculableEvalGroupRank(LocalDate localDate) {
        List<EvalGroupRank> evalGroupRanks = evalGroupRankService.list(Wrappers.<EvalGroupRank>lambdaQuery()
                .ge(EvalGroupRank::getOverTime, localDate)
                .eq(EvalGroupRank::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isEmpty(evalGroupRanks)) {
            return;
        }
        List<String> rankBOS = new ArrayList<>();
        for (EvalGroupRank evalGroupRank : evalGroupRanks) {
            List<EvalGroup> evalGroups = evalGroupService.getEffectsByRankId(evalGroupRank.getId(), localDate);
            if (PublicUtil.isEmpty(evalGroups)) {
                continue;
            }
            rankBOS.add(convertRankStr(evalGroupRank, localDate, evalGroups));
        }

        if (PublicUtil.isNotEmpty(rankBOS)) {
            ListOperations<String, String> listOps = stringRedisTemplate.opsForList();
            listOps.rightPushAll(getCalcEvalRankKey(), rankBOS);
        }
    }


    /**
     * 计算考评排名组
     */
    public void calcEvalGroupRank() {
        ListOperations<String, String> listOps = stringRedisTemplate.opsForList();
        List<String> errList = new ArrayList<>();
        String jsonStr;
        while ((jsonStr = listOps.leftPop(getCalcEvalRankKey())) != null) {
            if (StringUtils.isValid(jsonStr)) {
                EvalGroupCalculableRankBO calculableBO = JSONObject.parseObject(jsonStr, EvalGroupCalculableRankBO.class);
                if (Objects.nonNull(calculableBO)) {
                    try {
                        calcEvalRank(calculableBO);
                    } catch (Exception ex) {
                        log.error("计算考评排名组失败：[{}]", jsonStr, ex);
                        errList.add(jsonStr);
                    }
                }
            }
        }
        if (!CollectionUtils.isEmpty(errList)) {
            listOps.rightPushAll(getCalcEvalRankKey(), errList);
        }
    }

    /**
     * 计算排名
     *
     * @param bo
     */
    @Transactional(rollbackFor = Exception.class)
    public void calcEvalRank(EvalGroupCalculableRankBO bo) {
        final List<Long> evalGroupIds = bo.getEvalGroupIds();

        List<EvalUserPool> pools = evalUserPoolService.list(Wrappers.<EvalUserPool>lambdaQuery()
                .in(EvalUserPool::getEvalGroupId, evalGroupIds)
                .eq(EvalUserPool::getInclusion, Boolean.TRUE)
                .eq(EvalUserPool::getYn, Boolean.TRUE)
                .orderByDesc(EvalUserPool::getScoreRatio)
        );
        calcEvalUserRank(pools);
        evalUserPoolService.updateBatchById(pools);
    }

    /**
     * 计算考评排名
     *
     * @param pools
     */
    public void calcEvalUserRank(List<EvalUserPool> pools) {
        int rank = 1;
        Optional<BigDecimal> lastKpiScoreRatioOp = pools.stream().map(EvalUserPool::getScoreRatio).max(BigDecimal::compareTo);
        if (Boolean.FALSE.equals(lastKpiScoreRatioOp.isPresent())) {
            return;
        }
        BigDecimal lastKpiScoreRatio = lastKpiScoreRatioOp.get();
        for (EvalUserPool pool : pools) {
            if (PublicUtil.isEmpty(pool.getScoreRatio())) {
                continue;
            }
            if (lastKpiScoreRatio.compareTo(pool.getScoreRatio()) != 0) {
                rank++;
            }
            pool.setRank(rank);
            lastKpiScoreRatio = pool.getScoreRatio();
        }
    }

    /**
     * 计算考评排名
     *
     * @param pools
     */
    public void calcEvalShopRank(List<EvalShopPool> pools) {
        int rank = 1;
        Optional<BigDecimal> lastKpiScoreRatioOp = pools.stream().map(EvalShopPool::getScoreRatio).max(BigDecimal::compareTo);
        if (Boolean.FALSE.equals(lastKpiScoreRatioOp.isPresent())) {
            return;
        }
        BigDecimal lastKpiScoreRatio = lastKpiScoreRatioOp.get();
        for (EvalShopPool pool : pools) {
            if (PublicUtil.isEmpty(pool.getScoreRatio())) {
                continue;
            }
            if (lastKpiScoreRatio.compareTo(pool.getScoreRatio()) != 0) {
                rank++;
            }
            pool.setRank(rank);
            lastKpiScoreRatio = pool.getScoreRatio();
        }
    }


    public String convertRankStr(EvalGroupRank evalGroupRank, LocalDate localDate, List<EvalGroup> evalGroups) {
        EvalGroupCalculableRankBO rankBO = EvalGroupCalculableRankBO.builder()
                .groupId(evalGroupRank.getGroupId())
//                .scopeType(evalGroupRank.getScopeType())
//                .egcs(evalGroupRank.getEgcs())
                .evalGroupIds(evalGroups.stream().map(EvalGroup::getId).collect(Collectors.toList()))
                .localDate(localDate)
                .build();
        return JSONObject.toJSONString(rankBO);
    }

    private EvalPoolIndicatorDetail createPoolIndicatorValue(BigDecimal score, EvalGroupIndicator indicator, Long groupId) {
        EvalPoolIndicatorDetail indicatorDetail = new EvalPoolIndicatorDetail();
        indicatorDetail.setEvalGroupId(indicator.getEvalGroupId());
        indicatorDetail.setEvalGroupIndicatorId(indicator.getId());
        indicatorDetail.setScore(score);
        indicatorDetail.setYn(Boolean.TRUE);
        indicatorDetail.setGroupId(groupId);
        return indicatorDetail;
    }

    /**
     * 创建实体
     *
     * @return
     */
    private EvalPoolRewardDetail createDetail(EvalGroupUserShop userShop) {
        EvalPoolRewardDetail poolDetail = new EvalPoolRewardDetail();
        poolDetail.setPoolId(userShop.getPoolId());
        poolDetail.setEvalGroupId(userShop.getEvalGroupId());
        poolDetail.setScopeType(userShop.getScopeType());
        poolDetail.setAmount(BigDecimal.ZERO);
        poolDetail.setDataDate(userShop.getDataDate());
        poolDetail.setGroupId(userShop.getGroupId());
        poolDetail.setYn(Boolean.TRUE);
        return poolDetail;
    }

    public EvalGroupUserShop convertBO(EvalGroupUser user, EvalUserPool userPool) {
        EvalGroupUserShop userShop = new EvalGroupUserShop();
        userShop.setName(user.getUserName());
        userShop.setScopeType(EvalScopeEnum.STAFF);
        userShop.setEvalGroupId(user.getEvalGroupId());
        userShop.setPoolId(userPool.getId());
        userShop.setPostId(userPool.getPostId());
        userShop.setShopId(userPool.getShopId());
        userShop.setReferId(user.getUserId());
        userShop.setDataDate(user.getDataDate());
        userShop.setGroupId(user.getGroupId());
        userShop.setScore(BigDecimal.ZERO);
        userShop.setReward(BigDecimal.ZERO);
        userShop.setIgnored(user.getIgnored());
        return userShop;
    }

    public EvalGroupUserShop convertBO(EvalShopPool shopPool, EvalGroup evalGroup, LocalDate beginDate, LocalDate dataDate, Boolean ignored) {
        EvalGroupUserShop userShop = new EvalGroupUserShop();
        userShop.setName(shopPool.getShopName());
        userShop.setReferId(shopPool.getShopId());
        userShop.setPoolId(shopPool.getId());
        userShop.setShopId(shopPool.getShopId());
        userShop.setScopeType(EvalScopeEnum.SHOP);
        userShop.setEvalId(userShop.getEvalId());
        userShop.setEvalGroupId(evalGroup.getId());
        userShop.setBeginDate(beginDate);
        userShop.setDataDate(dataDate);
        userShop.setGroupId(evalGroup.getGroupId());
        userShop.setIgnored(ignored);
        userShop.setScore(BigDecimal.ZERO);
        userShop.setReward(BigDecimal.ZERO);
        return userShop;
    }

}
