package cn.fw.morax.service.biz.kpi;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.ehr.sdk.api.enums.StaffStatusEnum;
import cn.fw.morax.common.config.RoleVal;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.db.kpi.KpiAssess;
import cn.fw.morax.domain.dto.kpi.KpiAssessDTO;
import cn.fw.morax.domain.dto.kpi.KpiAssessDetailDTO;
import cn.fw.morax.domain.dto.kpi.KpiAssessStaffDTO;
import cn.fw.morax.domain.dto.query.KpiAssessQueryDTO;
import cn.fw.morax.domain.enums.KpiAssessTypeEnum;
import cn.fw.morax.domain.vo.RoleInfoVO;
import cn.fw.morax.domain.vo.kpi.*;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PerformanceStaffDTO;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.rpc.erp.ErpRpcService;
import cn.fw.morax.rpc.erp.dto.RoleInfoDTO;
import cn.fw.morax.rpc.erp.dto.RpcUserRoleInfoDTO;
import cn.fw.morax.service.data.kpi.KpiAssessDetailService;
import cn.fw.morax.service.data.kpi.KpiAssessPenaltyService;
import cn.fw.morax.service.data.kpi.KpiAssessPenaltyStaffService;
import cn.fw.morax.service.data.kpi.KpiAssessService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author jiangchao
 * @des: 人员质量评测
 * @date 2023/3/17 16:01
 */
@RequiredArgsConstructor
@Service
@Slf4j
public class KpiAssessBizService {

    private final RoleVal roleVal;
    private final ErpRpcService erpRpcService;
    private final EhrRpcService ehrRpcService;
    private final KpiAssessService kpiAssessService;
    private final KpiAssessDetailService kpiAssessDetailService;
    private final KpiAssessPenaltyService kpiAssessPenaltyService;
    private final KpiAssessPenaltyStaffService kpiAssessPenaltyStaffService;

    @Value("${spring.cache.custom.global-prefix}:kpi:assess:save:")
    @Getter
    private String savePrefix;

    /**
     * 分页查询
     *
     * @param dto
     * @return
     */
    public AppPage<KpiAssessVO> kpiAssessPage(KpiAssessQueryDTO dto) {
        PageData<KpiAssess> pageData = kpiAssessService.page(new PageData<>(dto.getCurrent(), dto.getPageSize()),
                Wrappers.<KpiAssess>lambdaQuery()
                        .eq(KpiAssess::getBackup, Boolean.FALSE)
                        .eq(KpiAssess::getGroupId, dto.getGroupId())
                        .eq(PublicUtil.isNotEmpty(dto.getPostId()), KpiAssess::getPostId, dto.getPostId())
                        .eq(KpiAssess::getYn, Boolean.TRUE)
        );

        return PublicUtil.toPage(pageData, kpiAssess -> {
            KpiAssessVO kpiAssessVO = PublicUtil.copy(kpiAssess, KpiAssessVO.class);
            List<KpiAssessDetail> kpiAssessDetails = kpiAssessDetailService.list(Wrappers.<KpiAssessDetail>lambdaQuery()
                    .eq(KpiAssessDetail::getKpiAssessId, kpiAssessVO.getId())
                    .eq(KpiAssessDetail::getYn, Boolean.TRUE)
            );
            List<String> roleNames = kpiAssessDetails.stream().map(KpiAssessDetail::getRoleNames)
                    .collect(ArrayList::new, List::addAll, List::addAll);
            kpiAssessVO.setRoleNames(roleNames);
            return kpiAssessVO;
        });
    }


    public KpiAssessVO kpiAssessDetail(Long id) {
        KpiAssess kpiAssess = kpiAssessService.getById(id);
        BV.notNull(kpiAssess, "配置不存在，请重试");
        KpiAssessVO kpiAssessVO = PublicUtil.copy(kpiAssess, KpiAssessVO.class);
        List<KpiAssessDetail> kpiAssessDetails = kpiAssessDetailService.list(Wrappers.<KpiAssessDetail>lambdaQuery()
                .eq(KpiAssessDetail::getKpiAssessId, id)
                .eq(KpiAssessDetail::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isEmpty(kpiAssessDetails)) {
            return kpiAssessVO;
        }
        List<KpiAssessDetailVO> kpiAssessDetailVos = PublicUtil.copyList(kpiAssessDetails, KpiAssessDetailVO.class);
        kpiAssessVO.setDetails(kpiAssessDetailVos);
        return kpiAssessVO;
    }

    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getSavePrefix()", key = "#dto.getPostId()", message = "保存中，请勿重复操作")
    public void saveKpiAssess(KpiAssessDTO dto) {
        checkAssess(dto);

        KpiAssess kpiAssess = convertDB(dto);
        kpiAssessService.saveOrUpdate(kpiAssess);
        final Long kpiAssessId = kpiAssess.getId();

        kpiAssessDetailService.update(Wrappers.<KpiAssessDetail>lambdaUpdate()
                .eq(KpiAssessDetail::getKpiAssessId, kpiAssessId)
                .set(KpiAssessDetail::getYn, Boolean.FALSE)
        );
        List<KpiAssessDetail> kpiAssessDetails = PublicUtil.copyList(dto.getDetails(), KpiAssessDetail.class);
        kpiAssessDetails.forEach(kpiAssessDetail -> transferDB(kpiAssessId, dto.getType(), kpiAssessDetail));
        kpiAssessDetailService.saveBatch(kpiAssessDetails);
    }

    public void transferDB(Long kpiAssessId, KpiAssessTypeEnum type, KpiAssessDetail detail) {
        detail.setKpiAssessId(kpiAssessId);
        if (KpiAssessTypeEnum.STAR_D.equals(type)) {
            detail.setCapPersonPenaltyScore(BigDecimal.ZERO);
            detail.setCapPersonPenaltyMoney(BigDecimal.ZERO);
            detail.setAwardScore(BigDecimal.ZERO);
            detail.setCapPersonAwardScore(BigDecimal.ZERO);
            detail.setCapAwardScore(BigDecimal.ZERO);
            detail.setAwardMoney(BigDecimal.ZERO);
            detail.setCapPersonAwardMoney(BigDecimal.ZERO);
            detail.setCapAwardMoney(BigDecimal.ZERO);
        }
    }

    public void checkAssess(KpiAssessDTO dto) {
        List<KpiAssess> kpiAssesses = kpiAssessService.getConfigs(dto.getId(), dto.getType(), dto.getGroupId(), dto.getPostId(), dto.getShopIds());
        BV.isTrue(PublicUtil.isEmpty(kpiAssesses), "岗位、门店存在重复配置，请重试");

        if (KpiAssessTypeEnum.TURN_POSITIVE_AFTER.equals(dto.getType())) {
            List<KpiAssessDetailDTO> errorDetails = dto.getDetails().stream()
                    .filter(detail ->
                            PublicUtil.isEmpty(detail.getCapPersonPenaltyScore()) ||
                            PublicUtil.isEmpty(detail.getCapPersonPenaltyMoney()) ||
                            PublicUtil.isEmpty(detail.getAwardScore()) ||
                            PublicUtil.isEmpty(detail.getCapPersonAwardScore()) ||
                            PublicUtil.isEmpty(detail.getCapAwardScore()) ||
                            PublicUtil.isEmpty(detail.getAwardMoney()) ||
                            PublicUtil.isEmpty(detail.getCapPersonAwardMoney()) ||
                            PublicUtil.isEmpty(detail.getCapAwardMoney())
                    )
                    .collect(Collectors.toList());
            BV.isTrue(PublicUtil.isEmpty(errorDetails), "转正后保护期员工需要设置封顶单人惩罚分、奖励，请重试");
        }

        Set<String> roleCodes = new HashSet<>();
        dto.getDetails().stream().forEach(detail -> detail.getRoleCodes().stream().forEach(roleCode -> {
            if (! roleCodes.add(roleCode)) {
                throw new BusinessException("角色重复配置，请重试");
            }
        }));
    }

    @Transactional(rollbackFor = Exception.class)
    public void removeById(Long id) {
        kpiAssessService.removeById(id);
        kpiAssessDetailService.update(Wrappers.<KpiAssessDetail>lambdaUpdate()
                .eq(KpiAssessDetail::getKpiAssessId, id)
                .set(KpiAssessDetail::getYn, Boolean.FALSE)
        );
    }

    /**
     * 转正保护期员工
     *
     * @param postId
     * @param shopIds
     * @param localDate
     * @return
     */
    public List<KpiGroupUserVO> kpiRegularProtectionStaffs(Long postId, List<Long> shopIds, String staffName, LocalDate localDate) {
        List<PerformanceStaffDTO> staffs = ehrRpcService.queryKpiStaffByName(postId, new ArrayList<>(shopIds), staffName);
        if (PublicUtil.isEmpty(staffs)) {
            return new ArrayList<>();
        }
        List<KpiGroupUserVO> regularProtectionStaffs = new ArrayList<>();
        LocalDate currentTime = LocalDate.now();

        for (PerformanceStaffDTO staff : staffs) {
            //转正两个月内不计算绩效  绩效开始时间为转正的两个月后
            if (StaffStatusEnum.REGULAR.getValue().equals(staff.getStaffStatus())) {
                LocalDate regularDate = PublicUtil.isNotEmpty(staff.getRegularDate()) ? staff.getRegularDate() : staff.getEntryDate();
                LocalDate startKpiDate = PublicUtil.getKpiDateOfRegularStaff(regularDate);
                //开始计算考评日期在 当前时间之前
                if (startKpiDate.isAfter(currentTime)) {
                    KpiGroupUserVO kpiUser = this.convertToGroupUser(staff, localDate);
                    regularProtectionStaffs.add(kpiUser);
                }
            }
        }
        return regularProtectionStaffs;
    }

    /**
     * 转正保护期员工
     *
     * @return
     */
    public List<KpiGroupUserVO> queryAssessStaffs(KpiAssessStaffDTO dto) {
        List<KpiGroupUserVO> users = new ArrayList<>();
        Set<Long> staffIds = new HashSet<>();
        final String staffName = dto.getStaffName();
        final List<Long> shopIds = dto.getShopIds();
        for (String roleCode : dto.getRoleCodes()) {
            List<RpcUserRoleInfoDTO> userRoleInfos = this.getUserEnableRoleInfos(roleCode, new ArrayList<>(shopIds));
            if (PublicUtil.isNotEmpty(userRoleInfos)) {
                staffIds.addAll(userRoleInfos.stream().map(RpcUserRoleInfoDTO::getUserId).collect(Collectors.toSet()));
            }
        }
        if (PublicUtil.isEmpty(staffIds)) {
            return new ArrayList<>();
        }
        List<StaffBaseInfoDTO> staffDtos = ehrRpcService.queryStaffBaseInfo(new ArrayList<>(staffIds));
        staffDtos.stream().filter(staff -> {
            if (PublicUtil.isNotEmpty(dto.getStaffName())) {
                return staff.getName().contains(staffName);
            }
            return Boolean.TRUE;
        }).forEach(staffDto -> users.add(convertToGroupUser(staffDto)));
        return users;
    }

    /**
     * 对查询的人员去重
     *
     * @param roleCode
     * @param shopIds
     * @return
     */
    public List<RpcUserRoleInfoDTO> getUserEnableRoleInfos(String roleCode, List<Long> shopIds) {
        List<RpcUserRoleInfoDTO> userRoleInfos = erpRpcService.getUserEnableRoleInfos(roleCode, shopIds);
        if (PublicUtil.isEmpty(userRoleInfos)) {
            return new ArrayList<>();
        }
        Set<Long> staffIds = new HashSet<>();
        List<RpcUserRoleInfoDTO> repeatUserRoleInfos = Lists.newArrayListWithCapacity(userRoleInfos.size());
        for (RpcUserRoleInfoDTO rpcUserRoleInfoDTO : userRoleInfos) {
            if (staffIds.add(rpcUserRoleInfoDTO.getUserId())) {
                repeatUserRoleInfos.add(rpcUserRoleInfoDTO);
            }
        }
        return repeatUserRoleInfos;
    }

    /**
     * 转换员工基础信息
     *
     * @param staff
     * @return
     */
    private KpiGroupUserVO convertToGroupUser(StaffBaseInfoDTO staff){
        KpiGroupUserVO kpiUser = KpiGroupUserVO.builder()
                .userId(staff.getId())
                .userName(staff.getName())
                .postId(staff.getPostId())
                .postName(staff.getPostName())
                .shopId(staff.getShopId())
                .shopName(staff.getShopName())
                .groupId(staff.getGroupId())
                .build();
        return kpiUser;
    }

    /**
     * 转换员工基础信息
     *
     * @param staff
     * @return
     */
    private KpiGroupUserVO convertToGroupUser(PerformanceStaffDTO staff, LocalDate now){
        KpiGroupUserVO kpiUser = KpiGroupUserVO.builder()
                .userId(staff.getId())
                .userName(staff.getName())
                .postId(staff.getPostId())
                .postName(staff.getPostName())
                .shopId(staff.getShopId())
                .shopName(staff.getShopName())
                .groupId(staff.getGroupId())
                .dataDate(now)
                .build();
        return kpiUser;
    }

    public KpiAssess convertDB(KpiAssessDTO dto) {
        KpiAssess kpiAssess = new KpiAssess();
        if (PublicUtil.isNotEmpty(dto.getId())) {
            kpiAssess.setId(dto.getId());
        } else {
            kpiAssess.setCreateBy(dto.getUserId());
        }
        kpiAssess.setType(dto.getType());
        kpiAssess.setPostId(dto.getPostId());
        kpiAssess.setPostName(dto.getPostName());
        kpiAssess.setShopIds(dto.getShopIds());
        kpiAssess.setShopNames(dto.getShopNames());
        if (KpiAssessTypeEnum.STAR_D.equals(dto.getType())) {
            kpiAssess.setRank(0);
        } else {
            kpiAssess.setRank(dto.getRank());
        }
        kpiAssess.setRank(dto.getRank());
        kpiAssess.setGroupId(dto.getGroupId());
        kpiAssess.setYn(Boolean.TRUE);
        kpiAssess.setBackup(Boolean.FALSE);
        kpiAssess.setUpdateBy(dto.getUserId());
        return kpiAssess;
    }

    public List<RoleInfoVO> assessRoleInfos() {
        List<RoleInfoDTO> manageRoles = erpRpcService.manageRoles();
        RoleInfoDTO roleInfo = erpRpcService. queryRole(roleVal.getHrOperation());
        if (PublicUtil.isNotEmpty(roleInfo)) {
            manageRoles.add(roleInfo);
        }
        return PublicUtil.copyList(manageRoles, RoleInfoVO.class);
    }

    public KpiAssessResultVO assessExpected(Long userId, LocalDate dataDate) {
        List<KpiAssessPenalty> penalties = kpiAssessPenaltyService.list(Wrappers.<KpiAssessPenalty>lambdaQuery()
                .eq(KpiAssessPenalty::getUserId, userId)
                .eq(KpiAssessPenalty::getDataDate, dataDate)
//                .eq(KpiAssessPenalty::getType, kpiAssessType)
                .eq(KpiAssessPenalty::getExpectedPenalty, Boolean.TRUE)
                .eq(KpiAssessPenalty::getYn, Boolean.TRUE)
        );
        return queryAssess(penalties, Boolean.TRUE);
    }

    public KpiAssessResultVO queryAssess(List<KpiAssessPenalty> penalties, Boolean filterZero) {
        if (PublicUtil.isEmpty(penalties)) {
            return new KpiAssessResultVO();
        }
        KpiAssessResultVO resultVO = new KpiAssessResultVO();
        Map<KpiAssessTypeEnum, List<KpiAssessPenaltyStaffVO>> result = new HashMap<>();
        List<Long> penaltyIds =  penalties.stream().map(KpiAssessPenalty::getId).collect(Collectors.toList());
        List<KpiAssessPenaltyStaffVO> penaltyStaffs = kpiAssessPenaltyStaffService.getAssesses(penaltyIds);
        if (filterZero) {
            penaltyStaffs = penaltyStaffs.stream()
                    .filter(staff -> (staff.getScore().compareTo(BigDecimal.ZERO)) > 0 || (staff.getAward().compareTo(BigDecimal.ZERO)) > 0)
                    .collect(Collectors.toList());
        }
        Map<KpiAssessTypeEnum, List<KpiAssessPenaltyStaffVO>> typeMap = penaltyStaffs.stream().collect(Collectors.groupingBy(KpiAssessPenaltyStaffVO::getType));
        for (Map.Entry<KpiAssessTypeEnum, List<KpiAssessPenaltyStaffVO>> entry : typeMap.entrySet()) {
            for (KpiAssessPenaltyStaffVO staff : entry.getValue()) {
                if (KpiAssessTypeEnum.TURN_POSITIVE_AFTER.equals(staff.getType())) {
                    staff.setRankDiff(new BigDecimal(Optional.ofNullable(staff.getRank()).orElse(0) -
                            Optional.ofNullable(staff.getTargetRank()).orElse(0)).abs().intValue());
                }
            }
            result.put(entry.getKey(), entry.getValue());
        }
        resultVO.setStarStaffs(result.getOrDefault(KpiAssessTypeEnum.STAR_D, new ArrayList<>()));
        resultVO.setTurnPositiveStaffs(result.getOrDefault(KpiAssessTypeEnum.TURN_POSITIVE_AFTER, new ArrayList<>()));
        return resultVO;
    }

    public KpiAssessResultVO staffKpiAssessDetail(String businessToken) {
        String[] keys =  businessToken.split(Constant.SEPARATOR);
        Long userId = Long.parseLong(keys[0]);
        YearMonth monthly = YearMonth.parse(keys[1], DateTimeFormatter.ofPattern("yyyy-MM"));
        List<KpiAssessPenalty> penalties = kpiAssessPenaltyService.list(Wrappers.<KpiAssessPenalty>lambdaQuery()
                .eq(KpiAssessPenalty::getUserId, userId)
                .eq(KpiAssessPenalty::getDataDate, monthly.atEndOfMonth())
                .eq(KpiAssessPenalty::getExpectedPenalty, Boolean.FALSE)
                .eq(KpiAssessPenalty::getYn, Boolean.TRUE)
        );
        return queryAssess(penalties, Boolean.TRUE);
    }

}
