package cn.fw.morax.service.biz.kpi;

import cn.fw.morax.common.config.RoleVal;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.common.utils.ThreadPoolUtil;
import cn.fw.morax.domain.bo.kpi.KpiAssessCalculableBO;
import cn.fw.morax.domain.bo.kpi.KpiGroupCalculableBO;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.enums.KpiAssessTypeEnum;
import cn.fw.morax.domain.enums.KpiIgnoreCauseEnum;
import cn.fw.morax.domain.enums.StarLevelEnum;
import cn.fw.morax.domain.enums.WorkTypeEnum;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.rpc.erp.ErpRpcService;
import cn.fw.morax.rpc.erp.dto.RpcUserRoleInfoDTO;
import cn.fw.morax.rpc.erp.dto.RpcUserRoleShopDTO;
import cn.fw.morax.service.biz.CommonService;
import cn.fw.morax.service.component.PunishStaffProducer;
import cn.fw.morax.service.data.kpi.*;
import cn.fw.office.sdk.event.PunishStaffReportEvent;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : KpiCalculateService
 * @description : 绩效计算类
 * @date : 2022-04-15 11:40
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class KpiCalculateAssessService {
    private final RoleVal roleVal;
    private final KpiGroupService kpiGroupService;
    private final KpiPoolService kpiPoolService;
    private final KpiPoolIndicatorValueService kpiPoolIndicatorValueService;
    private final KpiGroupUserService kpiGroupUserService;
    private final KpiGroupIndicatorService kpiGroupIndicatorService;
    private final KpiRewardPointService kpiRewardPointService;
    private final StringRedisTemplate stringRedisTemplate;
    private final KpiPoolCommonService kpiPoolCommonService;
    private final KpiAssessService kpiAssessService;
    private final KpiAssessDetailService kpiAssessDetailService;
    private final KpiAssessPenaltyService kpiAssessPenaltyService;
    private final KpiAssessPenaltyStaffService kpiAssessPenaltyStaffService;
    private final ErpRpcService erpRpcService;
    private final EhrRpcService ehrRpcService;
    private final CommonService commonService;
    private final PunishStaffProducer punishStaffProducer;
    private final PlatformTransactionManager platformTransactionManager;
    private final TransactionDefinition transactionDefinition;

    @Value("${spring.cache.custom.global-prefix}:calculable:kpi-group")
    @Getter
    private String key;

    @Value("${spring.cache.custom.global-prefix}:calculate-scoring-rate:kpi-group")
    @Getter
    private String calcuScoringRateKey;

    @Value("${spring.cache.custom.global-prefix}:calculable:kpi:assess")
    @Getter
    private String assessKpiKey;

    /**
     * 缓存计算考评绩效组
     *
     * @param localDate 日期
     */
    public void cacheAssessCalculableKpiGroupIds(final LocalDate localDate) {
        log.info("定时任务【缓存需要计算绩效的绩效组】开始执行");
        Set<Long> kpiGroupIds = new HashSet<>();
        List<KpiAssess> kpiAssesses = kpiAssessService.list(Wrappers.<KpiAssess>lambdaQuery()
            .eq(KpiAssess::getYn, Boolean.TRUE)
        );
        Map<Long, List<KpiAssess>> postKpiAssesses = kpiAssesses.stream().collect(Collectors.groupingBy(KpiAssess::getPostId));
        for (Map.Entry<Long, List<KpiAssess>> entry : postKpiAssesses.entrySet()) {
            List<Long> shopIds = entry.getValue().stream().flatMap(kpiAssess -> kpiAssess.getShopIds().stream())
                    .distinct().collect(Collectors.toList());
            Long postId = entry.getKey();
            List<KpiGroup> kpiGroups = kpiGroupService.getKpis(postId, shopIds, localDate);
            kpiGroupIds.addAll(kpiGroups.stream().map(KpiGroup::getId).collect(Collectors.toSet()));
        }

        if (CollectionUtils.isEmpty(kpiGroupIds)) {
            return;
        }
        log.info("缓存计算考评绩效组信息，kpiGroupIds：{}，日期：{}", JSON.toJSONString(kpiGroupIds), localDate);
        cacheGroupIds(kpiGroupIds, localDate);
    }

    private void cacheGroupIds(final Set<Long> kpiGroupIds, final LocalDate localDate) {
        String[] array = kpiGroupIds.stream()
                .map(kpiGroupId -> new KpiGroupCalculableBO(kpiGroupId, localDate))
                .map(JSONObject::toJSONString)
                .toArray(String[]::new);
        stringRedisTemplate.opsForSet().add(getKey(), array);
    }


    /**
     * 备份当月人员质量考核数据
     *
     */
    @Transactional(rollbackFor = Exception.class)
    public void backupKpiAssesses() {
        log.info("定时任务【备份当月人员质量考核数据】开始执行");
        List<KpiAssess> kpiAssesses = kpiAssessService.list(Wrappers.<KpiAssess>lambdaQuery()
                .eq(KpiAssess::getBackup, Boolean.FALSE)
                .eq(KpiAssess::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(kpiAssesses)) {
            return;
        }
        //备份数据
        List<KpiAssess> backKpiAssesses = new ArrayList<>();
        for (KpiAssess kpiAssess : kpiAssesses) {
            List<KpiAssessDetail> kpiAssessDetails = kpiAssessDetailService.list(Wrappers.<KpiAssessDetail>lambdaQuery()
                    .eq(KpiAssessDetail::getKpiAssessId, kpiAssess.getId())
                    .eq(KpiAssessDetail::getYn, Boolean.TRUE)
            );
            kpiAssess.setId(null);
            kpiAssess.setBackup(Boolean.TRUE);
            kpiAssessService.save(kpiAssess);
            backKpiAssesses.add(kpiAssess);
            final Long kpiAssessId = kpiAssess.getId();
            kpiAssessDetails.stream().forEach(kpiAssessDetail -> {
                kpiAssessDetail.setId(null);
                kpiAssessDetail.setKpiAssessId(kpiAssessId);
            });
            kpiAssessDetailService.saveBatch(kpiAssessDetails);
        }
    }

    /**
     * 缓存计算人员质量考核
     *
     */
    public void cacheCalcKpiAssessIds(final LocalDate localDate, Boolean backup) {
        log.info("定时任务【缓存需要计算绩效的绩效组】开始执行");

        LambdaQueryWrapper<KpiAssess> queryWrapper = Wrappers.<KpiAssess>lambdaQuery().eq(KpiAssess::getYn, Boolean.TRUE);
        if (Boolean.TRUE.equals(backup)) {
            queryWrapper.gt(KpiAssess::getCreateTime, localDate);
        }

        List<KpiAssess> backKpiAssesses = kpiAssessService.list(
                queryWrapper.eq(KpiAssess::getBackup, backup)
        );
        if (CollectionUtils.isEmpty(backKpiAssesses)) {
            return;
        }

        log.info("缓存计算人员质量考核信息，kpiAssesses：{}", JSON.toJSONString(backKpiAssesses));
        String[] array = backKpiAssesses.stream()
                .map(kpiAssess -> new KpiAssessCalculableBO(kpiAssess.getId(), localDate))
                .map(JSONObject::toJSONString)
                .toArray(String[]::new);
        stringRedisTemplate.opsForSet().add(getAssessKpiKey(), array);
    }


    /**
     * 计算人员质量考核
     */
    public void calcAssessKpi() {
        BoundSetOperations<String, String> setOps = stringRedisTemplate.boundSetOps(getAssessKpiKey());
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        List<String> overflowsList = new ArrayList<>();
        String str;
        while ((str = setOps.pop()) != null) {
            final KpiAssessCalculableBO bo = JSONObject.parseObject(str, KpiAssessCalculableBO.class);
            if (PublicUtil.isEmpty(bo)) {
                continue;
            }
            try {
                String finalStr = str;
                threadPool.execute(() -> {
                    TransactionStatus transactionStatus = platformTransactionManager.getTransaction(transactionDefinition);
                    try {
                        calculateKpiAssess(bo);
                        platformTransactionManager.commit(transactionStatus);
                    } catch (Exception ex) {
                        platformTransactionManager.rollback(transactionStatus);
                        log.error("计算人员质量考核失败：{}", JSON.toJSONString(bo), ex);
                        setOps.add(finalStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsList.add(str);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsList)) {
            for (String s : overflowsList) {
                setOps.add(s);
            }
        }
    }

    /**
     * 计算人员质量考核
     */
    public void calculateKpiAssess(KpiAssessCalculableBO calculableBO) {
        final Long kpiAssessId = calculableBO.getKpiAssessId();
        final LocalDate localDate = calculableBO.getLocalDate();
        final YearMonth monthly = YearMonth.from(localDate);
        KpiAssess kpiAssess = kpiAssessService.getById(kpiAssessId);
        if (Objects.isNull(kpiAssess)) {
            log.error("人员质量考核[{}]不存在，终止计算！", JSON.toJSONString(kpiAssessId));
            return;
        }
        //备份数据生成的是真实处罚
        Boolean expectedPenalty = Boolean.FALSE.equals(kpiAssess.getBackup());
        List<KpiAssessDetail> kpiAssessDetails = kpiAssessDetailService.list(Wrappers.<KpiAssessDetail>lambdaQuery()
                .eq(KpiAssessDetail::getKpiAssessId, kpiAssessId)
                .eq(KpiAssessDetail::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(kpiAssessDetails)) {
            log.error("人员质量考核详情[{}]为空，终止计算！", kpiAssessId);
            return;
        }

        //删除重复数据
        cleanRepeatPenalty(kpiAssessId, localDate, expectedPenalty);
        final KpiAssessTypeEnum type = kpiAssess.getType();
        final Integer rank = kpiAssess.getRank();
        final List<Long> shopIds = kpiAssess.getShopIds();

        for (KpiAssessDetail kpiAssessDetail : kpiAssessDetails) {
            Set<Long> repeatUserIds = new HashSet<>();
            for (String roleCode : kpiAssessDetail.getRoleCodes()) {
                List<RpcUserRoleInfoDTO> userRoleInfos = commonService.getUserEnableRoleInfos(roleCode, shopIds);
                if (PublicUtil.isEmpty(userRoleInfos)) {
                    continue;
                }
                for (RpcUserRoleInfoDTO userRoleInfo : userRoleInfos) {
                    //同一条配置，考核人员不能被考核多次
                    if (! repeatUserIds.add(userRoleInfo.getUserId())) {
                        continue;
                    }
                    List<Long> manageShopIds = userRoleInfo.getRangeList().stream().map(RpcUserRoleShopDTO::getShopId).collect(Collectors.toList());
                    if (PublicUtil.isEmpty(manageShopIds)) {
                        return;
                    }
                    List<StaffBaseInfoDTO> managerScopeStaffs;
                    if (roleVal.getHrOperation().equals(roleCode)) {
                        managerScopeStaffs = ehrRpcService.getStaffListByShop(manageShopIds);
                    } else {
                        managerScopeStaffs = ehrRpcService.getManagerScopeStaffs(roleCode, userRoleInfo.getUserId(), manageShopIds);
                    }
                    if (PublicUtil.isEmpty(managerScopeStaffs)) {
                        continue;
                    }
                    List<Long> manageStaffIds = managerScopeStaffs.stream().map(StaffBaseInfoDTO::getId).distinct().collect(Collectors.toList());
                    KpiAssessPenalty kpiAssessPenalty = convertDB(expectedPenalty, roleCode, kpiAssessDetail, userRoleInfo, localDate, kpiAssess);
                    if (KpiAssessTypeEnum.STAR_D.equals(type)) {
                        calcLevelD(kpiAssessDetail, monthly, manageStaffIds, kpiAssessPenalty);
                    } else {
                        calcRankDiff(rank, kpiAssessDetail, localDate, manageStaffIds, kpiAssessPenalty);
                    }
                }
            }
        }
    }

    public void cleanRepeatPenalty(Long kpiAssessId, LocalDate localDate, Boolean expectedPenalty) {
        List<KpiAssessPenalty> repeatPenalties = kpiAssessPenaltyService.list(Wrappers.<KpiAssessPenalty>lambdaQuery()
                .eq(KpiAssessPenalty::getKpiAssessId, kpiAssessId)
                .eq(KpiAssessPenalty::getDataDate, localDate)
                .eq(KpiAssessPenalty::getExpectedPenalty, expectedPenalty)
                .eq(KpiAssessPenalty::getYn, Boolean.TRUE));
        if (PublicUtil.isEmpty(repeatPenalties)) {
            return;
        }
        List<Long> ids = repeatPenalties.stream().map(KpiAssessPenalty::getId).collect(Collectors.toList());
        kpiAssessPenaltyService.removeByIds(ids);
        kpiAssessPenaltyStaffService.remove(Wrappers.<KpiAssessPenaltyStaff>lambdaQuery()
                .in(KpiAssessPenaltyStaff::getKpiAssessPenaltyId, ids)
                .eq(KpiAssessPenaltyStaff::getYn, Boolean.TRUE)
        );
    }

    public void calcLevelD(KpiAssessDetail kpiAssessDetail, YearMonth monthly,
                           List<Long> manageStaffIds, KpiAssessPenalty kpiAssessPenalty) {
        List<KpiAssessPenaltyStaff> penaltyDetails = new ArrayList<>();
        List<KpiPool> pools = kpiPoolService.list(Wrappers.<KpiPool>lambdaQuery()
                .in(KpiPool::getUserId, manageStaffIds)
                .eq(KpiPool::getMonthly, monthly)
                .eq(KpiPool::getInclusion, Boolean.TRUE)
                .eq(KpiPool::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isEmpty(pools)) {
            return ;
        }
        final BigDecimal penaltyScore = kpiAssessDetail.getPenaltyScore();
        final BigDecimal capPenaltyScore = kpiAssessDetail.getCapPenaltyScore();
        final BigDecimal penaltyMoney = kpiAssessDetail.getPenaltyMoney();
        final BigDecimal capPenaltyMoney = kpiAssessDetail.getCapPenaltyMoney();
        for (KpiPool pool : pools) {
            if (StarLevelEnum.D.equals(pool.getActualStar())) {
                penaltyDetails.add(convertDB(pool, Boolean.TRUE, penaltyScore, penaltyMoney));
                continue;
            }
            penaltyDetails.add(convertDB(pool, Boolean.TRUE, BigDecimal.ZERO, BigDecimal.ZERO));
        }
        BigDecimal score = penaltyDetails.stream().map(KpiAssessPenaltyStaff::getScore).reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal award = penaltyDetails.stream().map(KpiAssessPenaltyStaff::getAward).reduce(BigDecimal.ZERO, BigDecimal::add);
        score = score.compareTo(capPenaltyScore) > 0 ? capPenaltyScore : score;
        award = award.compareTo(capPenaltyMoney) > 0 ? capPenaltyMoney : award;
        //惩罚分数、金额为正数，转负数
        kpiAssessPenalty.setScore(score.negate());
        kpiAssessPenalty.setAward(award.negate());

        kpiAssessPenaltyService.save(kpiAssessPenalty);
        final Long kpiAssessPenaltyId = kpiAssessPenalty.getId();
        penaltyDetails.stream().forEach(penaltyDetail -> penaltyDetail.setKpiAssessPenaltyId(kpiAssessPenaltyId));
        kpiAssessPenaltyStaffService.saveBatch(penaltyDetails);
    }

    public void calcRankDiff(Integer rank, KpiAssessDetail kpiAssessDetail, LocalDate dataDate,
                           List<Long> manageStaffIds, KpiAssessPenalty kpiAssessPenalty) {
        final YearMonth monthly = YearMonth.from(dataDate);
        final BigDecimal penaltyScore = kpiAssessDetail.getPenaltyScore();
        final BigDecimal capPersonPenaltyScore = kpiAssessDetail.getCapPersonPenaltyScore().negate();
        final BigDecimal capPenaltyScore = kpiAssessDetail.getCapPenaltyScore().negate();
        final BigDecimal penaltyMoney = kpiAssessDetail.getPenaltyMoney();
        final BigDecimal capPersonPenaltyMoney = kpiAssessDetail.getCapPersonPenaltyMoney().negate();
        final BigDecimal capPenaltyMoney = kpiAssessDetail.getCapPenaltyMoney().negate();

        final BigDecimal awardScore = kpiAssessDetail.getAwardScore();
        final BigDecimal capPersonAwardScore = kpiAssessDetail.getCapPersonAwardScore();
        final BigDecimal capAwardScore = kpiAssessDetail.getCapAwardScore();
        final BigDecimal awardMoney = kpiAssessDetail.getAwardMoney();
        final BigDecimal capPersonAwardMoney = kpiAssessDetail.getCapPersonAwardMoney();
        final BigDecimal capAwardMoney = kpiAssessDetail.getCapAwardMoney();

        List<KpiAssessPenaltyStaff> penaltyDetails = new ArrayList<>();
        List<KpiGroupUser> turnPositiveStaffs = kpiGroupUserService.list(Wrappers.<KpiGroupUser>lambdaQuery()
                .in(KpiGroupUser::getUserId, manageStaffIds)
                .eq(KpiGroupUser::getDataDate, dataDate)
                .eq(KpiGroupUser::getIgnored, Boolean.TRUE)
                .eq(KpiGroupUser::getIgnoreCause, KpiIgnoreCauseEnum.TURN_POSITIVE_AFTER)
                .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isEmpty(turnPositiveStaffs)) {
            kpiAssessPenaltyService.save(kpiAssessPenalty);
            return;
        }
        List<Long> turnPositiveStaffIds = turnPositiveStaffs.stream().map(KpiGroupUser::getUserId).distinct().collect(Collectors.toList());
        List<KpiPool> pools = kpiPoolService.list(Wrappers.<KpiPool>lambdaQuery()
                .in(KpiPool::getUserId, turnPositiveStaffIds)
                .eq(KpiPool::getMonthly, monthly)
                .eq(KpiPool::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isEmpty(pools)) {
            kpiAssessPenaltyService.save(kpiAssessPenalty);
            return ;
        }
        for (KpiPool pool : pools) {
            Optional<Integer> assessRank = Optional.ofNullable(pool.getAssessRank());
            BigDecimal rankDiff = BigDecimal.ZERO;
            if (assessRank.isPresent()) {
                rankDiff = new BigDecimal(rank - assessRank.get());
            }
            log.info("人员质量评测计算排名差，目标排名：{}，绩效池：{}，结果：{}", rank, JSON.toJSONString(pool), rankDiff);
            KpiAssessPenaltyStaff penaltyStaff = null;
            //奖励
            if (rankDiff.compareTo(BigDecimal.ZERO) > 0) {
                BigDecimal poolAwardScore = rankDiff.multiply(awardScore);
                poolAwardScore = (capPersonAwardScore.compareTo(poolAwardScore) > 0) ? poolAwardScore : capPersonAwardScore;

                BigDecimal poolAwardMoney = rankDiff.multiply(awardMoney);
                poolAwardMoney = (capPersonAwardMoney.compareTo(poolAwardMoney) > 0) ? poolAwardMoney : capPersonAwardMoney;
                penaltyStaff = convertDB(pool, Boolean.FALSE, poolAwardScore, poolAwardMoney);

            }
            //处罚（负数）
            if (rankDiff.compareTo(BigDecimal.ZERO) < 0) {
                BigDecimal poolPenaltyScore = rankDiff.multiply(penaltyScore);
                poolPenaltyScore = (poolPenaltyScore.compareTo(capPersonPenaltyScore) > 0) ? poolPenaltyScore : capPersonPenaltyScore;

                BigDecimal poolPenaltyMoney = rankDiff.multiply(penaltyMoney);
                poolPenaltyMoney = (poolPenaltyMoney.compareTo(capPersonPenaltyMoney) > 0) ? poolPenaltyMoney : capPersonPenaltyMoney;
                penaltyStaff = convertDB(pool, Boolean.TRUE, poolPenaltyScore, poolPenaltyMoney);
            }

            if (rankDiff.compareTo(BigDecimal.ZERO) == 0) {
                penaltyStaff = convertDB(pool, Boolean.FALSE, BigDecimal.ZERO, BigDecimal.ZERO);
            }
            penaltyDetails.add(penaltyStaff);
        }
        BigDecimal userPenaltyScore = penaltyDetails.stream().filter(penalty -> Boolean.TRUE.equals(penalty.getPenalty()))
                .map(KpiAssessPenaltyStaff::getScore).reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal userPenaltyMoney = penaltyDetails.stream().filter(penalty -> Boolean.TRUE.equals(penalty.getPenalty()))
                .map(KpiAssessPenaltyStaff::getAward).reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal userAwardScore = penaltyDetails.stream().filter(penalty -> Boolean.FALSE.equals(penalty.getPenalty()))
                .map(KpiAssessPenaltyStaff::getScore).reduce(BigDecimal.ZERO, BigDecimal::add);
        BigDecimal userAwardMoney = penaltyDetails.stream().filter(penalty -> Boolean.FALSE.equals(penalty.getPenalty()))
                .map(KpiAssessPenaltyStaff::getAward).reduce(BigDecimal.ZERO, BigDecimal::add);

        userPenaltyScore = userPenaltyScore.compareTo(capPenaltyScore) > 0 ? userPenaltyScore : capPenaltyScore;
        userPenaltyMoney = userPenaltyMoney.compareTo(capPenaltyMoney) > 0 ? userPenaltyMoney : capPenaltyMoney;
        userAwardScore = userAwardScore.compareTo(capAwardScore) > 0 ? capAwardScore : userAwardScore;
        userAwardMoney = userAwardMoney.compareTo(capAwardMoney) > 0 ? capAwardMoney : userAwardMoney;

        kpiAssessPenalty.setScore(userPenaltyScore.add(userAwardScore));
        kpiAssessPenalty.setAward(userPenaltyMoney.add(userAwardMoney));
        if (kpiAssessPenalty.getScore().compareTo(BigDecimal.ZERO) == 0 && kpiAssessPenalty.getAward().compareTo(BigDecimal.ZERO) == 0) {
            return;
        }
        kpiAssessPenaltyService.save(kpiAssessPenalty);
        final Long kpiAssessPenaltyId = kpiAssessPenalty.getId();
        penaltyDetails.stream().forEach(penaltyDetail -> penaltyDetail.setKpiAssessPenaltyId(kpiAssessPenaltyId));
        kpiAssessPenaltyStaffService.saveBatch(penaltyDetails);
    }

    public KpiAssessPenaltyStaff convertDB(KpiPool pool, Boolean penalty, BigDecimal score, BigDecimal award) {
        return KpiAssessPenaltyStaff.builder()
                .kpiPoolId(pool.getId())
                .userId(pool.getUserId())
                .userName(pool.getUserName())
                .postId(pool.getPostId())
                .postName(pool.getPostName())
                .shopId(pool.getShopId())
                .shopName(pool.getShopName())
                .rank(pool.getAssessRank())
                .starLevel(pool.getActualStar())
                .penalty(penalty)
                .score(score)
                .award(award)
                .yn(Boolean.TRUE)
                .build();
    }

    public KpiAssessPenalty convertDB(Boolean expectedPenalty, String roleCode, KpiAssessDetail kpiAssessDetail, RpcUserRoleInfoDTO userRoleInfo, LocalDate dataDate, KpiAssess kpiAssess) {
        KpiAssessPenalty kpiAssessPenalty = KpiAssessPenalty.builder()
                .score(BigDecimal.ZERO)
                .award(BigDecimal.ZERO)
                .expectedPenalty(expectedPenalty)
                .kpiAssessId(kpiAssessDetail.getKpiAssessId())
                .kpiAssessDetailId(kpiAssessDetail.getId())
                .userId(userRoleInfo.getUserId())
                .userName(userRoleInfo.getUserName())
                .roleCode(roleCode)
                .dataDate(dataDate)
                .type(kpiAssess.getType())
                .groupId(kpiAssess.getGroupId())
                .build();
        if (PublicUtil.isNotEmpty(kpiAssess.getRank())) {
            kpiAssessPenalty.setRank(kpiAssess.getRank());
        }
        return kpiAssessPenalty;
    }

    /**
     * 推送奖惩数据
     *
     * @param monthly
     */
    public void pushAssessPunish(YearMonth monthly) {
        Date dataDate = DateUtil.localDate2Date(monthly.atEndOfMonth());
        List<KpiAssessPenalty> monthlyPenalties = kpiAssessPenaltyService.list(Wrappers.<KpiAssessPenalty>lambdaQuery()
                .eq(KpiAssessPenalty::getDataDate, monthly.atEndOfMonth())
                .eq(KpiAssessPenalty::getExpectedPenalty, Boolean.FALSE)
                .eq(KpiAssessPenalty::getYn, Boolean.TRUE)
        );

        Map<Long, List<KpiAssessPenalty>> penaltyMap = monthlyPenalties.stream().collect(Collectors.groupingBy(KpiAssessPenalty::getUserId));
        //推送
        for (Map.Entry<Long, List<KpiAssessPenalty>> entry : penaltyMap.entrySet()) {
            final Long userId = entry.getKey();
            List<KpiAssessPenalty> penalties = entry.getValue();
            BigDecimal score = penalties.stream().map(KpiAssessPenalty::getScore).reduce(BigDecimal.ZERO, BigDecimal::add);
            BigDecimal award = penalties.stream().map(KpiAssessPenalty::getAward).reduce(BigDecimal.ZERO, BigDecimal::add);

            if ((BigDecimal.ZERO.compareTo(score) > 0) || (BigDecimal.ZERO.compareTo(award) > 0)) {
                PunishStaffReportEvent punishStaff = createPunishStaff(userId, score, award, monthly, dataDate);
                punishStaffProducer.send(punishStaff);
            }
        }
    }

    public PunishStaffReportEvent createPunishStaff(Long userId, BigDecimal score, BigDecimal award, YearMonth monthly, Date dataDate) {
        PunishStaffReportEvent punishStaff = new PunishStaffReportEvent();
        punishStaff.setPerformance((BigDecimal.ZERO.compareTo(score) > 0) ? score.abs() : BigDecimal.ZERO);
        punishStaff.setFine((BigDecimal.ZERO.compareTo(award) > 0) ? award.abs() : BigDecimal.ZERO);
        punishStaff.setStaffId(userId);
        punishStaff.setKey(WorkTypeEnum.KPI_ASSESS.getKey());
        punishStaff.setReason(WorkTypeEnum.KPI_ASSESS.getName());
        punishStaff.setBusinessToken(getBusinessToken(userId, monthly));
        punishStaff.setCloseTime(dataDate);
        return punishStaff;
    }

    public String getBusinessToken(Long userId, YearMonth monthly) {
        String monthlyStr = DateTimeFormatter.ofPattern("yyyy-MM").format(monthly);
        return userId + Constant.SEPARATOR + monthlyStr;
    }


}
