package cn.fw.morax.service.biz.kpi;

import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.ThreadPoolUtil;
import cn.fw.morax.domain.bo.kpi.KpiGroupCalculableBO;
import cn.fw.morax.domain.bo.kpi.KpiIndicatorBO;
import cn.fw.morax.domain.db.kpi.*;
import cn.fw.morax.domain.enums.ScoreWayEnum;
import cn.fw.morax.service.biz.calculator.kpi.KpiAbsBaseCalculator;
import cn.fw.morax.service.data.kpi.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.BoundSetOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.concurrent.RejectedExecutionException;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.stream.Collectors;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : KpiCalculateService
 * @description : 绩效计算类
 * @date : 2022-04-15 11:40
 */
@Slf4j
@Service
public class KpiCalculateService {
    private final KpiGroupService kpiGroupService;
    private final KpiPoolService kpiPoolService;
    private final KpiPoolIndicatorValueService kpiPoolIndicatorValueService;
    private final KpiGroupUserService kpiGroupUserService;
    private final KpiGroupIndicatorService kpiGroupIndicatorService;
    private final KpiRewardPointService kpiRewardPointService;
    private final StringRedisTemplate stringRedisTemplate;
    private final KpiPoolCommonService kpiPoolCommonService;
    private final Map<ScoreWayEnum, KpiAbsBaseCalculator> calculatorMap;

    @Value("${spring.cache.custom.global-prefix}:calculable:kpi-group")
    @Getter
    private String key;

    @Value("${spring.cache.custom.global-prefix}:calculate-scoring-rate:kpi-group")
    @Getter
    private String calcuScoringRateKey;

    @Autowired
    public KpiCalculateService(final KpiGroupService kpiGroupService,
                               final KpiPoolService kpiPoolService,
                               final KpiPoolIndicatorValueService kpiPoolIndicatorValueService,
                               final KpiGroupUserService kpiGroupUserService,
                               final KpiGroupIndicatorService kpiGroupIndicatorService,
                               final KpiRewardPointService kpiRewardPointService,
                               final StringRedisTemplate stringRedisTemplate,
                               final KpiPoolCommonService kpiPoolCommonService,
                               final List<KpiAbsBaseCalculator> kpiCalculatorList) {
        this.kpiGroupService = kpiGroupService;
        this.kpiPoolService = kpiPoolService;
        this.kpiPoolIndicatorValueService = kpiPoolIndicatorValueService;
        this.kpiGroupUserService = kpiGroupUserService;
        this.kpiGroupIndicatorService = kpiGroupIndicatorService;
        this.kpiRewardPointService = kpiRewardPointService;
        this.stringRedisTemplate = stringRedisTemplate;
        this.kpiPoolCommonService = kpiPoolCommonService;
        this.calculatorMap = kpiCalculatorList.stream().collect(Collectors.toMap(KpiAbsBaseCalculator::getCalMethod, v -> v));
    }

    /**
     * 缓存需要计算绩效的绩效组信息
     *
     * @param localDate 日期
     */
    public void cacheCalculableKpiGroupIds(final LocalDate localDate) {
        log.info("定时任务【缓存需要计算绩效的绩效组】开始执行");
        Set<Long> kpiGroupIds = kpiGroupService.queryCalculableKpiGroupIds(localDate);
        if (CollectionUtils.isEmpty(kpiGroupIds)) {
            return;
        }
        log.info("缓存需要计算绩效的绩效组信息，kpiGroupIds：{}，日期：{}", JSON.toJSONString(kpiGroupIds), localDate);
        cacheGroupIds(kpiGroupIds, localDate);
    }

    /**
     * 计算绩效
     */
    public void calculateKpi() {
        BoundSetOperations<String, String> setOps = stringRedisTemplate.boundSetOps(getKey());
        ThreadPoolExecutor threadPool = ThreadPoolUtil.getInstance().getThreadPool();
        List<String> overflowsList = new ArrayList<>();
        String str;
        while ((str = setOps.pop()) != null) {
            final KpiGroupCalculableBO bo = JSONObject.parseObject(str, KpiGroupCalculableBO.class);
            if (Objects.isNull(bo)) {
                continue;
            }
            try {
                String finalStr = str;
                threadPool.execute(() -> {
                    try {
                        calculateKpi(bo);
                    } catch (Exception ex) {
                        log.error("计算绩效数据失败：{}", bo, ex);
                        setOps.add(finalStr);
                    }
                });
            } catch (RejectedExecutionException re) {
                overflowsList.add(str);
            }
        }
        if (!CollectionUtils.isEmpty(overflowsList)) {
            for (String s : overflowsList) {
                setOps.add(s);
            }
        }
    }

    /**
     * 计算对应日期绩效组的绩效数据
     * 按组计算
     *
     * @param bo
     */

    @Transactional(rollbackFor = Exception.class)
    public void calculateKpi(KpiGroupCalculableBO bo) {
        final Long kpiGroupId = bo.getKpiGroupId();
        KpiGroup kpiGroup = kpiGroupService.getById(kpiGroupId);
        if (Objects.isNull(kpiGroup)) {
            log.error("绩效组[{}]不存在，终止计算！", kpiGroupId);
            return;
        }
        if (!Boolean.TRUE.equals(kpiGroup.getYn())) {
            log.error("绩效组[{}]已被删除，终止计算！", kpiGroupId);
            return;
        }
        final LocalDate localDate = bo.getLocalDate();
        List<KpiGroupUser> userList = kpiGroupUserService.list(Wrappers.<KpiGroupUser>lambdaQuery()
                .eq(KpiGroupUser::getKpiGroupId, kpiGroupId)
                .eq(KpiGroupUser::getDataDate, localDate)
                .eq(KpiGroupUser::getYn, Boolean.TRUE)
        );
        if (CollectionUtils.isEmpty(userList)) {
            log.error("绩效组[{}]人员为空，终止计算！", kpiGroupId);
            return;
        }
        userList.forEach(this::calcUserKpi);
        BoundListOperations<String, String> listOps = stringRedisTemplate.boundListOps(getCalcuScoringRateKey());
        listOps.rightPush(JSONObject.toJSONString(bo));
    }

    /**
     * 计算绩效组人员的绩效数据
     * 计算单个人员绩效数据
     *
     * @param user
     */
    private void calcUserKpi(KpiGroupUser user) {
        KpiPool kpiPool = kpiPoolCommonService.inspectionPool(user);
//        if (Boolean.TRUE.equals(user.getIgnored())) {
//            return;
//        }
        final Long poolId = kpiPool.getId();
        Long kpiGroupId = kpiPool.getKpiGroupId();
        Set<KpiIndicatorBO> indicatorSet = kpiGroupIndicatorService.allIndicators(DateUtil.localDate2Date(user.getDataDate()), kpiGroupId);
        if (CollectionUtils.isEmpty(indicatorSet)) {
            return;
        }
        List<KpiPoolIndicatorValue> poolIndicatorValueList = new ArrayList<>();
        for (KpiIndicatorBO kpiIndicatorBO : indicatorSet) {
            KpiAbsBaseCalculator calculator = calculatorMap.get(kpiIndicatorBO.getScoreWay());
            if (calculator == null) {
                continue;
            }
            BigDecimal score = calculator.calculate(user, kpiIndicatorBO);
            KpiPoolIndicatorValue indicatorValue = createPoolIndicatorValue(score, kpiIndicatorBO);
            indicatorValue.setDataDate(user.getDataDate());
            indicatorValue.setUserId(user.getUserId());
            indicatorValue.setKpiPoolId(poolId);
            poolIndicatorValueList.add(indicatorValue);
        }

        if (CollectionUtils.isNotEmpty(poolIndicatorValueList)) {
            kpiPoolIndicatorValueService.remove(Wrappers.<KpiPoolIndicatorValue>lambdaQuery()
                    .eq(KpiPoolIndicatorValue::getUserId, user.getUserId())
                    .eq(KpiPoolIndicatorValue::getDataDate, user.getDataDate())
                    .eq(KpiPoolIndicatorValue::getKpiPoolId, poolId)
                    .eq(KpiPoolIndicatorValue::getYn, Boolean.TRUE));

            kpiPoolIndicatorValueService.saveBatch(poolIndicatorValueList);
        }
        BigDecimal totalScore = BigDecimal.ZERO;
        for (KpiPoolIndicatorValue indicatorValue : poolIndicatorValueList) {
            totalScore = totalScore.add(indicatorValue.getIndicatorScore());
        }
        calcTotalScore(totalScore, kpiPool, user.getDataDate());
        kpiPoolService.updateById(kpiPool);
    }

    private KpiPoolIndicatorValue createPoolIndicatorValue(BigDecimal score, KpiIndicatorBO bo) {
        KpiPoolIndicatorValue indicatorValue = new KpiPoolIndicatorValue();
        indicatorValue.setKpiGroupIndicatorId(bo.getKpiGroupIndicatorId());
        indicatorValue.setIndicatorScore(score);
        indicatorValue.setYn(Boolean.TRUE);
        indicatorValue.setGroupId(bo.getGroupId());
        return indicatorValue;
    }

    private void cacheGroupIds(final Set<Long> kpiGroupIds, final LocalDate localDate) {
        String[] array = kpiGroupIds.stream()
                .map(kpiGroupId -> new KpiGroupCalculableBO(kpiGroupId, localDate))
                .map(JSONObject::toJSONString)
                .toArray(String[]::new);
        stringRedisTemplate.opsForSet().add(getKey(), array);
    }

    public static void main(String[] args) {
        System.out.println(LocalDateTime.of(LocalDate.now(), LocalTime.MAX));
    }

    /**
     * 处理奖惩的绩效分
     *
     * @param baseScore
     * @param pool
     */
    private void calcTotalScore(BigDecimal baseScore, KpiPool pool, LocalDate dataDate) {
        //取创建时间在今天之前的数据
        List<KpiRewardPoint> list = kpiRewardPointService.list(Wrappers.<KpiRewardPoint>lambdaQuery()
                .eq(KpiRewardPoint::getUserId, pool.getUserId())
                .eq(KpiRewardPoint::getPostId, pool.getPostId())
                .eq(KpiRewardPoint::getShopId, pool.getShopId())
                .eq(KpiRewardPoint::getMonthly, pool.getMonthly())
                .le(KpiRewardPoint::getCreateTime, LocalDateTime.of(dataDate, LocalTime.MAX))
                .eq(KpiRewardPoint::getGroupId, pool.getGroupId())
        );
        if (CollectionUtils.isEmpty(list)) {
            pool.setKpiScore(baseScore);
            return;
        }

        BigDecimal resScore = list.stream().map(KpiRewardPoint::getScorePoint).reduce(BigDecimal.ZERO, BigDecimal::add);
        resScore = baseScore.add(resScore);
        if (resScore.compareTo(BigDecimal.ZERO) < 0) {
            resScore = BigDecimal.ZERO;
        }
        pool.setKpiScore(resScore);
    }
}
