package cn.fw.morax.service.biz.salary;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.morax.common.pojo.ExcelFieldData;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.ExcelDataUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.vo.salary.StaffPersonTaxVO;
import cn.fw.morax.domain.vo.salary.StaffSalaryDetailVO;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.morax.sdk.dto.salary.GeneralSalaryProjectReq;
import cn.fw.morax.service.biz.CommonService;
import cn.hutool.poi.excel.ExcelReader;
import cn.hutool.poi.excel.ExcelUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

@RequiredArgsConstructor
@Service
@Slf4j
public class SalaryExtraBizService {

    private final SalaryReportService salaryReportService;
    private final EhrRpcService ehrRpcService;

    /**
     * 个税模板文件
     *
     * @param request
     * @param response
     */
    public void personTaxTemplateFile(HttpServletRequest request, HttpServletResponse response) {
        String fileName = "个税模板.xlsx";
        String encodeFileName = CommonService.getEncodeName(request, fileName);
        CommonService.downloadTemplateExcel(response, encodeFileName, StaffPersonTaxVO.class);
    }

    /**
     * 上传个税信息
     *
     * @param file
     * @param user
     */
    public List<StaffPersonTaxVO> uploadPersonTax(MultipartFile file, LoginAuthBean user) {
        List<StaffPersonTaxVO> staffPersonTaxVOS = null;
        try {
            ExcelReader reader = ExcelUtil.getReader(file.getInputStream());
            Map<String, String> headerAlias = ExcelDataUtil.getTitleMappingField(StaffPersonTaxVO.class);
            reader.setHeaderAlias(headerAlias);
            staffPersonTaxVOS = reader.readAll(StaffPersonTaxVO.class);
            this.checkPersonTax(staffPersonTaxVOS);
            this.pushExtraSalary(user.getGroupId(), staffPersonTaxVOS);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return staffPersonTaxVOS;
    }

    /**
     * 推送额外工资
     *
     * @param groupId
     * @param personTaxs
     */
    public void pushExtraSalary(Long groupId, List<StaffPersonTaxVO> personTaxs) {
        if (PublicUtil.isEmpty(personTaxs)) {
            return;
        }
        LocalDateTime lastMonthTime = YearMonth.now().plusMonths(-1).atEndOfMonth().atTime(23, 59,59);
        Date date = DateUtil.localDateTime2Date(lastMonthTime);
        List<String> idNumbers = personTaxs.stream().map(StaffPersonTaxVO::getIdNumber).distinct().collect(Collectors.toList());
        List<StaffInfoDTO> staffs = ehrRpcService.queryStaffInfoByIdNumbers(groupId, idNumbers);
        Map<String, StaffInfoDTO> idNumberStaffMap = staffs.stream().collect(Collectors.toMap(StaffInfoDTO::getIdNumber, Function.identity(), (v1, v2) -> v1));

        for (StaffPersonTaxVO staffPersonTax : personTaxs) {
            if (! idNumberStaffMap.containsKey(staffPersonTax.getIdNumber())) {
                log.error("根据身份证号未能找到员工信息： 姓名：{}，身份证：{}", staffPersonTax.getName(), staffPersonTax.getIdNumber());
                continue;
            }
            StaffInfoDTO staff = idNumberStaffMap.get(staffPersonTax.getIdNumber());
            GeneralSalaryProjectReq projectReq = new GeneralSalaryProjectReq();
            projectReq.setUserId(staff.getId());
            projectReq.setPostId(staff.getPostId());
            projectReq.setShopId(staff.getShopId());
            projectReq.setDataTime(date);
            projectReq.setAmount(staffPersonTax.getTax());
//            projectReq.setDataId(projectReq.getDataId());
//            projectReq.setDataName(projectReq.getDataName());
            salaryReportService.personTaxProject(projectReq);
        }

    }

    /**
     * 检查员工个税
     *
     * @param staffPersonTaxVOS
     */
    private void checkPersonTax(List<StaffPersonTaxVO> staffPersonTaxVOS) {
        if (PublicUtil.isEmpty(staffPersonTaxVOS)) {
            throw new BusinessException("Excel数据不能被解析");
        }

        for (StaffPersonTaxVO staffPersonTax : staffPersonTaxVOS) {
            if (PublicUtil.isEmpty(staffPersonTax)) {
                throw new BusinessException("员工信息为空");
            }
            if (PublicUtil.isEmpty(staffPersonTax.getIdNumber())) {
                throw new BusinessException("身份证信息不能为空");
            }
            if (PublicUtil.isEmpty(staffPersonTax.getTax())) {
                throw new BusinessException("个税信息不能为空");
            }
        }

    }

}
