package cn.fw.morax.service.biz.salary;

import cn.fw.ehr.sdk.api.enums.StaffShopTypeEnum;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.salary.SalaryGroupUser;
import cn.fw.morax.domain.db.salary.SalaryPool;
import cn.fw.morax.domain.enums.StarLevelEnum;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.rpc.ehr.dto.StaffShopInfoDTO;
import cn.fw.morax.service.data.salary.SalaryGroupUserService;
import cn.fw.morax.service.data.salary.SalaryPoolService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.temporal.TemporalAdjusters;
import java.util.Objects;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryPoolCommonService
 * @description : 绩效池公用方法
 * @date : 2022-09-27 11:40
 */
@Slf4j
@Service
@AllArgsConstructor
public class SalaryPoolCommonService {

    private SalaryGroupUserService salaryGroupUserService;
    private SalaryPoolService salaryPoolService;
    private EhrRpcService ehrRpcService;

    /**
     * 查询薪酬池
     *
     * @param user
     * @return
     */
    public SalaryPool inspectionPool(SalaryGroupUser user) {
        SalaryPool salaryPool = salaryPoolService.getOne(Wrappers.<SalaryPool>lambdaQuery()
                        .eq(SalaryPool::getSgc, user.getSgc())
                        .eq(SalaryPool::getUserId, user.getUserId())
                        .eq(SalaryPool::getMonthly, YearMonth.from(user.getDataDate()))
                        .eq(SalaryPool::getYn, Boolean.TRUE)
                , Boolean.FALSE);
        //先按照薪酬组查询，查不到通过门店、岗位查（调岗，同岗位、同门店不算绩效组变动，不生成新绩效池数据）
        if (Objects.isNull(salaryPool)) {
            salaryPool = salaryPoolService.getOne(Wrappers.<SalaryPool>lambdaQuery()
                            .eq(SalaryPool::getUserId, user.getUserId())
                            .eq(SalaryPool::getPostId, user.getPostId())
                            .eq(SalaryPool::getShopId, user.getShopId())
                            .eq(SalaryPool::getMonthly, YearMonth.from(user.getDataDate()))
                            .eq(SalaryPool::getYn, Boolean.TRUE)
                    , Boolean.FALSE);
            if (Objects.isNull(salaryPool)) {
                salaryPool = this.createPoolData(user);
            } else {
                salaryPool = this.modifyPoolData(salaryPool, user);
            }
        } else if ((!salaryPool.getShopId().equals(user.getShopId())) ||
                (!salaryPool.getSalaryGroupId().equals(user.getSalaryGroupId()))) {
            //若门店信息与员工在职门店不匹配，说明员工是调门店，绩效组没变
            //若绩效组id不匹配，说明绩效组编辑生成了新绩效组，对应绩效池中的绩效组id修改
            salaryPool = this.modifyPoolData(salaryPool, user);
        }
        return salaryPool;
    }

    /**
     * 获取员工社保公积金门店
     *
     * @param userId
     */
    private StaffShopInfoDTO getFundShopInfo(Long userId) {
        StaffBaseInfoDTO staff = ehrRpcService.queryStaffBaseInfo(userId);
        if (PublicUtil.isEmpty(staff) || PublicUtil.isEmpty(staff.getStaffShopList())) {
            log.error("员工没有社保公积金门店，userId:{}", userId);
        }
        return staff.getStaffShopList().stream()
                .filter(shop -> StaffShopTypeEnum.FUND_SHOP.getValue().equals(shop.getType()))
                .findFirst()
                .orElse(null);

    }

    @Transactional(rollbackFor = Exception.class)
    public SalaryPool modifyPoolData(SalaryPool pool, SalaryGroupUser user) {
        pool.setSalaryGroupId(user.getSalaryGroupId());
        pool.setShopId(user.getShopId());
        pool.setShopName(user.getShopName());
        pool.setSgc(user.getSgc());
        salaryPoolService.updateById(pool);
        return pool;
    }

    @Transactional(rollbackFor = Exception.class)
    public SalaryPool createPoolData(SalaryGroupUser user) {
        SalaryPool salaryPool = new SalaryPool();
        salaryPool.setSgc(user.getSgc());
        salaryPool.setSalaryGroupId(user.getSalaryGroupId());
        salaryPool.setUserId(user.getUserId());
        salaryPool.setUserName(user.getUserName());
        salaryPool.setPostId(user.getPostId());
        salaryPool.setPostName(user.getPostName());
        salaryPool.setShopId(user.getShopId());
        salaryPool.setShopName(user.getShopName());
        salaryPool.setStarLevel(StarLevelEnum.C);
        salaryPool.setReward(BigDecimal.ZERO);
        salaryPool.setDataDate(user.getDataDate());
        salaryPool.setMonthly(YearMonth.from(user.getDataDate()));
        salaryPool.setRegular(Boolean.FALSE);
        salaryPool.setPaid(Boolean.FALSE);
        salaryPool.setGroupId(user.getGroupId());
        salaryPool.setYn(Boolean.TRUE);
        StaffShopInfoDTO staffShopInfoDTO = this.getFundShopInfo(user.getUserId());
        if (PublicUtil.isNotEmpty(staffShopInfoDTO)) {
            salaryPool.setFundShopId(staffShopInfoDTO.getShopId());
            salaryPool.setFundShopName(staffShopInfoDTO.getShopName());
        }
        salaryPoolService.save(salaryPool);
        return salaryPool;
    }

    /**
     * 查询工作天数占比
     *
     * @param userId
     * @param sgc
     * @param date
     * @return
     */
    public BigDecimal getWorkDayProportion(Long userId, String sgc, LocalDate date) {
        BigDecimal workDayProportion = BigDecimal.ZERO;
        int count = salaryGroupUserService.count(Wrappers.<SalaryGroupUser>lambdaQuery()
                .eq(SalaryGroupUser::getUserId, userId)
                .eq(SalaryGroupUser::getSgc, sgc)
                .eq(SalaryGroupUser::getProbationer, Boolean.FALSE)
                .eq(SalaryGroupUser::getFrozen, Boolean.FALSE)
                .eq(SalaryGroupUser::getYn, Boolean.TRUE)
                .ge(SalaryGroupUser::getDataDate, date.with(TemporalAdjusters.firstDayOfMonth()))
                .le(SalaryGroupUser::getDataDate, date)
        );
        if (count > 0) {
            int lengthOfMonth = YearMonth.from(date).lengthOfMonth();
            workDayProportion = new BigDecimal(count).divide(new BigDecimal(lengthOfMonth), 4, RoundingMode.DOWN);
            log.info("上班天数【" + count + "】总天数【" + lengthOfMonth + "】比例为：" + workDayProportion);
        }
        return workDayProportion;
    }

}
