package cn.fw.morax.service.biz.salary;

import cn.fw.common.exception.BusinessException;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.ehr.sdk.api.enums.RoleManagerEnum;
import cn.fw.ehr.sdk.api.result.PostRoleVo;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.salary.SalaryGroup;
import cn.fw.morax.domain.db.salary.SalaryProject;
import cn.fw.morax.domain.dto.query.SalaryIndicatorQueryDTO;
import cn.fw.morax.domain.dto.query.SalaryProjectPostQueryDTO;
import cn.fw.morax.domain.dto.salary.SalaryProjectDTO;
import cn.fw.morax.domain.enums.ApplicableRoleEnum;
import cn.fw.morax.domain.enums.SalaryCalMethodEnum;
import cn.fw.morax.domain.vo.salary.SalaryProjectSelectorVO;
import cn.fw.morax.domain.vo.salary.SalaryProjectVO;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.PostInfoDTO;
import cn.fw.morax.rpc.ehr.dto.PostRoleDTO;
import cn.fw.morax.service.data.salary.SalaryProjectService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

@RequiredArgsConstructor
@Service
@Slf4j
public class SalaryProjectBizService {

    private final SalaryGroupBizService salaryGroupBizService;
    private final SalaryProjectService salaryProjectService;
    private final EhrRpcService ehrRpcService;

    @Value("${spring.cache.custom.global-prefix}:salary:project:save:")
    @Getter
    private String savePrefix;
    @Value("${spring.cache.custom.global-prefix}:salary:project:enable:")
    @Getter
    private String enablePrefix;

    /**
     * 分页查询
     *
     * @param dto
     * @return
     */
    public AppPage<SalaryProjectVO> salaryIndicatorsPage(SalaryIndicatorQueryDTO dto) {
//        PageData<SalaryProject> pageData = salaryProjectService.page(new PageData<SalaryProject>(dto.getCurrent(), dto.getPageSize()),
//                Wrappers.<SalaryProject>lambdaQuery()
//                        .and(PublicUtil.isNotEmpty(dto.getKeyword()), w ->
//                                w.like(SalaryProject::getSalaryCode, dto.getKeyword())
//                                        .or().like(SalaryProject::getSalaryName, dto.getKeyword())
//                        )
//                        .apply(PublicUtil.isNotEmpty(dto.getRoleCode()),
//                                MessageFormatUtil.transferWithQuotationMarks("FIND_IN_SET({0}, role_codes)", dto.getRoleCode()))
//                        .eq(SalaryProject::getYn, Boolean.TRUE)
//        );
//        return PublicUtil.toPage(pageData, salaryProject -> PublicUtil.copy(salaryProject, SalaryProjectVO.class));


        if (PublicUtil.isNotEmpty(dto.getRoleCode())) {
            List<ApplicableRoleEnum> roleTypes = new ArrayList<ApplicableRoleEnum>() {{
                add(ApplicableRoleEnum.ALL);
                add(ApplicableRoleEnum.MANAGEMENT_ROLE);
            }};
            dto.setAllRoleTypes(roleTypes);
        }
        Long total = salaryProjectService.indicatorsPageTotal(dto);
        return PublicUtil.<SalaryProjectVO, SalaryIndicatorQueryDTO>queryPage(dto, total, new Function<SalaryIndicatorQueryDTO, List<SalaryProjectVO>>() {
            @Override
            public List<SalaryProjectVO> apply(SalaryIndicatorQueryDTO basePageQuery) {
                return salaryProjectService.indicatorsPage(dto);
            }
        });
    }

    /**
     * 保存指标
     *
     * @param dto
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getSavePrefix()", key = "#dto.getSalaryName()", message = "保存中，请勿重复操作")
    public void saveSalaryIndicator(SalaryProjectDTO dto) {
        this.checkIndicatorRole(dto);
        this.checkIndicatorName(dto.getId(), dto.getSalaryName());
        SalaryProject salaryProject = this.convertToPo(dto);
        if (PublicUtil.isEmpty(dto.getId())) {
            String salaryCode = PublicUtil.getUUID();
            this.checkSalaryCode(salaryCode);
            salaryProject.setSalaryCode(salaryCode);
        } else {
            SalaryProject originSalaryProject = salaryProjectService.getById(dto.getId());
            BV.notNull(originSalaryProject, "指标不存在，请重试");
            salaryProject.setSalaryCode(originSalaryProject.getSalaryCode());
            salaryProject.setId(originSalaryProject.getId());
        }

        salaryProjectService.saveOrUpdate(salaryProject);
    }

    /**
     * 转换为db对象
     *
     * @return
     */
    public SalaryProject convertToPo(SalaryProjectDTO dto) {
        SalaryProject salaryProject = SalaryProject.builder()
                .salaryName(dto.getSalaryName())
                .optionalMethod(dto.getOptionalMethod())
                .roleType(dto.getRoleType())
                .roleNames(dto.getRoleNames())
                .roleCodes(dto.getRoleCodes())
                .enable(Boolean.FALSE)
                .visible(dto.getVisible())
                .roleType(dto.getRoleType())
                .build();
        if (ApplicableRoleEnum.PORTION.equals(dto.getRoleType())) {
            if (PublicUtil.isNotEmpty(dto.getRoleCodes())) {
                salaryProject.setRoleCodes(dto.getRoleCodes());
            }
            if (PublicUtil.isNotEmpty(dto.getRoleNames())) {
                salaryProject.setRoleNames(dto.getRoleNames());
            }
        }
        return salaryProject;
    }

    /**
     * 薪酬指标选择器
     *
     * @param postId
     * @param shopIds
     * @return
     */
    public List<SalaryProjectSelectorVO> indicatorSelector(Long postId, String shopIds) {
        List<Long> postIds = new ArrayList<Long>() {{
            add(postId);
        }};
        List<PostInfoDTO> postInfoDTOS = ehrRpcService.getPostInfoList(postIds);
        List<String> roleCodes = postInfoDTOS.stream()
                .map(PostInfoDTO::getRoleList)
                .filter(Objects::nonNull)
                .flatMap(Collection::stream)
                .map(PostRoleDTO::getRoleCode)
                .distinct().collect(Collectors.toList());
        Boolean queryManagerRole = hasManagerRole(roleCodes);
        SalaryProjectPostQueryDTO dto = new SalaryProjectPostQueryDTO(roleCodes, queryManagerRole);
        dto.setEnable(Boolean.TRUE);
        List<SalaryProjectSelectorVO> selectorVOS = salaryProjectService.getPostIndicators(dto);
        selectorVOS.stream().filter(vo -> PublicUtil.isNotEmpty(vo.getOptionalMethod())).forEach(selectorVO -> {
            List<SalaryProjectSelectorVO.OptionalMethod> optionalMethods = new ArrayList<>();
            selectorVO.getOptionalMethod().stream().forEach(value -> {
                SalaryCalMethodEnum calMethodEnum = SalaryCalMethodEnum.ofValue(value);
                if (PublicUtil.isNotEmpty(calMethodEnum)) {
                    optionalMethods.add(selectorVO.new OptionalMethod(calMethodEnum.getValue(), calMethodEnum.getName()));
                }
            });
            selectorVO.setOptionalMethods(optionalMethods);
        });
        return selectorVOS;
    }

    /**
     * 薪酬指标选择器
     *
     * @return
     */
    public List<SalaryProjectSelectorVO> allIndicators() {
        List<SalaryProject> salaryProjects = salaryProjectService.list(Wrappers.<SalaryProject>lambdaQuery()
                .eq(SalaryProject::getYn, Boolean.TRUE)
        );
        List<SalaryProjectSelectorVO> selectorVOS = PublicUtil.copyList(salaryProjects, SalaryProjectSelectorVO.class);
        selectorVOS.stream().filter(vo -> PublicUtil.isNotEmpty(vo.getOptionalMethod())).forEach(selectorVO -> {
            List<SalaryProjectSelectorVO.OptionalMethod> optionalMethods = new ArrayList<>();
            selectorVO.getOptionalMethod().stream().forEach(value -> {
                SalaryCalMethodEnum calMethodEnum = SalaryCalMethodEnum.ofValue(value);
                if (PublicUtil.isNotEmpty(calMethodEnum)) {
                    optionalMethods.add(selectorVO.new OptionalMethod(calMethodEnum.getValue(), calMethodEnum.getName()));
                }
            });
            selectorVO.setOptionalMethods(optionalMethods);
        });
        return selectorVOS;
    }

    /**
     * 是否拥有管理角色
     *
     * @return
     */
    private Boolean hasManagerRole(List<String> roleCodes) {
        if (PublicUtil.isEmpty(roleCodes)) {
            return Boolean.FALSE;
        }
        HashSet<String> roleCodeSet = new HashSet<>(roleCodes);
        return Arrays.stream(RoleManagerEnum.values())
                .anyMatch(roleManagerEnum -> roleCodeSet.contains(roleManagerEnum.getCode()));
    }

    /**
     * 检查指标角色配置
     *
     * @param dto
     */
    private void checkIndicatorRole(SalaryProjectDTO dto) {
        Boolean portionNotSelectRole = ApplicableRoleEnum.PORTION.equals(dto.getRoleType()) &&
                PublicUtil.isEmpty(dto.getRoleNames());
        BV.isFalse(portionNotSelectRole, "自定义角色，请选择至少一个角色");
    }

    /**
     * 检查指标名称
     *
     * @param indicatorName
     */
    private void checkIndicatorName(Long id, String indicatorName) {
        long taskCount = salaryProjectService.count(Wrappers.<SalaryProject>lambdaQuery()
                .eq(SalaryProject::getSalaryName, indicatorName)
                .eq(SalaryProject::getYn, Boolean.TRUE)
                .ne(PublicUtil.isNotEmpty(id), SalaryProject::getId, id)
        );
        BV.isFalse(taskCount > 0, "指标名称已存在，请重新输入");
    }

    /**
     * 检查薪酬项编码
     *
     * @param salaryCode
     */
    private void checkSalaryCode(String salaryCode) {
        long taskCount = salaryProjectService.count(Wrappers.<SalaryProject>lambdaQuery()
                .eq(SalaryProject::getSalaryCode, salaryCode)
                .eq(SalaryProject::getYn, Boolean.TRUE)
        );
        if (taskCount > 0) {
            log.error("薪酬项编码生成重复：{}", salaryCode);
            throw new BusinessException("薪酬项编码已存在，请重试");
        }
    }

    /**
     * 薪酬项详情
     *
     * @param salaryProjectId
     * @return
     */
    public SalaryProjectVO getSalaryIndicatorById(Long salaryProjectId) {
        SalaryProject salaryProject = salaryProjectService.getById(salaryProjectId);
        BV.notNull(salaryProject, "薪酬项不存在");
        SalaryProjectVO salaryProjectVo = PublicUtil.copy(salaryProject, SalaryProjectVO.class);
        return salaryProjectVo;
    }

    /**
     * 薪酬指标库禁用启用
     *
     * @param salaryProjectId
     * @param enable
     */
    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getEnablePrefix()", key = "#salaryProjectId", message = "修改中，请勿重复操作")
    public void enableIndicator(Long salaryProjectId, Boolean enable) {
        SalaryProject salaryProject = salaryProjectService.getById(salaryProjectId);
        BV.notNull(salaryProject, "薪酬指标不存在，请重试");
        BV.isFalse(enable.equals(salaryProject.getEnable()), "薪酬指标已经是" + ((enable) ? "启用" : "禁用") + "状态，请重试");
        //禁用情况：正在使用的指标不能禁用，并提示哪些地方在用
        if (!enable) {
            List<SalaryGroup> salaryGroups = salaryGroupBizService.getSalaryGroupByIndicatorId(salaryProjectId);
            Set<String> postNames = salaryGroups.stream().map(SalaryGroup::getPostName).collect(Collectors.toSet());
            if (PublicUtil.isNotEmpty(postNames)) {
                throw new BusinessException(String.join(",", postNames) + "岗位正在使用此薪酬指标，不能禁用");
            }
        }
        salaryProjectService.update(Wrappers.<SalaryProject>lambdaUpdate()
                .set(SalaryProject::getEnable, enable)
                .eq(SalaryProject::getId, salaryProjectId)
        );
    }


}
