package cn.fw.morax.service.biz.salary;

import cn.fw.common.exception.BusinessException;
import cn.fw.ehr.sdk.api.enums.StaffShopTypeEnum;
import cn.fw.morax.common.constant.Constant;
import cn.fw.morax.common.utils.DateUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.PayrollRecord;
import cn.fw.morax.domain.db.salary.*;
import cn.fw.morax.domain.dto.salary.SalaryExtraDTO;
import cn.fw.morax.domain.enums.ExtraSalaryTypeEnum;
import cn.fw.morax.domain.enums.SalaryPayrollStatusEnum;
import cn.fw.morax.rpc.ehr.EhrRpcService;
import cn.fw.morax.rpc.ehr.dto.StaffBaseInfoDTO;
import cn.fw.morax.rpc.ehr.dto.StaffShopInfoDTO;
import cn.fw.morax.rpc.oop.OopRpcService;
import cn.fw.morax.rpc.oop.dto.ShopDTO;
import cn.fw.morax.sdk.dto.CustomList;
import cn.fw.morax.sdk.dto.salary.*;
import cn.fw.morax.service.data.PayrollRecordService;
import cn.fw.morax.service.data.salary.*;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.YearMonth;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalaryReportService
 * @description : 薪酬数据上报
 * @date : 2022-04-22 10:28
 */

@RequiredArgsConstructor
@Slf4j
@Service
public class SalaryReportService {

    private final SalaryExtraProjectService salaryExtraProjectService;
    private final SalaryGroupUserService salaryGroupUserService;
    private final SalaryPoolService salaryPoolService;
    private final OopRpcService oopRpcService;
    private final EhrRpcService ehrRpcService;
    private final PayrollRecordService payrollRecordService;
    private final SalaryProjectService salaryProjectService;
    private final SalaryGroupService salaryGroupService;
    private final SalaryPoolCommonService salaryPoolCommonService;
    private final StringRedisTemplate stringRedisTemplate;
    private final PlatformTransactionManager platformTransactionManager;
    private final TransactionDefinition transactionDefinition;
    private final SalaryGeneralSettinService salaryGeneralSettinService;
    private final SalaryClosureService salaryClosureService;
    private final SalarySettingCommonService salarySettingCommonService;
    @Value("${spring.cache.custom.global-prefix}:extra-salary")
    @Getter
    private String extraSalaryKey;


    /**
     * 额外薪资数据保存失败的重试
     */
    public void retrySaveReward() {
        ListOperations<String, String> listOps = stringRedisTemplate.opsForList();
        String str;
        List<String> failList = new ArrayList<>();
        while ((str = listOps.leftPop(getExtraSalaryKey())) != null) {
            try {
                SalaryExtraDTO dto = JSONObject.parseObject(str, SalaryExtraDTO.class);
                if (Objects.isNull(dto)) {
                    continue;
                }
                saveExtraProjectTX(dto);
            } catch (Exception e) {
                failList.add(str);
                log.error("薪酬额外薪资数据保存失败", e);
            }
        }

        if (!CollectionUtils.isEmpty(failList)) {
            listOps.rightPushAll(getExtraSalaryKey(), failList);
        }
    }


    /**
     * 奖惩项薪资上报
     *
     * @param req
     */
    public void salaryRewardReport(SalaryRewardReq req) {
        SalaryExtraDTO extraDTO = transferExtraProject(req);
        try {
            saveExtraProjectTX(extraDTO);
        } catch (Exception e) {
            cacheFailProject(extraDTO, getExtraSalaryKey());
            log.error("奖惩项薪资上报数据处理失败", e);
        }
    }

    /**
     * 社保项上报
     *
     * @param req
     */
    public void socialSecurityProject(GeneralSalaryProjectReq req) {
        SalaryExtraDTO extraDTO = transferExtraProject(req, ExtraSalaryTypeEnum.NECESSARY);
        try {
            saveExtraProjectTX(extraDTO);
        } catch (Exception e) {
            cacheFailProject(extraDTO, getExtraSalaryKey());
            log.error("社保项上报数据处理失败", e);
        }
    }

    /**
     * 个税上报
     *
     * @param req
     */
    public void personTaxProject(GeneralSalaryProjectReq req) {
        SalaryExtraDTO extraDTO = transferExtraProject(req, ExtraSalaryTypeEnum.PERSON_TAX);
        try {
            saveExtraProjectTX(extraDTO);
        } catch (Exception e) {
            cacheFailProject(extraDTO, getExtraSalaryKey());
            log.error("社保项上报数据处理失败", e);
        }
    }

    /**
     * 补贴项上报
     *
     * @param req
     */
    public void subsidyProject(GeneralSalaryProjectReq req) {
        SalaryExtraDTO extraDTO = transferExtraProject(req, ExtraSalaryTypeEnum.SUBSIDY);
        try {
            saveExtraProjectTX(extraDTO);
        } catch (Exception e) {
            cacheFailProject(extraDTO, getExtraSalaryKey());
            log.error("补贴项薪资上报数据处理失败", e);
        }
    }


    /**
     * 社保项上报
     *
     * @param reqs
     */
    public void subsidyProjects(CustomList<GeneralSalaryProjectReq> reqs) {
        String[] jsonArr = reqs.stream().map(r -> {
            SalaryExtraDTO extraDTO = transferExtraProject(r, ExtraSalaryTypeEnum.SUBSIDY);
            return JSONObject.toJSONString(extraDTO);
        }).toArray(String[]::new);
        BoundListOperations<String, String> opsList = stringRedisTemplate.boundListOps(getExtraSalaryKey());
        opsList.rightPushAll(jsonArr);
    }

    /**
     * 事务保存额外薪酬项
     *
     * @param extraDTO
     */
    public void saveExtraProjectTX(SalaryExtraDTO extraDTO) {
        TransactionStatus transactionStatus = platformTransactionManager.getTransaction(transactionDefinition);
        try {
            saveExtraProject(extraDTO);
            platformTransactionManager.commit(transactionStatus);
        } catch (Exception e) {
            platformTransactionManager.rollback(transactionStatus);
            throw e;
        }
    }

    /**
     * 保存额外薪酬项
     *
     * @param extraDTO
     */
    public void saveExtraProject(SalaryExtraDTO extraDTO) {
        //上报日期超过计算最终日期   算到下一个月
        StaffBaseInfoDTO staff = ehrRpcService.queryStaffBaseInfo(extraDTO.getUserId());
        if (PublicUtil.isEmpty(staff)) {
            log.error("保存额外薪酬项，人员信息未找到：{}", JSON.toJSONString(extraDTO));
            throw new BusinessException("人员信息未找到");
        }
        Long groupId = staff.getGroupId();
        extraDTO.setDataDate(transferDate(groupId, extraDTO.getDataDate()));
        cleanCurMonthSubsidy(extraDTO);

        Optional<SalaryPool> optional = queryPoolByUser(extraDTO.getUserId(), extraDTO.getPostId(), extraDTO.getShopId(), extraDTO.getDataDate());
        Long fundShopId;
        Long shopId;
        String userName;

        if (optional.isPresent()) {
            SalaryPool salaryPool = optional.get();
            fundShopId = PublicUtil.isNotEmpty(salaryPool.getFundShopId()) ? salaryPool.getFundShopId() : salaryPool.getShopId();
            groupId = salaryPool.getGroupId();
            shopId = salaryPool.getShopId();
            userName = salaryPool.getUserName();
        } else {
            if (PublicUtil.isEmpty(staff.getStaffShopList())) {
                log.error("保存额外薪酬项，员工门店列表为空：{}", JSON.toJSONString(extraDTO));
                throw new BusinessException("员工门店列表为空");
            }
            Optional<StaffShopInfoDTO> fundShop = staff.getStaffShopList().stream()
                    .filter(shop -> StaffShopTypeEnum.FUND_SHOP.getValue().equals(shop.getType()))
                    .findFirst();
            fundShopId = fundShop.isPresent() ? fundShop.get().getShopId() : staff.getShopId();
            groupId = staff.getGroupId();
            shopId = staff.getShopId();
            userName = staff.getName();
        }

        SalaryExtraProject extraProject = transferBaseExtra(extraDTO);
        if (Objects.isNull(extraProject)) {
            return;
        }
        extraProject.setPostId(extraDTO.getPostId());
        extraProject.setShopId(extraDTO.getShopId());
        extraProject.setGroupId(groupId);
        salaryExtraProjectService.save(extraProject);
        if (Boolean.TRUE.equals(extraDTO.getTimely())) {
            Long dealerId = Optional.ofNullable(oopRpcService.queryShop(fundShopId)).map(ShopDTO::getDealerId).orElse(null);
            PayrollRecord pr = PayrollRecord.builder()
                    .userId(extraProject.getUserId())
                    .userName(userName)
                    .shopId(shopId)
                    .fundShopId(fundShopId)
                    .dealerId(dealerId)
                    .bizId(String.valueOf(extraProject.getId()))
                    .payment(ExtraSalaryTypeEnum.AWARD.equals(extraProject.getSalaryType()) || ExtraSalaryTypeEnum.SUBSIDY.equals(extraProject.getSalaryType()))
                    .salary(Boolean.FALSE)
                    .payrollStatus(SalaryPayrollStatusEnum.WAITING)
                    .monthly(YearMonth.from(extraProject.getDataDate()))
                    .groupId(groupId)
                    .build();
            pr.setAmount(extraProject.getMoney());
            payrollRecordService.save(pr);
        }
    }

    /**
     * 清除当月补贴数据
     */
    public void cleanCurMonthSubsidy(SalaryExtraDTO extraDTO) {
        if (! ExtraSalaryTypeEnum.SUBSIDY.equals(extraDTO.getSalaryType())) {
            return;
        }
        LocalDate dataDate = DateUtil.date2LocalDate(extraDTO.getDataDate());
        LocalDate firstDay = DateUtil.date2LocalDate(extraDTO.getDataDate()).with(TemporalAdjusters.firstDayOfMonth());
        //如果是第一天 清除结束日期也是第一天， 否则是前一天
        LocalDate cleanEndDay = (firstDay.compareTo(dataDate) == 0) ? firstDay : dataDate.minusDays(1);
        salaryExtraProjectService.remove(Wrappers.<SalaryExtraProject>lambdaQuery()
                .eq(SalaryExtraProject::getUserId, extraDTO.getUserId())
                .eq(SalaryExtraProject::getSalaryType, ExtraSalaryTypeEnum.SUBSIDY)
                .ge(SalaryExtraProject::getDataDate, firstDay)
                .le(SalaryExtraProject::getDataDate, cleanEndDay)
        );
    }

    /**
     * 转换日期，超过上月计算截止日期，计算到下个月
     *
     * @param dataDate
     * @return
     */
    public Date transferDate(Long shopId, Date dataDate) {
        YearMonth monthly = DateUtil.date2YearMonth(dataDate);
        SalaryClosure salaryClosure = salarySettingCommonService.queryShopClosure(shopId, monthly);
        //门店薪酬未关单，可以保存
        if (PublicUtil.isEmpty(salaryClosure)) {
            return dataDate;
        }
        //已经关单算到今天
        return new Date();
    }

    /**
     * 查询薪酬池
     *
     * @param userId
     * @param postId
     * @param shopId
     * @param date
     * @return
     */
    private Optional<SalaryPool> queryPoolByUser(Long userId, Long postId, Long shopId, Date date) {
        SalaryGroupUser user = salaryGroupUserService.queryUserByPostShopAndUserId(userId, postId, shopId, date);
        if (PublicUtil.isEmpty(user)) {
            return Optional.empty();
        }
        SalaryPool salaryPool = salaryPoolCommonService.inspectionPool(user);
        if (PublicUtil.isEmpty(salaryPool)) {
            return Optional.empty();
        }
        return Optional.of(salaryPool);
    }

    /**
     * 缓存处理失败的数据
     *
     * @param dto
     */
    private <T> void cacheFailProject(T dto, String key) {
        BoundListOperations<String, String> listOps = stringRedisTemplate.boundListOps(key);
        String jsonString = JSONObject.toJSONString(dto);
        listOps.rightPush(jsonString);
    }

    /**
     * 对象转换
     *
     * @param dto SalaryExtraDTO
     * @return SalaryExtraProject
     */
    private SalaryExtraProject transferBaseExtra(SalaryExtraDTO dto) {
        if (Objects.isNull(dto)) {
            return null;
        }
        SalaryExtraProject salaryExtraProject = new SalaryExtraProject();
        salaryExtraProject.setUserId(dto.getUserId());
        salaryExtraProject.setMoney(dto.getMoney());
        salaryExtraProject.setSalaryType(dto.getSalaryType());
        salaryExtraProject.setDataId(dto.getDataId());
        salaryExtraProject.setDataName(dto.getDataName());
        salaryExtraProject.setDataDate(DateUtil.date2LocalDate(dto.getDataDate()));
        salaryExtraProject.setTimely(PublicUtil.isEmpty(dto.getTimely()) ? Boolean.FALSE : dto.getTimely());
        salaryExtraProject.setYn(Boolean.TRUE);
        if (PublicUtil.isNotEmpty(dto.getTypeId())) {
            dto.setTypeId(dto.getTypeId());
        }
        if (PublicUtil.isEmpty(dto.getDataId())) {
            salaryExtraProject.setDataId("");
        }
        if (PublicUtil.isEmpty(dto.getDataName())) {
            salaryExtraProject.setDataName("");
        }
        if (PublicUtil.isNotEmpty(dto.getTypeName())) {
            dto.setTypeName(dto.getTypeName());
        }
        return salaryExtraProject;
    }


    /**
     * 对象转换
     *
     * @param req SalaryRewardReq
     * @return SalaryExtraDTO
     */
    private SalaryExtraDTO transferExtraProject(SalaryRewardReq req) {
        SalaryExtraDTO dto = new SalaryExtraDTO();
        dto.setUserId(req.getUserId());
        dto.setMoney(req.getAmount());
        dto.setSalaryType(ExtraSalaryTypeEnum.AWARD);
        if (!Boolean.TRUE.equals(req.getAward())) {
            dto.setSalaryType(ExtraSalaryTypeEnum.PENALTY);
        }
        dto.setDataId(req.getDataId());
        dto.setDataName(req.getDataName());
        dto.setDataDate(req.getDataTime());
        dto.setPostId(req.getPostId());
        dto.setShopId(req.getShopId());
        dto.setTimely(req.getTimely());
        if (PublicUtil.isNotEmpty(req.getTypeId())) {
            dto.setTypeId(req.getTypeId());
        }
        if (PublicUtil.isNotEmpty(req.getTypeName())) {
            dto.setTypeName(req.getTypeName());
        }
        return dto;
    }

    /**
     * 对象转换
     *
     * @param req SalaryRewardReq
     * @return SalaryExtraDTO
     */
    private SalaryExtraDTO transferExtraProject(GeneralSalaryProjectReq req, ExtraSalaryTypeEnum salaryTypeEnum) {
        SalaryExtraDTO dto = new SalaryExtraDTO();
        dto.setUserId(req.getUserId());
        dto.setSalaryType(salaryTypeEnum);
        dto.setMoney(req.getAmount());
        dto.setDataId(req.getDataId());
        dto.setDataName(req.getDataName());
        dto.setDataDate(req.getDataTime());
        dto.setPostId(req.getPostId());
        dto.setShopId(req.getShopId());
        dto.setTimely(Boolean.FALSE);
        return dto;
    }

    /**
     * 查询薪酬组人员
     *
     * @param salaryGroupUserQuery
     * @return
     */
    public List<SalaryGroupUserResult> querySalaryGroupUser(SalaryGroupUserQuery salaryGroupUserQuery) {
        final String salaryCode = salaryGroupUserQuery.getSalaryCode();
        final Long postId = salaryGroupUserQuery.getPostId();
        final Date dataDate = new Date(salaryGroupUserQuery.getDataDate());
        log.info("查询薪酬组人员，日期：{}", new SimpleDateFormat("yyyy-MM-dd").format(dataDate));

        SalaryProject salaryProject = salaryProjectService.queryByCode(salaryCode, Boolean.FALSE);
        BV.notNull(salaryProject, () -> "指标码不存在或已失效");

        ArrayList<SalaryGroupUserResult> list = new ArrayList<>();

        List<Long> salaryGroupIds = salaryGroupService.querySalaryGroupIdsByCode(salaryCode,
                postId, salaryGroupUserQuery.getGroupId(), dataDate);

        if (CollectionUtils.isEmpty(salaryGroupIds)) {
            log.error("没有对应的薪酬组数据【{}】", salaryGroupIds);
            return list;
        }

        List<SalaryGroupUser> groupUsers = salaryGroupUserService.list(Wrappers.<SalaryGroupUser>lambdaQuery()
                .in(SalaryGroupUser::getSalaryGroupId, new HashSet<>(salaryGroupIds))
                .eq(SalaryGroupUser::getDataDate, DateUtil.date2LocalDate(dataDate))
        );

        if (CollectionUtils.isEmpty(groupUsers)) {
            log.error("没有对应日期的薪酬组人员数据【薪酬组id：{}，日期：{}】", JSON.toJSONString(salaryGroupIds), DateUtil.getStringDateShort(dataDate));
            return list;
        }

        Map<Long, List<SalaryGroupUser>> salaryGroupUserMap = groupUsers.stream().collect(Collectors.groupingBy(SalaryGroupUser::getSalaryGroupId));
        for (List<SalaryGroupUser> valueArr : salaryGroupUserMap.values()) {
            SalaryGroupUserResult result = new SalaryGroupUserResult();
            result.setRoleCodeList(salaryProject.getRoleCodes());
            result.setUserIdList(transferUser(valueArr));
            list.add(result);
        }

        return list;
    }

    /**
     * 转换器  SalaryGroupUser -> SimpleUserData
     *
     * @param groupUsers
     * @return
     */
    private List<SalaryGroupUserResult.SimpleUserData> transferUser(List<SalaryGroupUser> groupUsers) {
        List<SalaryGroupUserResult.SimpleUserData> list = new ArrayList<>();
        for (SalaryGroupUser salaryGroupUser : groupUsers) {
            SalaryGroupUserResult.SimpleUserData simpleUserData = new SalaryGroupUserResult.SimpleUserData();
            simpleUserData.setUserId(salaryGroupUser.getUserId());
            simpleUserData.setUserName(salaryGroupUser.getUserName());
            simpleUserData.setShopId(salaryGroupUser.getShopId());
            simpleUserData.setShopName(salaryGroupUser.getShopName());
            list.add(simpleUserData);
        }

        return list;
    }

    public SalaryGroupUserResult queryGroupSalaryUser(SalaryGroupUserQuery salaryGroupUserQuery) {
        final Long groupId = salaryGroupUserQuery.getGroupId();
        final Date dataDate = new Date(salaryGroupUserQuery.getDataDate());
        log.info("查询薪酬组人员，日期：{}", new SimpleDateFormat("yyyy-MM-dd").format(dataDate));
        final YearMonth monthly = YearMonth.from(DateUtil.date2LocalDateTime(dataDate));
        List<SalaryGroupUserResult.SimpleUserData> userData = new ArrayList<>();

        List<SalaryPool> pools = salaryPoolService.list(Wrappers.<SalaryPool>lambdaQuery()
                .eq(SalaryPool::getGroupId, groupId)
                .eq(SalaryPool::getMonthly, monthly)
                .eq(SalaryPool::getYn, Boolean.TRUE)
        );
        SalaryGroupUserResult result = new SalaryGroupUserResult();
        if (PublicUtil.isEmpty(pools)) {
            result.setUserIdList(new ArrayList<>());
            return result;
        }
        for (SalaryPool pool : pools) {
            SalaryGroupUserResult.SimpleUserData simpleUserData = new SalaryGroupUserResult.SimpleUserData();
            simpleUserData.setUserId(pool.getUserId());
            simpleUserData.setUserName(pool.getUserName());
            simpleUserData.setShopId(pool.getShopId());
            simpleUserData.setShopName(pool.getShopName());
            userData.add(simpleUserData);
        }
        result.setUserIdList(userData);
        return result;
    }

    /**
     * 查询薪酬系统配置
     *
     * @param param
     * @return
     */
    public List<SalaryGeneralSettinResult> queryGroupSettins(CustomList<Long> param) {
        List<Long> groupIds = param.getList();
        if (PublicUtil.isEmpty(groupIds)) {
            return new ArrayList<>();
        }

        List<SalaryGeneralSettin> salaryGeneralSettins = salaryGeneralSettinService.list(Wrappers.<SalaryGeneralSettin>lambdaQuery()
                .in(SalaryGeneralSettin::getGroupId, groupIds)
                .eq(SalaryGeneralSettin::getYn, Boolean.TRUE)
        );
        Map<Long, SalaryGeneralSettin> groupSettingMap = salaryGeneralSettins.stream().
                collect(Collectors.toMap(SalaryGeneralSettin::getGroupId, Function.identity(), (v1, v2) -> v1));
        List<SalaryGeneralSettinResult> results = new ArrayList<>();
        for (Long groupId : groupIds) {
            Optional<SalaryGeneralSettin> settin = Optional.ofNullable(groupSettingMap.get(groupId));

            Integer payoffLimit = settin.map(SalaryGeneralSettin::getPayoffLimit).orElse(0);
            Boolean leaderVisibility = settin.map(SalaryGeneralSettin::getLeaderVisibility).orElse(Boolean.FALSE);

            SalaryGeneralSettinResult result = SalaryGeneralSettinResult.builder()
                    .leaderVisibility(leaderVisibility)
                    .payoffLimit(payoffLimit)
                    .groupId(groupId)
                    .build();
            results.add(result);
        }
        return results;
    }

    /**
     * 薪酬结算
     * @param salaryClosureReq
     */
    public void salaryClosure(SalaryClosureReq salaryClosureReq) {
        this.checkClosureTime(salaryClosureReq);
        LocalDate lastMonthDay = LocalDate.now().with(TemporalAdjusters.lastDayOfMonth());
        Long groupId = salaryClosureReq.getGroupId();
        SalaryGeneralSettin setting = salaryGeneralSettinService.getOne(Wrappers.<SalaryGeneralSettin>lambdaUpdate()
                .eq(SalaryGeneralSettin::getGroupId, groupId)
        );
        Integer appealDays = Optional.ofNullable(setting).map(SalaryGeneralSettin::getAppealDays).orElse(3);
        LocalDate settlementTime = LocalDate.now();
        LocalDate appealEndTime = settlementTime.plusDays(appealDays);
        if (appealEndTime.compareTo(lastMonthDay) > 0) {
            appealEndTime = lastMonthDay;
        }
        YearMonth monthly = YearMonth.from(DateUtil.date2LocalDate(salaryClosureReq.getMonthly()));
        List<SalaryClosure> salaryClosures = convertDB(salaryClosureReq, monthly, appealEndTime);
        if (PublicUtil.isNotEmpty(salaryClosures)) {
            salaryClosureService.saveBatch(salaryClosures);
        }
    }

    public void checkClosureTime(SalaryClosureReq salaryClosureReq) {
        YearMonth curMonthly = YearMonth.now();
        YearMonth closureMonthly = DateUtil.date2YearMonth(salaryClosureReq.getMonthly());
        if (closureMonthly.compareTo(curMonthly) > 0) {
            throw new BusinessException("关单月份只能在当月之前");
        }
    }

    public List<SalaryClosure> convertDB(SalaryClosureReq salaryClosureReq, YearMonth monthly, LocalDate appealEndTime) {
        LocalDate closureTime = LocalDate.now();
        List<Long> shopIds = salaryClosureReq.getShopIds();
        Long groupId = salaryClosureReq.getGroupId();
        List<SalaryClosure> settlements = salaryClosureService.list(Wrappers.<SalaryClosure>lambdaQuery()
                .in(SalaryClosure::getMonthly, monthly)
                .in(SalaryClosure::getShopId, shopIds)
                .eq(SalaryClosure::getGroupId, groupId)
                .eq(SalaryClosure::getYn, Boolean.TRUE)
        );
        Set<Long> repetitionShopIds = settlements.stream().map(SalaryClosure::getShopId).collect(Collectors.toSet());
        List<SalaryClosure> settlementMonthlies = Lists.newArrayListWithCapacity(shopIds.size());
        for (Long shopId : salaryClosureReq.getShopIds()) {
            if (repetitionShopIds.contains(shopId)) {
                continue;
            }
            SalaryClosure salaryClosure = SalaryClosure.builder()
                    .shopId(shopId)
                    .monthly(monthly)
                    .appealEndTime(appealEndTime)
                    .closureTime(closureTime)
                    .groupId(groupId)
                    .closureKpi(Boolean.FALSE)
                    .createConfirmTodo(Boolean.FALSE)
                    .createSheet(Boolean.FALSE)
                    .yn(Boolean.TRUE)
                    .build();
            salaryClosure.setApproveEndTime(salaryClosure.getAppealEndTime().plusDays(1));
            settlementMonthlies.add(salaryClosure);

        }
        return settlementMonthlies;
    }

}
