package cn.fw.morax.service.biz.salary;

import cn.fw.morax.common.utils.MessageFormatUtil;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.salary.SalaryClosure;
import cn.fw.morax.service.data.salary.SalaryClosureService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.YearMonth;
import java.util.List;

/**
 * @author : kurisu
 * @version : 1.0
 * @className : SalarySettingCommonService
 * 1. 业务系统关单  2.计算薪酬  3. 推送薪酬确认通知  4. 绩效、薪酬申诉期  5. 薪酬审批
 * 6. 再次发送待办  7. 审批结束  8. 计算薪酬  9. 抽取报表数据 10. 推送工资支付待办
 * @description : 薪酬公用方法
 * @date : 2022-09-27 11:40
 */
@Slf4j
@Service
@AllArgsConstructor
public class SalarySettingCommonService {

    private SalaryClosureService salaryClosureService;

    /**
     * 获取没有关单绩效的门店
     *
     * @param groupId
     * @return
     */
    public List<SalaryClosure> queryNoClosureKpis(Long groupId) {
        List<SalaryClosure> settlements = salaryClosureService.list(Wrappers.<SalaryClosure>lambdaQuery()
                .eq(SalaryClosure::getMonthly, YearMonth.now().minusMonths(1))
                .eq(SalaryClosure::getGroupId, groupId)
                .eq(SalaryClosure::getClosureKpi, Boolean.FALSE)
                .eq(SalaryClosure::getYn, Boolean.TRUE)
        );
        return settlements;
    }

    /**
     * 获取没有创建确认待办的关单薪酬
     *
     * @param
     * @return
     */
    public List<SalaryClosure> queryNoCreateConfirmTodos(Long groupId, YearMonth monthly) {
        List<SalaryClosure> settlements = salaryClosureService.list(Wrappers.<SalaryClosure>lambdaQuery()
                .eq(SalaryClosure::getMonthly, monthly)
                .eq(SalaryClosure::getGroupId, groupId)
                .eq(SalaryClosure::getCreateConfirmTodo, Boolean.FALSE)
                .eq(SalaryClosure::getYn, Boolean.TRUE)
        );
        return settlements;
    }

    /**
     * 查看今天是否有关单数据
     *
     * @param groupId
     * @param monthly
     * @return
     */
    public Boolean lastDayHasClosureForCalc(Long groupId, YearMonth monthly) {
        //申述审批截止时间
        List<SalaryClosure> settlements = salaryClosureService.list(Wrappers.<SalaryClosure>lambdaQuery()
                .eq(SalaryClosure::getMonthly, monthly)
                .eq(SalaryClosure::getClosureTime, LocalDate.now().minusDays(1))
                .eq(SalaryClosure::getGroupId, groupId)
                .eq(SalaryClosure::getYn, Boolean.TRUE)
        );
        Boolean hasEndAppeal = PublicUtil.isNotEmpty(settlements);
        if (hasEndAppeal) {
            log.info("昨日有申述结束的关单薪酬，：{}", JSON.toJSONString(settlements));
        }
        return hasEndAppeal;
    }

    /**
     * 查看今天是否有关单数据
     *
     * @param groupId
     * @param monthly
     * @return
     */
    public Boolean lastDayHasClosureForReport(Long groupId, YearMonth monthly) {
        LocalDate lastDay = LocalDate.now().minusDays(1);
        //关单时间、申述审批截止时间、审批截止时间  都要更新报表数据
        List<SalaryClosure> settlements = salaryClosureService.list(Wrappers.<SalaryClosure>lambdaQuery()
                .eq(SalaryClosure::getMonthly, monthly)
                .and(w -> w.eq(SalaryClosure::getClosureTime, lastDay).or()
                        .eq(SalaryClosure::getAppealEndTime, lastDay).or()
                        .eq(SalaryClosure::getApproveEndTime, lastDay))
                .eq(SalaryClosure::getGroupId, groupId)
                .eq(SalaryClosure::getYn, Boolean.TRUE)
        );
        Boolean hasEndAppeal = PublicUtil.isNotEmpty(settlements);
        if (hasEndAppeal) {
            log.info("昨日有申述结束的关单薪酬，：{}", JSON.toJSONString(settlements));
        }
        return hasEndAppeal;
    }

    /**
     * 查询上月未生成工资单
     * @param groupId
     * @return
     */
    public List<SalaryClosure> queryNoCreateSheets(Long groupId) {
        //上月未生成工资单  且  已经过了申述审批时间
        List<SalaryClosure> settlements = salaryClosureService.list(Wrappers.<SalaryClosure>lambdaQuery()
                .eq(SalaryClosure::getMonthly, YearMonth.now().minusMonths(1))
                .ge(SalaryClosure::getApproveEndTime, LocalDateTime.now())
                .eq(SalaryClosure::getCreateSheet, Boolean.FALSE)
                .eq(SalaryClosure::getGroupId, groupId)
                .eq(SalaryClosure::getYn, Boolean.TRUE)
        );
        return settlements;
    }


    /**
     * 查询门店关单数据
     * @param shopId
     * @param monthly
     * @return
     */
    public SalaryClosure queryShopClosure(Long shopId, YearMonth monthly) {
        SalaryClosure settlement = salaryClosureService.getOne(Wrappers.<SalaryClosure>lambdaQuery()
                .eq(SalaryClosure::getMonthly, monthly)
                .eq(SalaryClosure::getShopId, shopId)
                .eq(SalaryClosure::getYn, Boolean.TRUE)
                , Boolean.FALSE
        );
        return settlement;
    }

    public void updateClosureKpi(SalaryClosure salaryClosure) {
        salaryClosure.setClosureKpi(Boolean.TRUE);
        salaryClosureService.updateById(salaryClosure);
    }

}
