package cn.fw.morax.service.component;

import cn.fw.morax.common.config.TodoVal;
import cn.fw.morax.common.pojo.event.ImportEvalIndicatorEvent;
import cn.fw.morax.common.utils.PublicUtil;
import cn.fw.morax.domain.db.eval.EvalIndicatorImport;
import cn.fw.morax.domain.db.eval.EvalIndicatorImportDetail;
import cn.fw.morax.domain.db.eval.EvalIndicatorImportRecord;
import cn.fw.morax.domain.db.eval.EvalIndicatorValue;
import cn.fw.morax.domain.enums.DataImportStatusEnum;
import cn.fw.morax.domain.enums.DimensionTypeEnum;
import cn.fw.morax.domain.vo.eval.EvalIndicatorImportUserVO;
import cn.fw.morax.domain.vo.eval.EvalReportVO;
import cn.fw.morax.domain.vo.eval.EvalShopVO;
import cn.fw.morax.rpc.backlog.TodoRpcService;
import cn.fw.morax.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.morax.service.data.eval.EvalIndicatorImportDetailService;
import cn.fw.morax.service.data.eval.EvalIndicatorImportRecordService;
import cn.fw.morax.service.data.eval.EvalIndicatorImportService;
import cn.fw.morax.service.data.eval.EvalIndicatorValueService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.common.collect.Lists;
import com.google.common.eventbus.Subscribe;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

@Component
@Slf4j
@RequiredArgsConstructor
public class EvalIndicatorSaveObserver {

    private final TodoVal todoVal;
    private final TodoRpcService todoRpcService;
    private final EvalIndicatorValueService evalIndicatorValueService;
    private final EvalIndicatorImportService evalIndicatorImportService;
    private final EvalIndicatorImportRecordService evalIndicatorImportRecordService;
    private final EvalIndicatorImportDetailService evalIndicatorImportDetailService;

    /**
     * 保存导入的考评
     *
     * @param event
     */
    @Subscribe
    @Transactional(rollbackFor = Exception.class)
    public void saveEvalIndicator(ImportEvalIndicatorEvent event) {
        //睡10秒保证数据落库
        try {
            Thread.sleep(10 * 1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        String code = event.getCode();
        LocalDate dataDate = event.getDataDate();
        Long recordId = event.getEvalIndicatorImportRecordId();
        EvalIndicatorImportRecord importRecord = evalIndicatorImportRecordService.getById(recordId);
        Long groupId = importRecord.getGroupId();
        DimensionTypeEnum dimensionType = importRecord.getDimensionType();
        List<EvalIndicatorImportDetail> details =  evalIndicatorImportDetailService.list(Wrappers.<EvalIndicatorImportDetail>lambdaQuery()
                .eq(EvalIndicatorImportDetail::getIndicatorCode, code)
                .eq(EvalIndicatorImportDetail::getRecordId, recordId)
                .isNull(EvalIndicatorImportDetail::getErrorType)
                .eq(EvalIndicatorImportDetail::getYn, Boolean.TRUE)
        );

        //转换保存数据
        LambdaQueryWrapper<EvalIndicatorValue> indicatorValueWrappers = Wrappers.<EvalIndicatorValue>lambdaQuery()
                .eq(EvalIndicatorValue::getYn, Boolean.TRUE);
        List<EvalIndicatorValue> evalIndicatorValues = Lists.newArrayListWithCapacity(details.size());
        for (EvalIndicatorImportDetail detail : details) {
            evalIndicatorValues.add(transferPo(detail, dimensionType, groupId, dataDate));
        }

        //删除当天重复数据
        if (DimensionTypeEnum.STAFF.equals(importRecord.getDimensionType())) {
            Set<Long> referIds = evalIndicatorValues.stream().map(EvalIndicatorValue::getUserId).collect(Collectors.toSet());
            indicatorValueWrappers.in(EvalIndicatorValue::getUserId, referIds);
        } else {
            Set<Long> referIds = evalIndicatorValues.stream().map(EvalIndicatorValue::getShopId).collect(Collectors.toSet());
            indicatorValueWrappers.in(EvalIndicatorValue::getShopId, referIds);
        }

        //保存数据
        if (PublicUtil.isEmpty(evalIndicatorValues)) {
            return;
        }
        evalIndicatorValueService.remove(indicatorValueWrappers
                .eq(EvalIndicatorValue::getIndicatorCode, code)
                .eq(EvalIndicatorValue::getDataDate, dataDate)
                .eq(EvalIndicatorValue::getDimensionType, importRecord.getDimensionType())
        );
        evalIndicatorValueService.saveBatch(evalIndicatorValues);
        this.checkDataReport(event.getCode(), event.getDataDate(), event.getGroupId());
    }

    /**
     * 考评数据录入详情
     *
     * @param
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    public void checkDataReport(String code, LocalDate dataDate, Long groupId) {
        List<EvalIndicatorImport> indicatorImports = evalIndicatorImportService.list(Wrappers.<EvalIndicatorImport>lambdaQuery()
                .eq(EvalIndicatorImport::getStatus, DataImportStatusEnum.IMPORTING)
                .eq(EvalIndicatorImport::getIndicatorCode, code)
                .eq(EvalIndicatorImport::getGroupId, groupId)
                .eq(EvalIndicatorImport::getDataDate, dataDate)
                .eq(EvalIndicatorImport::getYn, Boolean.TRUE)
        );
        if (PublicUtil.isEmpty(indicatorImports)) {
            return;
        }

        List<EvalIndicatorImport> completeImports = new ArrayList<>();
        for (EvalIndicatorImport evalIndicatorImport : indicatorImports) {
            EvalReportVO evalReportVO = new EvalReportVO();
            try {
                evalReportVO = JSON.parseObject(evalIndicatorImport.getContent(), EvalReportVO.class);
            } catch (Exception e) {
                log.error("[{}]导入数据解析失败", JSON.toJSONString(evalIndicatorImport), e);
            }

            List<EvalIndicatorImportUserVO> users = Optional.ofNullable(evalReportVO.getUsers()).orElse(new ArrayList<>());
            List<EvalShopVO> shops = Optional.ofNullable(evalReportVO.getShops()).orElse(new ArrayList<>());
            if (PublicUtil.isNotEmpty(users)) {
                List<Long> userIds = users.stream().map(EvalIndicatorImportUserVO::getUserId).collect(Collectors.toList());
                List<EvalIndicatorValue> evalIndicatorValues = evalIndicatorValueService.list(Wrappers.<EvalIndicatorValue>lambdaQuery()
                        .in(EvalIndicatorValue::getUserId, userIds)
                        .eq(EvalIndicatorValue::getIndicatorCode, code)
                        .eq(EvalIndicatorValue::getDataDate, evalIndicatorImport.getDataDate())
                        .eq(EvalIndicatorValue::getDimensionType, DimensionTypeEnum.STAFF)
                        .eq(EvalIndicatorValue::getYn, Boolean.TRUE)
                );
                Set<Long> reportDataUserIds = evalIndicatorValues.stream().map(EvalIndicatorValue::getUserId).collect(Collectors.toSet());
                userIds.removeAll(reportDataUserIds);
                if (PublicUtil.isNotEmpty(userIds)) {
                    continue;
                }
            }

            if (PublicUtil.isNotEmpty(shops)) {
                List<Long> shopIds = shops.stream().map(EvalShopVO::getShopId).collect(Collectors.toList());
                List<EvalIndicatorValue> evalIndicatorValues = evalIndicatorValueService.list(Wrappers.<EvalIndicatorValue>lambdaQuery()
                        .in(EvalIndicatorValue::getShopId, shopIds)
                        .eq(EvalIndicatorValue::getIndicatorCode, code)
                        .eq(EvalIndicatorValue::getDataDate, evalIndicatorImport.getDataDate())
                        .eq(EvalIndicatorValue::getDimensionType, DimensionTypeEnum.SHOP)
                        .eq(EvalIndicatorValue::getYn, Boolean.TRUE)
                );
                Set<Long> reportDataShopIds = evalIndicatorValues.stream().map(EvalIndicatorValue::getShopId).collect(Collectors.toSet());
                shopIds.removeAll(reportDataShopIds);
                if (PublicUtil.isNotEmpty(shopIds)) {
                    continue;
                }
            }
            //完成
            completeImports.add(evalIndicatorImport);
        }

        if (PublicUtil.isEmpty(completeImports)) {
            return;
        }

        List<Long> completeImportIds = new ArrayList<>();
        for (EvalIndicatorImport evalIndicatorImport : completeImports) {
            BackLogItemDTO dto = new BackLogItemDTO(evalIndicatorImport.getImportUserId(), todoVal.getEvalIndicatorImport(),
                    String.valueOf(evalIndicatorImport.getId()), new Date(), null);
            if (todoRpcService.cancel(dto)) {
                completeImportIds.add(evalIndicatorImport.getId());
            } else {
                log.error("考评录入待办取消失败：{}", JSON.toJSONString(evalIndicatorImport));
            }
        }

        if (PublicUtil.isNotEmpty(completeImportIds)) {
            evalIndicatorImportService.update(Wrappers.<EvalIndicatorImport>lambdaUpdate()
                    .set(EvalIndicatorImport::getStatus, DataImportStatusEnum.COMPLETION)
                    .in(EvalIndicatorImport::getId, completeImportIds)
                    .eq(EvalIndicatorImport::getYn, Boolean.TRUE)
            );
        }
    }

    public EvalIndicatorValue transferPo(EvalIndicatorImportDetail detail,
                                         DimensionTypeEnum dimensionType,
                                         Long groupId, LocalDate dataDate) {
        EvalIndicatorValue indicatorValue = new EvalIndicatorValue();
        if (PublicUtil.isNotEmpty(detail.getUserId())) {
            indicatorValue.setUserId(detail.getUserId());
        }
        if (PublicUtil.isNotEmpty(detail.getUserName())) {
            indicatorValue.setUserName(detail.getUserName());
        }
        indicatorValue.setShopId(detail.getShopId());
        indicatorValue.setShopName(detail.getShopName());
        indicatorValue.setDimensionType(dimensionType);
        indicatorValue.setIndicatorCode(detail.getIndicatorCode());
        indicatorValue.setIndicatorValue(detail.getIndicatorValue());
        indicatorValue.setDataType(detail.getDataType());
        indicatorValue.setDataDate(dataDate);
        indicatorValue.setGroupId(groupId);
        indicatorValue.setYn(Boolean.TRUE);
        return indicatorValue;
    }

}